@@ -615,6 +615,16 @@ func TestMergeInput(t *testing.T) {
 				require.Equal(t, "bar", c.Agent.TrustDomain)
 			},
 		},
+		{
+			msg: "require_pq_kem should be configurable by file",
+			fileInput: func(c *Config) {
+				c.Agent.Experimental.RequirePQKEM = true
+			},
+			cliInput: func(c *agentConfig) {},
+			test: func(t *testing.T, c *Config) {
+				require.True(t, c.Agent.Experimental.RequirePQKEM)
+			},
+		},
 	}
 	cases = append(cases, mergeInputCasesOS()...)
 
@@ -688,6 +698,24 @@ func TestNewAgentConfig(t *testing.T) {
 				require.True(t, c.InsecureBootstrap)
 			},
 		},
+		{
+			msg: "retry_bootstrap should be correctly set to false",
+			input: func(c *Config) {
+				c.Agent.RetryBootstrap = false
+			},
+			test: func(t *testing.T, c *agent.Config) {
+				require.False(t, c.RetryBootstrap)
+			},
+		},
+		{
+			msg: "retry_bootstrap should be correctly set to true",
+			input: func(c *Config) {
+				c.Agent.RetryBootstrap = true
+			},
+			test: func(t *testing.T, c *agent.Config) {
+				require.True(t, c.RetryBootstrap)
+			},
+		},
 		{
 			msg: "join_token should be correctly configured",
 			input: func(c *Config) {
@@ -882,7 +910,7 @@ func TestNewAgentConfig(t *testing.T) {
 		{
 			msg: "x509_svid_cache_max_size is set",
 			input: func(c *Config) {
-				c.Agent.Experimental.X509SVIDCacheMaxSize = 100
+				c.Agent.X509SVIDCacheMaxSize = 100
 			},
 			test: func(t *testing.T, c *agent.Config) {
 				require.EqualValues(t, 100, c.X509SVIDCacheMaxSize)
@@ -899,7 +927,7 @@ func TestNewAgentConfig(t *testing.T) {
 		{
 			msg: "x509_svid_cache_max_size is zero",
 			input: func(c *Config) {
-				c.Agent.Experimental.X509SVIDCacheMaxSize = 0
+				c.Agent.X509SVIDCacheMaxSize = 0
 			},
 			test: func(t *testing.T, c *agent.Config) {
 				require.EqualValues(t, 0, c.X509SVIDCacheMaxSize)
@@ -909,7 +937,7 @@ func TestNewAgentConfig(t *testing.T) {
 			msg:         "x509_svid_cache_max_size is negative",
 			expectError: true,
 			input: func(c *Config) {
-				c.Agent.Experimental.X509SVIDCacheMaxSize = -10
+				c.Agent.X509SVIDCacheMaxSize = -10
 			},
 			test: func(t *testing.T, c *agent.Config) {
 				require.Nil(t, c)
@@ -994,8 +1022,25 @@ func TestNewAgentConfig(t *testing.T) {
 				require.Nil(t, c)
 			},
 		},
+
+		{
+			msg:   "require PQ KEM is disabled (default)",
+			input: func(c *Config) {},
+			test: func(t *testing.T, c *agent.Config) {
+				require.Equal(t, false, c.TLSPolicy.RequirePQKEM)
+			},
+		},
+		{
+			msg: "require PQ KEM is enabled",
+			input: func(c *Config) {
+				c.Agent.Experimental.RequirePQKEM = true
+			},
+			test: func(t *testing.T, c *agent.Config) {
+				require.Equal(t, true, c.TLSPolicy.RequirePQKEM)
+			},
+		},
 	}
-	cases = append(cases, newAgentConfigCasesOS()...)
+	cases = append(cases, newAgentConfigCasesOS(t)...)
 	for _, testCase := range cases {
 		testCase := testCase
 
