@@ -14,6 +14,7 @@ import (
 	"path/filepath"
 	"reflect"
 	"sort"
+	"strconv"
 	"strings"
 	"syscall"
 	"time"
@@ -30,10 +31,13 @@ import (
 	"github.com/spiffe/spire/pkg/common/bundleutil"
 	"github.com/spiffe/spire/pkg/common/catalog"
 	common_cli "github.com/spiffe/spire/pkg/common/cli"
+	"github.com/spiffe/spire/pkg/common/config"
+	"github.com/spiffe/spire/pkg/common/diskcertmanager"
 	"github.com/spiffe/spire/pkg/common/fflag"
 	"github.com/spiffe/spire/pkg/common/health"
 	"github.com/spiffe/spire/pkg/common/log"
 	"github.com/spiffe/spire/pkg/common/telemetry"
+	"github.com/spiffe/spire/pkg/common/tlspolicy"
 	"github.com/spiffe/spire/pkg/server"
 	"github.com/spiffe/spire/pkg/server/authpolicy"
 	bundleClient "github.com/spiffe/spire/pkg/server/bundle/client"
@@ -84,8 +88,6 @@ type serverConfig struct {
 	RateLimit          rateLimitConfig    `hcl:"ratelimit"`
 	SocketPath         string             `hcl:"socket_path"`
 	TrustDomain        string             `hcl:"trust_domain"`
-	// Temporary flag to allow disabling the inclusion of serial number in X509 CAs Subject field
-	ExcludeSNFromCASubject bool `hcl:"exclude_sn_from_ca_subject"`
 
 	ConfigPath string
 	ExpandEnv  bool
@@ -96,14 +98,20 @@ type serverConfig struct {
 	ProfilingFreq    int      `hcl:"profiling_freq"`
 	ProfilingNames   []string `hcl:"profiling_names"`
 
+	// Temporary configurables
+	// UseLegacyDownstreamX509CATTL is deprecated and should be removed in SPIRE 1.12.0.
+	UseLegacyDownstreamX509CATTL *bool `hcl:"use_legacy_downstream_x509_ca_ttl"`
+
 	UnusedKeyPositions map[string][]token.Pos `hcl:",unusedKeyPositions"`
 }
 
 type experimentalConfig struct {
-	AuthOpaPolicyEngine  *authpolicy.OpaEngineConfig `hcl:"auth_opa_policy_engine"`
-	CacheReloadInterval  string                      `hcl:"cache_reload_interval"`
-	EventsBasedCache     bool                        `hcl:"events_based_cache"`
-	PruneEventsOlderThan string                      `hcl:"prune_events_older_than"`
+	AuthOpaPolicyEngine   *authpolicy.OpaEngineConfig `hcl:"auth_opa_policy_engine"`
+	CacheReloadInterval   string                      `hcl:"cache_reload_interval"`
+	EventsBasedCache      bool                        `hcl:"events_based_cache"`
+	PruneEventsOlderThan  string                      `hcl:"prune_events_older_than"`
+	SQLTransactionTimeout string                      `hcl:"sql_transaction_timeout"`
+	RequirePQKEM          bool                        `hcl:"require_pq_kem"`
 
 	Flags fflag.RawConfig `hcl:"feature_flags"`
 
@@ -143,11 +151,19 @@ type bundleEndpointConfigProfile struct {
 }
 
 type bundleEndpointProfileHTTPSWebConfig struct {
-	ACME *bundleEndpointACMEConfig `hcl:"acme"`
+	ACME            *bundleEndpointACMEConfig      `hcl:"acme"`
+	ServingCertFile *bundleEndpointServingCertFile `hcl:"serving_cert_file"`
 }
 
 type bundleEndpointProfileHTTPSSPIFFEConfig struct{}
 
+type bundleEndpointServingCertFile struct {
+	CertFilePath        string        `hcl:"cert_file_path"`
+	KeyFilePath         string        `hcl:"key_file_path"`
+	FileSyncInterval    time.Duration `hcl:"-"`
+	RawFileSyncInterval string        `hcl:"file_sync_interval"`
+}
+
 type bundleEndpointACMEConfig struct {
 	DirectoryURL       string                 `hcl:"directory_url"`
 	DomainName         string                 `hcl:"domain_name"`
@@ -304,7 +320,7 @@ func ParseFile(path string, expandEnv bool) (*Config, error) {
 
 	// If envTemplate flag is passed, substitute $VARIABLES in configuration file
 	if expandEnv {
-		data = os.ExpandEnv(data)
+		data = config.ExpandEnv(data)
 	}
 
 	if err := hcl.Decode(&c, data); err != nil {
@@ -377,7 +393,8 @@ func NewServerConfig(c *Config, logOptions []log.Option, allowUnknownConfig bool
 	}
 	var reopenableFile *log.ReopenableFile
 	if c.Server.LogFile != "" {
-		reopenableFile, err := log.NewReopenableFile(c.Server.LogFile)
+		var err error
+		reopenableFile, err = log.NewReopenableFile(c.Server.LogFile)
 		if err != nil {
 			return nil, err
 		}
@@ -389,11 +406,12 @@ func NewServerConfig(c *Config, logOptions []log.Option, allowUnknownConfig bool
 		return nil, fmt.Errorf("could not start logger: %w", err)
 	}
 	sc.Log = logger
+
 	if reopenableFile != nil {
 		sc.LogReopener = log.ReopenOnSignal(logger, reopenableFile)
 	}
 
-	bindAddress, err := net.ResolveTCPAddr("tcp", fmt.Sprintf("%s:%d", c.Server.BindAddress, c.Server.BindPort))
+	bindAddress, err := net.ResolveTCPAddr("tcp", net.JoinHostPort(strings.Trim(c.Server.BindAddress, "[]"), strconv.Itoa(c.Server.BindPort)))
 	if err != nil {
 		return nil, fmt.Errorf(`could not resolve bind address "%s:%d": %w`, c.Server.BindAddress, c.Server.BindPort, err)
 	}
@@ -452,19 +470,17 @@ func NewServerConfig(c *Config, logOptions []log.Option, allowUnknownConfig bool
 						"refresh more often than 1 minute")
 				}
 
-				sc.Federation.BundleEndpoint.RefreshHint = &refreshHint
+				sc.Federation.BundleEndpoint.RefreshHint = refreshHint
 			} else {
-				sc.Log.Warn("Bundle endpoint refresh_hint is not set. This configuration " +
-					"will default to 5 minutes in a future release; please check if you " +
-					"need to specify it")
+				refreshHint := 5 * time.Minute
+				sc.Federation.BundleEndpoint.RefreshHint = refreshHint
 			}
 
 			if c.Server.Federation.BundleEndpoint != nil {
-				acmeConfig, err := parseBundleEndpointConfigProfile(c.Server.Federation.BundleEndpoint, sc.DataDir, sc.Log)
+				err := setBundleEndpointConfigProfile(c.Server.Federation.BundleEndpoint, sc.DataDir, sc.Log, &sc.Federation)
 				if err != nil {
 					return nil, err
 				}
-				sc.Federation.BundleEndpoint.ACME = acmeConfig
 			}
 		}
 
@@ -496,6 +512,12 @@ func NewServerConfig(c *Config, logOptions []log.Option, allowUnknownConfig bool
 	sc.ProfilingFreq = c.Server.ProfilingFreq
 	sc.ProfilingNames = c.Server.ProfilingNames
 
+	sc.TLSPolicy = tlspolicy.Policy{
+		RequirePQKEM: c.Server.Experimental.RequirePQKEM,
+	}
+
+	tlspolicy.LogPolicy(sc.TLSPolicy, log.NewHCLogAdapter(logger, "tlspolicy"))
+
 	for _, adminID := range c.Server.AdminIDs {
 		id, err := spiffeid.FromString(adminID)
 		if err != nil {
@@ -545,6 +567,20 @@ func NewServerConfig(c *Config, logOptions []log.Option, allowUnknownConfig bool
 		sc.CATTL = ttl
 	}
 
+	if c.Server.UseLegacyDownstreamX509CATTL != nil {
+		sc.UseLegacyDownstreamX509CATTL = *c.Server.UseLegacyDownstreamX509CATTL
+		if sc.UseLegacyDownstreamX509CATTL {
+			sc.Log.Warn("Using legacy downstream X509 CA TTL calculation; this option will be removed in a future release")
+		} else {
+			sc.Log.Info("Using preferred downstream X509 CA TTL calculation")
+		}
+	} else {
+		// The default value should be false in SPIRE 1.11.0 and the flag
+		// removed in SPIRE 1.12.0.
+		sc.UseLegacyDownstreamX509CATTL = true
+		sc.Log.Info("Using legacy downstream X509 CA TTL calculation by default; this default will change in a future release")
+	}
+
 	// If the configured TTLs can lead to surprises, then do our best to log an
 	// accurate message and guide the user to resolution
 	ttlChecks := []struct {
@@ -637,12 +673,6 @@ func NewServerConfig(c *Config, logOptions []log.Option, allowUnknownConfig bool
 		sc.CASubject = credtemplate.DefaultX509CASubject()
 	}
 
-	sc.ExcludeSNFromCASubject = c.Server.ExcludeSNFromCASubject
-	// TODO: remove exclude_sn_from_ca_subject in SPIRE v1.10.0
-	if sc.ExcludeSNFromCASubject {
-		sc.Log.Warn("The deprecated exclude_sn_from_ca_subject configurable will be removed in a future release")
-	}
-
 	sc.PluginConfigs, err = catalog.PluginConfigsFromHCLNode(c.Plugins)
 	if err != nil {
 		return nil, err
@@ -676,6 +706,14 @@ func NewServerConfig(c *Config, logOptions []log.Option, allowUnknownConfig bool
 		sc.PruneEventsOlderThan = interval
 	}
 
+	if c.Server.Experimental.SQLTransactionTimeout != "" {
+		interval, err := time.ParseDuration(c.Server.Experimental.SQLTransactionTimeout)
+		if err != nil {
+			return nil, fmt.Errorf("could not parse SQL transaction timeout interval: %w", err)
+		}
+		sc.SQLTransactionTimeout = interval
+	}
+
 	if c.Server.Experimental.EventsBasedCache {
 		sc.Log.Info("Using events based cache")
 	}
@@ -690,46 +728,54 @@ func NewServerConfig(c *Config, logOptions []log.Option, allowUnknownConfig bool
 	return sc, nil
 }
 
-func parseBundleEndpointConfigProfile(config *bundleEndpointConfig, dataDir string, log logrus.FieldLogger) (*bundle.ACMEConfig, error) {
+func setBundleEndpointConfigProfile(config *bundleEndpointConfig, dataDir string, log logrus.FieldLogger, federationConfig *server.FederationConfig) error {
 	switch {
 	case config.ACME != nil && config.Profile != nil:
-		return nil, errors.New("either bundle endpoint 'acme' or 'profile' can be set, but not both")
+		return errors.New("either bundle endpoint 'acme' or 'profile' can be set, but not both")
 
 	case config.ACME != nil:
 		log.Warn("ACME configuration within the bundle_endpoint is deprecated. Please use ACME configuration as part of the https_web profile instead.")
-		return configToACMEConfig(config.ACME, dataDir), nil
+		federationConfig.BundleEndpoint.ACME = configToACMEConfig(config.ACME, dataDir)
+		return nil
 
 	case config.Profile == nil:
 		log.Warn("Bundle endpoint is configured but has no profile set, using https_spiffe as default; please configure a profile explicitly. This will be fatal in a future release.")
-		return nil, nil
+		return nil
 	}
 
 	// Profile is set, parse it
 	configString, err := parseBundleEndpointProfileASTNode(config.Profile)
 	if err != nil {
-		return nil, err
+		return err
 	}
 
 	profileConfig := new(bundleEndpointConfigProfile)
 	if err := hcl.Decode(profileConfig, configString); err != nil {
-		return nil, fmt.Errorf("failed to decode configuration: %w", err)
+		return fmt.Errorf("failed to decode configuration: %w", err)
 	}
 
 	switch {
 	case profileConfig.HTTPSWeb != nil:
-		acme := profileConfig.HTTPSWeb.ACME
-		if acme == nil {
-			return nil, fmt.Errorf("malformed https_web profile configuration: ACME is required")
+		switch {
+		case profileConfig.HTTPSWeb.ACME != nil:
+			federationConfig.BundleEndpoint.ACME = configToACMEConfig(profileConfig.HTTPSWeb.ACME, dataDir)
+			return nil
+		case profileConfig.HTTPSWeb.ServingCertFile != nil:
+			federationConfig.BundleEndpoint.DiskCertManager, err = configToDiskCertManager(profileConfig.HTTPSWeb.ServingCertFile, log)
+			if err != nil {
+				return err
+			}
+			return nil
+		default:
+			return errors.New("malformed https_web profile configuration: 'acme' or 'serving_cert_file' is required")
 		}
-		acmeConfig := configToACMEConfig(acme, dataDir)
-		return acmeConfig, nil
 
 	// For now ignore SPIFFE configuration
 	case profileConfig.HTTPSSPIFFE != nil:
-		return nil, nil
+		return nil
 
 	default:
-		return nil, errors.New(`unknown bundle endpoint profile configured; current supported profiles are "https_spiffe" and 'https_web"`)
+		return errors.New(`unknown bundle endpoint profile configured; current supported profiles are "https_spiffe" and 'https_web"`)
 	}
 }
 
@@ -743,6 +789,28 @@ func configToACMEConfig(acme *bundleEndpointACMEConfig, dataDir string) *bundle.
 	}
 }
 
+func configToDiskCertManager(serviceCertFile *bundleEndpointServingCertFile, log logrus.FieldLogger) (*diskcertmanager.DiskCertManager, error) {
+	fileSyncInterval, err := time.ParseDuration(serviceCertFile.RawFileSyncInterval)
+	if err != nil {
+		return nil, err
+	}
+
+	serviceCertFile.FileSyncInterval = fileSyncInterval
+	if serviceCertFile.FileSyncInterval == time.Duration(0) {
+		serviceCertFile.FileSyncInterval = time.Hour
+	}
+
+	return diskcertmanager.New(
+		&diskcertmanager.Config{
+			CertFilePath:     serviceCertFile.CertFilePath,
+			KeyFilePath:      serviceCertFile.KeyFilePath,
+			FileSyncInterval: serviceCertFile.FileSyncInterval,
+		},
+		nil,
+		log,
+	)
+}
+
 func parseBundleEndpointProfile(config federatesWithConfig) (trustDomainConfig *bundleClient.TrustDomainConfig, err error) {
 	configString, err := parseBundleEndpointProfileASTNode(config.BundleEndpointProfile)
 	if err != nil {
