@@ -64,6 +64,7 @@ func TestParseConfigGood(t *testing.T) {
 	_, ok := trustDomainConfig.EndpointProfile.(bundleClient.HTTPSWebProfile)
 	assert.True(t, ok)
 	assert.True(t, c.Server.AuditLogEnabled)
+	assert.True(t, c.Server.Experimental.RequirePQKEM)
 	testParseConfigGoodOS(t, c)
 
 	// Parse/reprint cycle trims outer whitespace
@@ -72,28 +73,28 @@ func TestParseConfigGood(t *testing.T) {
 	// Check for plugins configurations
 	expectedPluginConfigs := catalog.PluginConfigs{
 		{
-			Type:     "plugin_type_server",
-			Name:     "plugin_name_server",
-			Path:     "./pluginServerCmd",
-			Checksum: "pluginServerChecksum",
-			Data:     data,
-			Disabled: false,
+			Type:       "plugin_type_server",
+			Name:       "plugin_name_server",
+			Path:       "./pluginServerCmd",
+			Checksum:   "pluginServerChecksum",
+			DataSource: catalog.FixedData(data),
+			Disabled:   false,
 		},
 		{
-			Type:     "plugin_type_server",
-			Name:     "plugin_disabled",
-			Path:     "./pluginServerCmd",
-			Checksum: "pluginServerChecksum",
-			Data:     data,
-			Disabled: true,
+			Type:       "plugin_type_server",
+			Name:       "plugin_disabled",
+			Path:       "./pluginServerCmd",
+			Checksum:   "pluginServerChecksum",
+			DataSource: catalog.FixedData(data),
+			Disabled:   true,
 		},
 		{
-			Type:     "plugin_type_server",
-			Name:     "plugin_enabled",
-			Path:     "./pluginServerCmd",
-			Checksum: "pluginServerChecksum",
-			Data:     data,
-			Disabled: false,
+			Type:       "plugin_type_server",
+			Name:       "plugin_enabled",
+			Path:       "./pluginServerCmd",
+			Checksum:   "pluginServerChecksum",
+			DataSource: catalog.FileData("plugin.conf"),
+			Disabled:   false,
 		},
 	}
 
@@ -455,6 +456,16 @@ func TestMergeInput(t *testing.T) {
 				require.True(t, c.Server.AuditLogEnabled)
 			},
 		},
+		{
+			msg: "require_pq_kem should be configurable by file",
+			fileInput: func(c *Config) {
+				c.Server.Experimental.RequirePQKEM = true
+			},
+			cliFlags: []string{},
+			test: func(t *testing.T, c *Config) {
+				require.True(t, c.Server.Experimental.RequirePQKEM)
+			},
+		},
 	}
 	cases = append(cases, mergeInputCasesOS(t)...)
 
@@ -504,6 +515,28 @@ func TestNewServerConfig(t *testing.T) {
 				require.Equal(t, 1337, c.BindAddress.Port)
 			},
 		},
+		{
+			msg: "IPv6 bind_address in square brackets and bind_port should be correctly parsed",
+			input: func(c *Config) {
+				c.Server.BindAddress = "[2001:101::]"
+				c.Server.BindPort = 1337
+			},
+			test: func(t *testing.T, c *server.Config) {
+				require.Equal(t, "2001:101::", c.BindAddress.IP.String())
+				require.Equal(t, 1337, c.BindAddress.Port)
+			},
+		},
+		{
+			msg: "IPv6 bind_address without square brackets and bind_port should be correctly parsed",
+			input: func(c *Config) {
+				c.Server.BindAddress = "2001:101::"
+				c.Server.BindPort = 1337
+			},
+			test: func(t *testing.T, c *server.Config) {
+				require.Equal(t, "2001:101::", c.BindAddress.IP.String())
+				require.Equal(t, 1337, c.BindAddress.Port)
+			},
+		},
 		{
 			msg: "bind_address with hostname value should be correctly parsed",
 			input: func(c *Config) {
@@ -634,8 +667,7 @@ func TestNewServerConfig(t *testing.T) {
 			test: func(t *testing.T, c *server.Config) {
 				require.Equal(t, "192.168.1.1", c.Federation.BundleEndpoint.Address.IP.String())
 				require.Equal(t, 1337, c.Federation.BundleEndpoint.Address.Port)
-				require.NotNil(t, c.Federation.BundleEndpoint.RefreshHint)
-				require.Equal(t, 10*time.Minute, *c.Federation.BundleEndpoint.RefreshHint)
+				require.Equal(t, 10*time.Minute, c.Federation.BundleEndpoint.RefreshHint)
 			},
 		},
 		{
@@ -673,8 +705,7 @@ func TestNewServerConfig(t *testing.T) {
 			test: func(t *testing.T, c *server.Config) {
 				require.Equal(t, "0.0.0.0", c.Federation.BundleEndpoint.Address.IP.String())
 				require.Equal(t, 8443, c.Federation.BundleEndpoint.Address.Port)
-				require.NotNil(t, c.Federation.BundleEndpoint.RefreshHint)
-				require.Equal(t, 10*time.Minute, *c.Federation.BundleEndpoint.RefreshHint)
+				require.Equal(t, 10*time.Minute, c.Federation.BundleEndpoint.RefreshHint)
 				require.Nil(t, c.Federation.BundleEndpoint.ACME)
 			},
 		},
@@ -682,14 +713,13 @@ func TestNewServerConfig(t *testing.T) {
 			msg: "bundle endpoint has web profile",
 			input: func(c *Config) {
 				c.Server.Federation = &federationConfig{
-					BundleEndpoint: bundleEndpointProfileHTTPSWebTest(t),
+					BundleEndpoint: bundleEndpointProfileHTTPSWebACMETest(t),
 				}
 			},
 			test: func(t *testing.T, c *server.Config) {
 				require.Equal(t, "0.0.0.0", c.Federation.BundleEndpoint.Address.IP.String())
 				require.Equal(t, 8443, c.Federation.BundleEndpoint.Address.Port)
-				require.NotNil(t, c.Federation.BundleEndpoint.RefreshHint)
-				require.Equal(t, 10*time.Minute, *c.Federation.BundleEndpoint.RefreshHint)
+				require.Equal(t, 10*time.Minute, c.Federation.BundleEndpoint.RefreshHint)
 
 				expectACME := &bundle.ACMEConfig{
 					DomainName: "example.org",
@@ -697,6 +727,23 @@ func TestNewServerConfig(t *testing.T) {
 					CacheDir:   "bundle-acme",
 				}
 				require.Equal(t, expectACME, c.Federation.BundleEndpoint.ACME)
+				require.Nil(t, c.Federation.BundleEndpoint.DiskCertManager)
+			},
+		},
+		{
+			msg: "bundle endpoint has web profile with certs on disk",
+			input: func(c *Config) {
+				c.Server.Federation = &federationConfig{
+					BundleEndpoint: bundleEndpointProfileHTTPSWebServingCertFileTest(t),
+				}
+			},
+			test: func(t *testing.T, c *server.Config) {
+				require.Equal(t, "0.0.0.0", c.Federation.BundleEndpoint.Address.IP.String())
+				require.Equal(t, 8443, c.Federation.BundleEndpoint.Address.Port)
+				require.Equal(t, 10*time.Minute, c.Federation.BundleEndpoint.RefreshHint)
+
+				require.Nil(t, c.Federation.BundleEndpoint.ACME)
+				require.NotNil(t, c.Federation.BundleEndpoint.DiskCertManager)
 			},
 		},
 		{
@@ -748,7 +795,7 @@ func TestNewServerConfig(t *testing.T) {
 			test: func(t *testing.T, c *server.Config) {
 				require.Equal(t, "192.168.1.1", c.Federation.BundleEndpoint.Address.IP.String())
 				require.Equal(t, 1337, c.Federation.BundleEndpoint.Address.Port)
-				require.Nil(t, c.Federation.BundleEndpoint.RefreshHint)
+				require.Equal(t, 5*time.Minute, c.Federation.BundleEndpoint.RefreshHint)
 			},
 		},
 		{
@@ -1146,8 +1193,24 @@ func TestNewServerConfig(t *testing.T) {
 				}, c.AdminIDs)
 			},
 		},
+		{
+			msg:   "require PQ KEM is disabled (default)",
+			input: func(c *Config) {},
+			test: func(t *testing.T, c *server.Config) {
+				require.Equal(t, false, c.TLSPolicy.RequirePQKEM)
+			},
+		},
+		{
+			msg: "require PQ KEM is enabled",
+			input: func(c *Config) {
+				c.Server.Experimental.RequirePQKEM = true
+			},
+			test: func(t *testing.T, c *server.Config) {
+				require.Equal(t, true, c.TLSPolicy.RequirePQKEM)
+			},
+		},
 	}
-	cases = append(cases, newServerConfigCasesOS()...)
+	cases = append(cases, newServerConfigCasesOS(t)...)
 
 	for _, testCase := range cases {
 		testCase := testCase
@@ -1529,10 +1592,10 @@ func TestLogOptions(t *testing.T) {
 		log.WithReopenableOutputFile(logFile),
 	}
 
-	agentConfig, err := NewServerConfig(defaultValidConfig(), logOptions, false)
+	serverConfig, err := NewServerConfig(defaultValidConfig(), logOptions, false)
 	require.NoError(t, err)
 
-	logger := agentConfig.Log.(*log.Logger).Logger
+	logger := serverConfig.Log.(*log.Logger).Logger
 
 	// defaultConfig() sets level to info,  which should override DEBUG set above
 	require.Equal(t, logrus.InfoLevel, logger.Level)
@@ -1679,7 +1742,7 @@ func TestHasCompatibleTTLs(t *testing.T) {
 			msg:                      "default_jwt_svid_ttl is small enough for the configured CA TTL but larger than the max",
 			caTTL:                    time.Hour * 24 * 7 * 4 * 6, // Six months
 			x509SvidTTL:              0,
-			jwtSvidTTL:               time.Hour * 24 * 7 * 2, // Two weeks,,
+			jwtSvidTTL:               time.Hour * 24 * 7 * 2, // Two weeks
 			hasCompatibleSvidTTL:     true,
 			hasCompatibleX509SvidTTL: true,
 			hasCompatibleJwtSvidTTL:  false,
@@ -1902,7 +1965,24 @@ func bundleEndpointProfileACMEAndProfileTest(t *testing.T) *bundleEndpointConfig
 	return config
 }
 
-func bundleEndpointProfileHTTPSWebTest(t *testing.T) *bundleEndpointConfig {
+func bundleEndpointProfileHTTPSWebServingCertFileTest(t *testing.T) *bundleEndpointConfig {
+	configString := `address = "0.0.0.0"
+        port = 8443
+        refresh_hint = "10m"
+        profile "https_web" {
+        	serving_cert_file {
+        		cert_file_path = "../../../../test/fixture/certs/svid.pem"
+        		key_file_path = "../../../../test/fixture/certs/svid_key.pem"
+        		file_sync_interval = "5m"
+        	}
+        }`
+	config := new(bundleEndpointConfig)
+	require.NoError(t, hcl.Decode(config, configString))
+
+	return config
+}
+
+func bundleEndpointProfileHTTPSWebACMETest(t *testing.T) *bundleEndpointConfig {
 	configString := `address = "0.0.0.0"
         port = 8443
         refresh_hint = "10m"
