@@ -10,6 +10,8 @@ import (
 	"sync"
 	"time"
 
+	"github.com/andres-erbsen/clock"
+	"github.com/sirupsen/logrus"
 	"github.com/spiffe/go-spiffe/v2/workloadapi"
 	admin_api "github.com/spiffe/spire/pkg/agent/api"
 	node_attestor "github.com/spiffe/spire/pkg/agent/attestor/node"
@@ -21,19 +23,27 @@ import (
 	"github.com/spiffe/spire/pkg/agent/plugin/nodeattestor"
 	"github.com/spiffe/spire/pkg/agent/storage"
 	"github.com/spiffe/spire/pkg/agent/svid/store"
+	"github.com/spiffe/spire/pkg/common/backoff"
 	"github.com/spiffe/spire/pkg/common/diskutil"
 	"github.com/spiffe/spire/pkg/common/health"
+	"github.com/spiffe/spire/pkg/common/nodeutil"
 	"github.com/spiffe/spire/pkg/common/profiling"
 	"github.com/spiffe/spire/pkg/common/rotationutil"
 	"github.com/spiffe/spire/pkg/common/telemetry"
 	"github.com/spiffe/spire/pkg/common/uptime"
 	"github.com/spiffe/spire/pkg/common/util"
+	"github.com/spiffe/spire/pkg/common/version"
 	_ "golang.org/x/net/trace" // registers handlers on the DefaultServeMux
 	"google.golang.org/grpc"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
 )
 
+const (
+	bootstrapBackoffInterval       = 5 * time.Second
+	bootstrapBackoffMaxElapsedTime = 1 * time.Minute
+)
+
 type Agent struct {
 	c *Config
 }
@@ -42,7 +52,10 @@ type Agent struct {
 // This method initializes the agent, including its plugins,
 // and then blocks on the main event loop.
 func (a *Agent) Run(ctx context.Context) error {
-	a.c.Log.Infof("Starting agent with data directory: %q", a.c.DataDir)
+	a.c.Log.WithFields(logrus.Fields{
+		telemetry.DataDir: a.c.DataDir,
+		telemetry.Version: version.Version(),
+	}).Info("Starting agent")
 	if err := diskutil.CreateDataDirectory(a.c.DataDir); err != nil {
 		return err
 	}
@@ -89,12 +102,51 @@ func (a *Agent) Run(ctx context.Context) error {
 		nodeAttestor = cat.GetNodeAttestor()
 	}
 
-	as, err := a.attest(ctx, sto, cat, metrics, nodeAttestor)
-	if err != nil {
-		return err
+	var as *node_attestor.AttestationResult
+
+	if a.c.RetryBootstrap {
+		attBackoffClock := clock.New()
+		attBackoff := backoff.NewBackoff(
+			attBackoffClock,
+			bootstrapBackoffInterval,
+			backoff.WithMaxElapsedTime(bootstrapBackoffMaxElapsedTime),
+		)
+
+		for {
+			as, err = a.attest(ctx, sto, cat, metrics, nodeAttestor)
+			if err == nil {
+				break
+			}
+
+			if status.Code(err) == codes.PermissionDenied {
+				return err
+			}
+
+			nextDuration := attBackoff.NextBackOff()
+			if nextDuration == backoff.Stop {
+				return err
+			}
+
+			a.c.Log.WithFields(logrus.Fields{
+				telemetry.Error:         err,
+				telemetry.RetryInterval: nextDuration,
+			}).Warn("Failed to retrieve attestation result")
+
+			select {
+			case <-ctx.Done():
+				return ctx.Err()
+			case <-attBackoffClock.After(nextDuration):
+				continue
+			}
+		}
+	} else {
+		as, err = a.attest(ctx, sto, cat, metrics, nodeAttestor)
+		if err != nil {
+			return err
+		}
 	}
 
-	svidStoreCache := a.newSVIDStoreCache()
+	svidStoreCache := a.newSVIDStoreCache(metrics)
 
 	manager, err := a.newManager(ctx, sto, cat, metrics, as, svidStoreCache, nodeAttestor)
 	if err != nil {
@@ -119,6 +171,7 @@ func (a *Agent) Run(ctx context.Context) error {
 		storeService.Run,
 		endpoints.ListenAndServe,
 		metrics.ListenAndServe,
+		catalog.ReconfigureTask(a.c.Log.WithField(telemetry.SubsystemName, "reconfigurer"), cat),
 		util.SerialRun(a.waitForTestDial, healthChecker.ListenAndServe),
 	}
 
@@ -207,6 +260,7 @@ func (a *Agent) attest(ctx context.Context, sto storage.Storage, cat catalog.Cat
 		Log:               a.c.Log.WithField(telemetry.SubsystemName, telemetry.Attestor),
 		ServerAddress:     a.c.ServerAddress,
 		NodeAttestor:      na,
+		TLSPolicy:         a.c.TLSPolicy,
 	}
 	return node_attestor.New(&config).Attest(ctx)
 }
@@ -226,24 +280,63 @@ func (a *Agent) newManager(ctx context.Context, sto storage.Storage, cat catalog
 		Storage:                  sto,
 		SyncInterval:             a.c.SyncInterval,
 		UseSyncAuthorizedEntries: a.c.UseSyncAuthorizedEntries,
-		SVIDCacheMaxSize:         a.c.X509SVIDCacheMaxSize,
+		X509SVIDCacheMaxSize:     a.c.X509SVIDCacheMaxSize,
+		JWTSVIDCacheMaxSize:      a.c.JWTSVIDCacheMaxSize,
 		SVIDStoreCache:           cache,
 		NodeAttestor:             na,
 		RotationStrategy:         rotationutil.NewRotationStrategy(a.c.AvailabilityTarget),
+		TLSPolicy:                a.c.TLSPolicy,
 	}
 
 	mgr := manager.New(config)
-	if err := mgr.Initialize(ctx); err != nil {
-		return nil, err
-	}
+	if a.c.RetryBootstrap {
+		initBackoffClock := clock.New()
+		initBackoff := backoff.NewBackoff(
+			initBackoffClock,
+			bootstrapBackoffInterval,
+			backoff.WithMaxElapsedTime(bootstrapBackoffMaxElapsedTime),
+		)
 
-	return mgr, nil
+		for {
+			err := mgr.Initialize(ctx)
+			if err == nil {
+				return mgr, nil
+			}
+
+			if nodeutil.ShouldAgentReattest(err) || nodeutil.ShouldAgentShutdown(err) {
+				return nil, err
+			}
+
+			nextDuration := initBackoff.NextBackOff()
+			if nextDuration == backoff.Stop {
+				return nil, err
+			}
+
+			a.c.Log.WithFields(logrus.Fields{
+				telemetry.Error:         err,
+				telemetry.RetryInterval: nextDuration,
+			}).Warn("Failed to initialize manager")
+
+			select {
+			case <-ctx.Done():
+				return nil, ctx.Err()
+			case <-initBackoffClock.After(nextDuration):
+				continue
+			}
+		}
+	} else {
+		if err := mgr.Initialize(ctx); err != nil {
+			return nil, err
+		}
+		return mgr, nil
+	}
 }
 
-func (a *Agent) newSVIDStoreCache() *storecache.Cache {
+func (a *Agent) newSVIDStoreCache(metrics telemetry.Metrics) *storecache.Cache {
 	config := &storecache.Config{
 		Log:         a.c.Log.WithField(telemetry.SubsystemName, "svid_store_cache"),
 		TrustDomain: a.c.TrustDomain,
+		Metrics:     metrics,
 	}
 
 	return storecache.New(config)
