@@ -9,6 +9,7 @@ import (
 	"testing"
 	"time"
 
+	"github.com/andres-erbsen/clock"
 	"github.com/sirupsen/logrus"
 	"github.com/sirupsen/logrus/hooks/test"
 	"github.com/spiffe/go-spiffe/v2/bundle/spiffebundle"
@@ -79,6 +80,7 @@ func TestSubscribeToX509SVIDs(t *testing.T) {
 		managerErr    error
 		expectMetrics []fakemetrics.MetricItem
 		expectResp    *delegatedidentityv1.SubscribeToX509SVIDsResponse
+		req           *delegatedidentityv1.SubscribeToX509SVIDsRequest
 	}{
 		{
 			testName:   "attest error",
@@ -86,6 +88,32 @@ func TestSubscribeToX509SVIDs(t *testing.T) {
 			expectCode: codes.Internal,
 			expectMsg:  "workload attestation failed",
 		},
+		{
+			testName:     "incorrectly populate both pid and selectors",
+			authSpiffeID: []string{"spiffe://example.org/one"},
+			identities: []cache.Identity{
+				identities[0],
+			},
+			expectCode: codes.InvalidArgument,
+			expectMsg:  "must provide either selectors or non-zero PID, but not both",
+			req: &delegatedidentityv1.SubscribeToX509SVIDsRequest{
+				Selectors: []*types.Selector{{Type: "sa", Value: "foo"}},
+				Pid:       447,
+			},
+		},
+		{
+			testName:     "incorrectly populate neither pid or selectors",
+			authSpiffeID: []string{"spiffe://example.org/one"},
+			identities: []cache.Identity{
+				identities[0],
+			},
+			expectCode: codes.InvalidArgument,
+			expectMsg:  "must provide either selectors or non-zero PID, but not both",
+			req: &delegatedidentityv1.SubscribeToX509SVIDsRequest{
+				Selectors: []*types.Selector{},
+				Pid:       0,
+			},
+		},
 		{
 			testName:     "access to \"privileged\" admin API denied",
 			authSpiffeID: []string{"spiffe://example.org/one/wrong"},
@@ -112,9 +140,10 @@ func TestSubscribeToX509SVIDs(t *testing.T) {
 				identities[0],
 			},
 			updates: []*cache.WorkloadUpdate{
-				{Identities: []cache.Identity{
-					identities[0],
-				},
+				{
+					Identities: []cache.Identity{
+						identities[0],
+					},
 					Bundle: bundle,
 				},
 			},
@@ -175,9 +204,8 @@ func TestSubscribeToX509SVIDs(t *testing.T) {
 			identities: []cache.Identity{
 				identities[0],
 			},
-			updates:       []*cache.WorkloadUpdate{{}},
-			expectResp:    &delegatedidentityv1.SubscribeToX509SVIDsResponse{},
-			expectMetrics: generateSubscribeToX509SVIDMetrics(),
+			updates:    []*cache.WorkloadUpdate{{}},
+			expectResp: &delegatedidentityv1.SubscribeToX509SVIDsResponse{},
 		},
 		{
 			testName:     "workload update without identity.SVID",
@@ -194,6 +222,40 @@ func TestSubscribeToX509SVIDs(t *testing.T) {
 			expectMsg:     "could not serialize response",
 			expectMetrics: generateSubscribeToX509SVIDMetrics(),
 		},
+		{
+			testName:     "workload update by PID with identity and federated bundles",
+			authSpiffeID: []string{"spiffe://example.org/one"},
+			req: &delegatedidentityv1.SubscribeToX509SVIDsRequest{
+				Pid: 447,
+			},
+			identities: []cache.Identity{
+				identities[0],
+			},
+			updates: []*cache.WorkloadUpdate{
+				{
+					Identities: []cache.Identity{
+						identities[0],
+					},
+					Bundle: bundle,
+					FederatedBundles: map[spiffeid.TrustDomain]*spiffebundle.Bundle{
+						federatedBundle1.TrustDomain(): federatedBundle1},
+				},
+			},
+			expectResp: &delegatedidentityv1.SubscribeToX509SVIDsResponse{
+				X509Svids: []*delegatedidentityv1.X509SVIDWithKey{
+					{
+						X509Svid: &types.X509SVID{
+							Id:        utilIDProtoFromString(t, x509SVID1.ID.String()),
+							CertChain: x509util.RawCertsFromCertificates(x509SVID1.Certificates),
+							ExpiresAt: x509SVID1.Certificates[0].NotAfter.Unix(),
+						},
+						X509SvidKey: pkcs8FromSigner(t, x509SVID1.PrivateKey),
+					},
+				},
+				FederatesWith: []string{federatedBundle1.TrustDomain().IDString()},
+			},
+			expectMetrics: generateSubscribeToX509SVIDMetrics(),
+		},
 		{
 			testName:     "workload update with identity and federated bundles",
 			authSpiffeID: []string{"spiffe://example.org/one"},
@@ -271,22 +333,24 @@ func TestSubscribeToX509SVIDs(t *testing.T) {
 				ManagerErr:   tt.managerErr,
 				Metrics:      metrics,
 			}
-			runTest(t, params,
-				func(ctx context.Context, client delegatedidentityv1.DelegatedIdentityClient) {
-					selectors := []*types.Selector{{Type: "sa", Value: "foo"}}
-					req := &delegatedidentityv1.SubscribeToX509SVIDsRequest{
-						Selectors: selectors,
-					}
+			runTest(t, params, func(ctx context.Context, client delegatedidentityv1.DelegatedIdentityClient) {
+				req := &delegatedidentityv1.SubscribeToX509SVIDsRequest{
+					Selectors: []*types.Selector{{Type: "sa", Value: "foo"}},
+				}
+				// if test params has a custom request, prefer that
+				if tt.req != nil {
+					req = tt.req
+				}
 
-					stream, err := client.SubscribeToX509SVIDs(ctx, req)
+				stream, err := client.SubscribeToX509SVIDs(ctx, req)
 
-					require.NoError(t, err)
-					resp, err := stream.Recv()
+				require.NoError(t, err)
+				resp, err := stream.Recv()
 
-					spiretest.RequireGRPCStatus(t, err, tt.expectCode, tt.expectMsg)
-					spiretest.RequireProtoEqual(t, tt.expectResp, resp)
-					require.Equal(t, tt.expectMetrics, metrics.AllMetrics())
-				})
+				spiretest.RequireGRPCStatus(t, err, tt.expectCode, tt.expectMsg)
+				spiretest.RequireProtoEqual(t, tt.expectResp, resp)
+				require.Equal(t, tt.expectMetrics, metrics.AllMetrics())
+			})
 		})
 	}
 }
@@ -409,6 +473,7 @@ func TestFetchJWTSVIDs(t *testing.T) {
 		authSpiffeID []string
 		audience     []string
 		selectors    []*types.Selector
+		pid          int32
 		expectCode   codes.Code
 		expectMsg    string
 		attestErr    error
@@ -427,6 +492,30 @@ func TestFetchJWTSVIDs(t *testing.T) {
 			expectCode: codes.Internal,
 			expectMsg:  "workload attestation failed",
 		},
+		{
+			testName:     "incorrectly populate both pid and selectors",
+			authSpiffeID: []string{"spiffe://example.org/one"},
+			selectors:    []*types.Selector{{Type: "sa", Value: "foo"}},
+			pid:          447,
+			audience:     []string{"AUDIENCE"},
+			identities: []cache.Identity{
+				identities[0],
+			},
+			expectCode: codes.InvalidArgument,
+			expectMsg:  "must provide either selectors or non-zero PID, but not both",
+		},
+		{
+			testName:     "incorrectly populate neither pid or selectors",
+			authSpiffeID: []string{"spiffe://example.org/one"},
+			selectors:    []*types.Selector{},
+			pid:          0,
+			audience:     []string{"AUDIENCE"},
+			identities: []cache.Identity{
+				identities[0],
+			},
+			expectCode: codes.InvalidArgument,
+			expectMsg:  "must provide either selectors or non-zero PID, but not both",
+		},
 		{
 			testName:     "Access to \"privileged\" admin API denied",
 			authSpiffeID: []string{"spiffe://example.org/one/wrong"},
@@ -509,6 +598,33 @@ func TestFetchJWTSVIDs(t *testing.T) {
 				},
 			},
 		},
+		{
+			testName:     "success with one identity by PID",
+			pid:          447,
+			authSpiffeID: []string{"spiffe://example.org/one"},
+			audience:     []string{"AUDIENCE"},
+			identities: []cache.Identity{
+				identities[0],
+			},
+			jwtSVIDsResp: map[spiffeid.ID]*client.JWTSVID{
+				id1: {
+					Token:     jwtSVID1Token,
+					ExpiresAt: time.Unix(1680786600, 0),
+					IssuedAt:  time.Unix(1680783000, 0),
+				},
+			},
+			expectResp: &delegatedidentityv1.FetchJWTSVIDsResponse{
+				Svids: []*types.JWTSVID{
+					{
+						Token:     jwtSVID1Token,
+						Id:        api.ProtoFromID(id1),
+						Hint:      "internal",
+						ExpiresAt: 1680786600,
+						IssuedAt:  1680783000,
+					},
+				},
+			},
+		},
 		{
 			testName:     "success with two identities",
 			authSpiffeID: []string{"spiffe://example.org/one"},
@@ -562,6 +678,7 @@ func TestFetchJWTSVIDs(t *testing.T) {
 					resp, err := client.FetchJWTSVIDs(ctx, &delegatedidentityv1.FetchJWTSVIDsRequest{
 						Audience:  tt.audience,
 						Selectors: tt.selectors,
+						Pid:       tt.pid,
 					})
 
 					spiretest.RequireGRPCStatus(t, err, tt.expectCode, tt.expectMsg)
@@ -578,6 +695,7 @@ func TestFetchJWTSVIDs(t *testing.T) {
 		})
 	}
 }
+
 func TestSubscribeToJWTBundles(t *testing.T) {
 	ca := testca.New(t, trustDomain1)
 
@@ -707,7 +825,11 @@ func runTest(t *testing.T, params testParams, fn func(ctx context.Context, clien
 		AuthorizedDelegates: params.AuthSpiffeID,
 	})
 
-	service.attestor = FakeAttestor{
+	service.peerAttestor = FakeAttestor{
+		err: params.AttestErr,
+	}
+
+	service.delegateWorkloadAttestor = FakeWorkloadPIDAttestor{
 		err: params.AttestErr,
 	}
 
@@ -722,7 +844,7 @@ func runTest(t *testing.T, params testParams, fn func(ctx context.Context, clien
 	ctx, cancel := context.WithTimeout(context.Background(), time.Minute)
 	defer cancel()
 
-	conn, _ := grpc.DialContext(ctx, "unix:"+addr.String(), grpc.WithTransportCredentials(insecure.NewCredentials()))
+	conn, _ := grpc.DialContext(ctx, "unix:"+addr.String(), grpc.WithTransportCredentials(insecure.NewCredentials())) //nolint: staticcheck // It is going to be resolved on #5152
 	t.Cleanup(func() { conn.Close() })
 
 	fn(ctx, delegatedidentityv1.NewDelegatedIdentityClient(conn))
@@ -739,6 +861,15 @@ func (fa FakeAttestor) Attest(context.Context) ([]*common.Selector, error) {
 	return fa.selectors, fa.err
 }
 
+type FakeWorkloadPIDAttestor struct {
+	selectors []*common.Selector
+	err       error
+}
+
+func (fa FakeWorkloadPIDAttestor) Attest(_ context.Context, _ int) ([]*common.Selector, error) {
+	return fa.selectors, fa.err
+}
+
 type FakeManager struct {
 	manager.Manager
 
@@ -863,9 +994,9 @@ func (m *FakeManager) SubscribeToBundleChanges() *cache.BundleStream {
 	return myCache.BundleCache.SubscribeToBundleChanges()
 }
 
-func newTestCache() *cache.Cache {
+func newTestCache() *cache.LRUCache {
 	log, _ := test.NewNullLogger()
-	return cache.New(log, trustDomain1, bundle1, telemetry.Blackhole{})
+	return cache.NewLRUCache(log, trustDomain1, bundle1, telemetry.Blackhole{}, cache.DefaultSVIDCacheMaxSize, cache.DefaultSVIDCacheMaxSize, clock.New())
 }
 
 func generateSubscribeToX509SVIDMetrics() []fakemetrics.MetricItem {
