@@ -1235,7 +1235,7 @@ func setupTestWithManager(t *testing.T, c Config, manager *FakeManager) *handler
 	listener, err := net.Listen("tcp", "localhost:0")
 	require.NoError(t, err)
 
-	conn, err := grpc.Dial(listener.Addr().String(), grpc.WithTransportCredentials(insecure.NewCredentials()))
+	conn, err := grpc.NewClient(listener.Addr().String(), grpc.WithTransportCredentials(insecure.NewCredentials()))
 	require.NoError(t, err)
 	log, _ := test.NewNullLogger()
 	unaryInterceptor, streamInterceptor := middleware.Interceptors(middleware.WithLogger(log))
