@@ -5,7 +5,9 @@ package k8ssat
 import (
 	"testing"
 
+	"github.com/spiffe/go-spiffe/v2/spiffeid"
 	"github.com/spiffe/spire/pkg/agent/plugin/nodeattestor"
+	"github.com/spiffe/spire/pkg/common/catalog"
 	"github.com/spiffe/spire/test/plugintest"
 	"github.com/stretchr/testify/require"
 )
@@ -13,24 +15,28 @@ import (
 func TestConfigureDefaultToken(t *testing.T) {
 	for _, tt := range []struct {
 		name            string
+		trustDomain     string
 		mountPoint      string
 		config          string
 		expectTokenPath string
 	}{
 		{
 			name:            "mountPoint set",
+			trustDomain:     "example.org",
 			mountPoint:      "c:\\somepath",
 			config:          `cluster = "production"`,
 			expectTokenPath: "c:\\somepath\\var\\run\\secrets\\kubernetes.io\\serviceaccount\\token",
 		},
 		{
 			name:            "no mountPoint",
+			trustDomain:     "example.org",
 			config:          `cluster = "production"`,
 			expectTokenPath: "\\var\\run\\secrets\\kubernetes.io\\serviceaccount\\token",
 		},
 		{
-			name:       "token path set on configuration",
-			mountPoint: "c:\\somepath",
+			name:        "token path set on configuration",
+			trustDomain: "example.org",
+			mountPoint:  "c:\\somepath",
 			config: `
 			cluster = "production"
 			token_path = "c:\\token"`,
@@ -44,7 +50,13 @@ func TestConfigureDefaultToken(t *testing.T) {
 
 			p := New()
 			var err error
-			plugintest.Load(t, builtin(p), new(nodeattestor.V1), plugintest.CaptureConfigureError(&err), plugintest.Configure(tt.config))
+			plugintest.Load(t, builtin(p), new(nodeattestor.V1),
+				plugintest.CaptureConfigureError(&err),
+				plugintest.CoreConfig(catalog.CoreConfig{
+					TrustDomain: spiffeid.RequireTrustDomainFromString(tt.trustDomain),
+				}),
+				plugintest.Configure(tt.config),
+			)
 			require.NoError(t, err)
 
 			require.Equal(t, tt.expectTokenPath, p.config.tokenPath)
