@@ -11,6 +11,7 @@ import (
 	"net/http"
 	"net/url"
 	"os"
+	"path/filepath"
 	"strconv"
 	"strings"
 	"sync"
@@ -21,11 +22,13 @@ import (
 	"github.com/hashicorp/hcl"
 	workloadattestorv1 "github.com/spiffe/spire-plugin-sdk/proto/spire/plugin/agent/workloadattestor/v1"
 	configv1 "github.com/spiffe/spire-plugin-sdk/proto/spire/service/common/config/v1"
-	"github.com/spiffe/spire/pkg/agent/common/cgroups"
+	"github.com/spiffe/spire/pkg/agent/common/sigstore"
 	"github.com/spiffe/spire/pkg/common/catalog"
 	"github.com/spiffe/spire/pkg/common/pemutil"
+	"github.com/spiffe/spire/pkg/common/pluginconf"
 	"github.com/spiffe/spire/pkg/common/telemetry"
 	"github.com/valyala/fastjson"
+	"golang.org/x/sync/singleflight"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
 	corev1 "k8s.io/api/core/v1"
@@ -41,7 +44,6 @@ const (
 	defaultTokenPath         = "/var/run/secrets/kubernetes.io/serviceaccount/token" //nolint: gosec // false positive
 	defaultNodeNameEnv       = "MY_NODE_NAME"
 	defaultReloadInterval    = time.Minute
-	maximumAmountCache       = 10
 )
 
 func BuiltIn() catalog.BuiltIn {
@@ -78,7 +80,7 @@ type HCLConfig struct {
 	// SkipKubeletVerification is set. Defaults to the cluster trust bundle.
 	KubeletCAPath string `hcl:"kubelet_ca_path"`
 
-	// SkipKubeletVerification controls whether or not the plugin will
+	// SkipKubeletVerification controls whether the plugin will
 	// verify the certificate presented by the kubelet.
 	SkipKubeletVerification bool `hcl:"skip_kubelet_verification"`
 
@@ -95,7 +97,7 @@ type HCLConfig struct {
 	// authentication with the kubelet. Must be used with CertificatePath.
 	PrivateKeyPath string `hcl:"private_key_path"`
 
-	// UseAnonymousAuthentication controls whether or not communication to the
+	// UseAnonymousAuthentication controls whether communication to the
 	// kubelet over the secure port is unauthenticated. This option is mutually
 	// exclusive with other authentication configuration fields TokenPath,
 	// CertificatePath, and PrivateKeyPath.
@@ -122,28 +124,22 @@ type HCLConfig struct {
 	// (e.g. when a postStart hook has yet to complete).
 	DisableContainerSelectors bool `hcl:"disable_container_selectors"`
 
+	// UseNewContainerLocator, if true, uses the new container locator
+	// mechanism instead of the legacy cgroup matchers. Defaults to true if
+	// unset. This configurable will be removed in a future release.
+	UseNewContainerLocator *bool `hcl:"use_new_container_locator"`
+
+	// VerboseContainerLocatorLogs, if true, dumps extra information to the log
+	// about mountinfo and cgroup information used to locate the container.
+	VerboseContainerLocatorLogs bool `hcl:"verbose_container_locator_logs"`
+
 	// Experimental enables experimental features.
-	Experimental *ExperimentalK8SConfig `hcl:"experimental,omitempty"`
+	Experimental experimentalK8SConfig `hcl:"experimental,omitempty"`
 }
 
-type ExperimentalK8SConfig struct {
+type experimentalK8SConfig struct {
 	// Sigstore contains sigstore specific configs.
-	Sigstore *SigstoreHCLConfig `hcl:"sigstore,omitempty"`
-}
-
-// SigstoreHCLConfig holds the sigstore configuration parsed from HCL
-type SigstoreHCLConfig struct {
-	// EnforceSCT is the parameter to be set as false in case of a private deployment not using the public CT
-	EnforceSCT *bool `hcl:"enforce_sct, omitempty"`
-
-	// RekorURL is the URL for the rekor server to use to verify signatures and public keys
-	RekorURL *string `hcl:"rekor_url,omitempty"`
-
-	// SkippedImages is a list of images that should skip sigstore verification
-	SkippedImages []string `hcl:"skip_signature_verification_image_list"`
-
-	// AllowedSubjects is a list of subjects that should be allowed after verification
-	AllowedSubjects map[string][]string `hcl:"allowed_subjects_list"`
+	Sigstore *sigstore.HCLConfig `hcl:"sigstore,omitempty"`
 }
 
 // k8sConfig holds the configuration distilled from HCL
@@ -161,14 +157,106 @@ type k8sConfig struct {
 	NodeName                   string
 	ReloadInterval             time.Duration
 	DisableContainerSelectors  bool
+	ContainerHelper            ContainerHelper
+	sigstoreConfig             *sigstore.Config
 
 	Client     *kubeletClient
 	LastReload time.Time
 }
 
+func (p *Plugin) buildConfig(coreConfig catalog.CoreConfig, hclText string, status *pluginconf.Status) *k8sConfig {
+	// Parse HCL config payload into config struct
+	newConfig := new(HCLConfig)
+	if err := hcl.Decode(newConfig, hclText); err != nil {
+		status.ReportErrorf("unable to decode configuration: %v", err)
+		return nil
+	}
+
+	// Determine max poll attempts with default
+	maxPollAttempts := newConfig.MaxPollAttempts
+	if maxPollAttempts <= 0 {
+		maxPollAttempts = defaultMaxPollAttempts
+	}
+
+	// Determine poll retry interval with default
+	var pollRetryInterval time.Duration
+	var err error
+	if newConfig.PollRetryInterval != "" {
+		pollRetryInterval, err = time.ParseDuration(newConfig.PollRetryInterval)
+		if err != nil {
+			status.ReportErrorf("unable to parse poll retry interval: %v", err)
+		}
+	}
+	if pollRetryInterval <= 0 {
+		pollRetryInterval = defaultPollRetryInterval
+	}
+
+	// Determine reload interval
+	var reloadInterval time.Duration
+	if newConfig.ReloadInterval != "" {
+		reloadInterval, err = time.ParseDuration(newConfig.ReloadInterval)
+		if err != nil {
+			status.ReportErrorf("unable to parse reload interval: %v", err)
+		}
+	}
+	if reloadInterval <= 0 {
+		reloadInterval = defaultReloadInterval
+	}
+
+	// Determine which kubelet port to hit. Default to the secure port if none
+	// is specified (this is backwards compatible because the read-only-port
+	// config value has always been required, so it should already be set in
+	// existing configurations that rely on it).
+	if newConfig.KubeletSecurePort > 0 && newConfig.KubeletReadOnlyPort > 0 {
+		status.ReportError("cannot use both the read-only and secure port")
+	}
+
+	port := newConfig.KubeletReadOnlyPort
+	secure := false
+	if port <= 0 {
+		port = newConfig.KubeletSecurePort
+		secure = true
+	}
+	if port <= 0 {
+		port = defaultSecureKubeletPort
+		secure = true
+	}
+
+	containerHelper := createHelper(p)
+	if err := containerHelper.Configure(newConfig, p.log); err != nil {
+		status.ReportError(err.Error())
+	}
+
+	// Determine the node name
+	nodeName := p.getNodeName(newConfig.NodeName, newConfig.NodeNameEnv)
+
+	var sigstoreConfig *sigstore.Config
+	if newConfig.Experimental.Sigstore != nil {
+		sigstoreConfig = sigstore.NewConfigFromHCL(newConfig.Experimental.Sigstore, p.log)
+	}
+
+	// return the kubelet client
+	return &k8sConfig{
+		Secure:                     secure,
+		Port:                       port,
+		MaxPollAttempts:            maxPollAttempts,
+		PollRetryInterval:          pollRetryInterval,
+		SkipKubeletVerification:    newConfig.SkipKubeletVerification,
+		TokenPath:                  newConfig.TokenPath,
+		CertificatePath:            newConfig.CertificatePath,
+		PrivateKeyPath:             newConfig.PrivateKeyPath,
+		UseAnonymousAuthentication: newConfig.UseAnonymousAuthentication,
+		KubeletCAPath:              newConfig.KubeletCAPath,
+		NodeName:                   nodeName,
+		ReloadInterval:             reloadInterval,
+		DisableContainerSelectors:  newConfig.DisableContainerSelectors,
+		ContainerHelper:            containerHelper,
+		sigstoreConfig:             sigstoreConfig,
+	}
+}
+
 type ContainerHelper interface {
 	Configure(config *HCLConfig, log hclog.Logger) error
-	GetOSSelectors(ctx context.Context, log hclog.Logger, containerStatus *corev1.ContainerStatus) ([]string, error)
 	GetPodUIDAndContainerID(pID int32, log hclog.Logger) (types.UID, string, error)
 }
 
@@ -176,19 +264,23 @@ type Plugin struct {
 	workloadattestorv1.UnsafeWorkloadAttestorServer
 	configv1.UnsafeConfigServer
 
-	log    hclog.Logger
-	clock  clock.Clock
-	fs     cgroups.FileSystem
-	c      ContainerHelper
-	getenv func(string) string
+	log     hclog.Logger
+	clock   clock.Clock
+	rootDir string
+	getenv  func(string) string
 
-	mu     sync.RWMutex
-	config *k8sConfig
+	mu               sync.RWMutex
+	config           *k8sConfig
+	containerHelper  ContainerHelper
+	sigstoreVerifier sigstore.Verifier
+
+	cachedPodList           map[string]*fastjson.Value
+	cachedPodListValidUntil time.Time
+	singleflight            singleflight.Group
 }
 
 func New() *Plugin {
 	return &Plugin{
-		fs:     cgroups.OSFileSystem{},
 		clock:  clock.New(),
 		getenv: os.Getenv,
 	}
@@ -199,12 +291,12 @@ func (p *Plugin) SetLogger(log hclog.Logger) {
 }
 
 func (p *Plugin) Attest(ctx context.Context, req *workloadattestorv1.AttestRequest) (*workloadattestorv1.AttestResponse, error) {
-	config, err := p.getConfig()
+	config, containerHelper, sigstoreVerifier, err := p.getConfig()
 	if err != nil {
 		return nil, err
 	}
 
-	podUID, containerID, err := p.c.GetPodUIDAndContainerID(req.Pid, p.log)
+	podUID, containerID, err := containerHelper.GetPodUIDAndContainerID(req.Pid, p.log)
 	if err != nil {
 		return nil, err
 	}
@@ -226,22 +318,15 @@ func (p *Plugin) Attest(ctx context.Context, req *workloadattestorv1.AttestReque
 	for attempt := 1; ; attempt++ {
 		log = log.With(telemetry.Attempt, attempt)
 
-		podListBytes, err := config.Client.GetPodList()
+		podList, err := p.getPodList(ctx, config.Client, config.PollRetryInterval/2)
 		if err != nil {
 			return nil, err
 		}
 
-		var parser fastjson.Parser
-		podList, err := parser.ParseBytes(podListBytes)
-		if err != nil {
-			return nil, status.Errorf(codes.Internal, "unable to parse kubelet response: %v", err)
-		}
-
 		var attestResponse *workloadattestorv1.AttestResponse
-		for _, podValue := range podList.GetArray("items") {
+		for podKey, podValue := range podList {
 			if podKnown {
-				uidBytes := podValue.Get("metadata", "uid").GetStringBytes()
-				if string(uidBytes) != string(podUID) {
+				if podKey != string(podUID) {
 					// The pod holding the container is known. Skip unrelated pods.
 					continue
 				}
@@ -267,14 +352,15 @@ func (p *Plugin) Attest(ctx context.Context, req *workloadattestorv1.AttestReque
 				selectorValues = append(selectorValues, getSelectorValuesFromPodInfo(pod)...)
 				if !config.DisableContainerSelectors {
 					selectorValues = append(selectorValues, getSelectorValuesFromWorkloadContainerStatus(containerStatus)...)
+				}
 
-					osSelector, err := p.c.GetOSSelectors(ctx, log, containerStatus)
-					switch {
-					case err != nil:
-						return nil, err
-					case len(osSelector) > 0:
-						selectorValues = append(selectorValues, osSelector...)
+				if sigstoreVerifier != nil {
+					log.Debug("Attempting to verify sigstore image signature", "image", containerStatus.Image)
+					sigstoreSelectors, err := p.sigstoreVerifier.Verify(ctx, containerStatus.ImageID)
+					if err != nil {
+						return nil, status.Errorf(codes.Internal, "error verifying sigstore image signature for imageID %s: %v", containerStatus.ImageID, err)
 					}
+					selectorValues = append(selectorValues, sigstoreSelectors...)
 				}
 
 			case podKnown && config.DisableContainerSelectors:
@@ -314,120 +400,80 @@ func (p *Plugin) Attest(ctx context.Context, req *workloadattestorv1.AttestReque
 	}
 }
 
-func (p *Plugin) Configure(_ context.Context, req *configv1.ConfigureRequest) (resp *configv1.ConfigureResponse, err error) {
-	// Parse HCL config payload into config struct
-	config := new(HCLConfig)
-	if err := hcl.Decode(config, req.HclConfiguration); err != nil {
-		return nil, status.Errorf(codes.InvalidArgument, "unable to decode configuration: %v", err)
-	}
-
-	// Determine max poll attempts with default
-	maxPollAttempts := config.MaxPollAttempts
-	if maxPollAttempts <= 0 {
-		maxPollAttempts = defaultMaxPollAttempts
-	}
-
-	// Determine poll retry interval with default
-	var pollRetryInterval time.Duration
-	if config.PollRetryInterval != "" {
-		pollRetryInterval, err = time.ParseDuration(config.PollRetryInterval)
-		if err != nil {
-			return nil, status.Errorf(codes.InvalidArgument, "unable to parse poll retry interval: %v", err)
-		}
-	}
-	if pollRetryInterval <= 0 {
-		pollRetryInterval = defaultPollRetryInterval
-	}
-
-	// Determine reload interval
-	var reloadInterval time.Duration
-	if config.ReloadInterval != "" {
-		reloadInterval, err = time.ParseDuration(config.ReloadInterval)
-		if err != nil {
-			return nil, status.Errorf(codes.InvalidArgument, "unable to parse reload interval: %v", err)
-		}
-	}
-	if reloadInterval <= 0 {
-		reloadInterval = defaultReloadInterval
-	}
-
-	// Determine which kubelet port to hit. Default to the secure port if none
-	// is specified (this is backwards compatible because the read-only-port
-	// config value has always been required, so it should already be set in
-	// existing configurations that rely on it).
-	if config.KubeletSecurePort > 0 && config.KubeletReadOnlyPort > 0 {
-		return nil, status.Error(codes.InvalidArgument, "cannot use both the read-only and secure port")
-	}
-
-	containerHelper := createHelper(p)
-	if err := containerHelper.Configure(config, p.log); err != nil {
+func (p *Plugin) Configure(ctx context.Context, req *configv1.ConfigureRequest) (resp *configv1.ConfigureResponse, err error) {
+	newConfig, _, err := pluginconf.Build(req, p.buildConfig)
+	if err != nil {
 		return nil, err
 	}
 
-	port := config.KubeletReadOnlyPort
-	secure := false
-	if port <= 0 {
-		port = config.KubeletSecurePort
-		secure = true
-	}
-	if port <= 0 {
-		port = defaultSecureKubeletPort
-		secure = true
-	}
-
-	// Determine the node name
-	nodeName := p.getNodeName(config.NodeName, config.NodeNameEnv)
-
-	// Configure the kubelet client
-	c := &k8sConfig{
-		Secure:                     secure,
-		Port:                       port,
-		MaxPollAttempts:            maxPollAttempts,
-		PollRetryInterval:          pollRetryInterval,
-		SkipKubeletVerification:    config.SkipKubeletVerification,
-		TokenPath:                  config.TokenPath,
-		CertificatePath:            config.CertificatePath,
-		PrivateKeyPath:             config.PrivateKeyPath,
-		UseAnonymousAuthentication: config.UseAnonymousAuthentication,
-		KubeletCAPath:              config.KubeletCAPath,
-		NodeName:                   nodeName,
-		ReloadInterval:             reloadInterval,
-		DisableContainerSelectors:  config.DisableContainerSelectors,
-	}
-
-	if err := p.reloadKubeletClient(c); err != nil {
+	if err := p.reloadKubeletClient(newConfig); err != nil {
 		return nil, err
 	}
 
-	// Set the config
-	p.setConfig(c)
-	p.setContainerHelper(containerHelper)
+	var sigstoreVerifier sigstore.Verifier
+	if newConfig.sigstoreConfig != nil {
+		verifier := sigstore.NewVerifier(newConfig.sigstoreConfig)
+		err = verifier.Init(ctx)
+		if err != nil {
+			return nil, status.Errorf(codes.InvalidArgument, "error initializing sigstore verifier: %v", err)
+		}
+		sigstoreVerifier = verifier
+	}
+
+	p.mu.Lock()
+	defer p.mu.Unlock()
+	p.config = newConfig
+	p.containerHelper = newConfig.ContainerHelper
+	p.sigstoreVerifier = sigstoreVerifier
+
 	return &configv1.ConfigureResponse{}, nil
 }
 
-func (p *Plugin) setConfig(config *k8sConfig) {
-	p.mu.Lock()
-	defer p.mu.Unlock()
-	p.config = config
+func (p *Plugin) Validate(_ context.Context, req *configv1.ValidateRequest) (resp *configv1.ValidateResponse, err error) {
+	_, notes, err := pluginconf.Build(req, p.buildConfig)
+
+	return &configv1.ValidateResponse{
+		Valid: err == nil,
+		Notes: notes,
+	}, nil
 }
 
-func (p *Plugin) getConfig() (*k8sConfig, error) {
+func (p *Plugin) getConfig() (*k8sConfig, ContainerHelper, sigstore.Verifier, error) {
 	p.mu.RLock()
 	defer p.mu.RUnlock()
 
 	if p.config == nil {
-		return nil, status.Error(codes.FailedPrecondition, "not configured")
+		return nil, nil, nil, status.Error(codes.FailedPrecondition, "not configured")
 	}
 	if err := p.reloadKubeletClient(p.config); err != nil {
 		p.log.Warn("Unable to load kubelet client", "err", err)
 	}
-	return p.config, nil
+	return p.config, p.containerHelper, p.sigstoreVerifier, nil
+}
+
+func (p *Plugin) setPodListCache(podList map[string]*fastjson.Value, cacheFor time.Duration) {
+	p.mu.Lock()
+	defer p.mu.Unlock()
+
+	p.cachedPodList = podList
+	p.cachedPodListValidUntil = p.clock.Now().Add(cacheFor)
+}
+
+func (p *Plugin) getPodListCache() map[string]*fastjson.Value {
+	p.mu.RLock()
+	defer p.mu.RUnlock()
+
+	if p.clock.Now().Sub(p.cachedPodListValidUntil) >= 0 {
+		return nil
+	}
+
+	return p.cachedPodList
 }
 
 func (p *Plugin) setContainerHelper(c ContainerHelper) {
 	p.mu.Lock()
 	defer p.mu.Unlock()
-	p.c = c
+	p.containerHelper = c
 }
 
 func (p *Plugin) reloadKubeletClient(config *k8sConfig) (err error) {
@@ -580,7 +626,7 @@ func (p *Plugin) loadToken(path string) (string, error) {
 
 // readFile reads the contents of a file through the filesystem interface
 func (p *Plugin) readFile(path string) ([]byte, error) {
-	f, err := p.fs.Open(path)
+	f, err := os.Open(filepath.Join(p.rootDir, path))
 	if err != nil {
 		return nil, err
 	}
@@ -599,16 +645,67 @@ func (p *Plugin) getNodeName(name string, env string) string {
 	}
 }
 
+func (p *Plugin) getPodList(ctx context.Context, client *kubeletClient, cacheFor time.Duration) (map[string]*fastjson.Value, error) {
+	result := p.getPodListCache()
+	if result != nil {
+		return result, nil
+	}
+
+	podList, err, _ := p.singleflight.Do("podList", func() (interface{}, error) {
+		result := p.getPodListCache()
+		if result != nil {
+			return result, nil
+		}
+
+		podListBytes, err := client.GetPodList(ctx)
+		if err != nil {
+			return nil, err
+		}
+
+		var parser fastjson.Parser
+		podList, err := parser.ParseBytes(podListBytes)
+		if err != nil {
+			return nil, status.Errorf(codes.Internal, "unable to parse kubelet response: %v", err)
+		}
+
+		items := podList.GetArray("items")
+		result = make(map[string]*fastjson.Value, len(items))
+
+		for _, podValue := range items {
+			uid := string(podValue.Get("metadata", "uid").GetStringBytes())
+
+			if uid == "" {
+				p.log.Warn("Pod has no UID", "pod", podValue)
+				continue
+			}
+
+			result[uid] = podValue
+		}
+
+		p.setPodListCache(result, cacheFor)
+
+		return result, nil
+	})
+	if err != nil {
+		return nil, err
+	}
+
+	return podList.(map[string]*fastjson.Value), nil
+}
+
 type kubeletClient struct {
 	Transport *http.Transport
 	URL       url.URL
 	Token     string
 }
 
-func (c *kubeletClient) GetPodList() ([]byte, error) {
+func (c *kubeletClient) GetPodList(ctx context.Context) ([]byte, error) {
+	ctx, cancel := context.WithTimeout(ctx, 10*time.Second)
+	defer cancel()
+
 	url := c.URL
 	url.Path = "/pods"
-	req, err := http.NewRequest("GET", url.String(), nil)
+	req, err := http.NewRequestWithContext(ctx, "GET", url.String(), nil)
 	if err != nil {
 		return nil, status.Errorf(codes.Internal, "unable to create request: %v", err)
 	}
