@@ -9,6 +9,7 @@ import (
 	"os/exec"
 	"path/filepath"
 	"runtime"
+	"slices"
 	"strings"
 	"testing"
 	"time"
@@ -104,6 +105,8 @@ type loadTest struct {
 	expectErr             string
 	expectPluginClient    bool
 	expectServiceClient   bool
+	expectLogEntries      []spiretest.LogEntry
+	epilogue              func(t *testing.T, cat *catalog.Catalog)
 }
 
 func testPlugin(t *testing.T, pluginPath string) {
@@ -213,21 +216,54 @@ func testPlugin(t *testing.T, pluginPath string) {
 			},
 		})
 	})
-	t.Run("configure success", func(t *testing.T) {
+	t.Run("configure from fixed success", func(t *testing.T) {
 		testLoad(t, pluginPath, loadTest{
 			registerConfigService: true,
 			mutateConfig: func(config *catalog.Config) {
-				config.PluginConfigs[0].Data = "GOOD"
+				config.PluginConfigs[0].DataSource = catalog.FixedData("GOOD")
 			},
 			expectPluginClient:  true,
 			expectServiceClient: true,
 		})
 	})
+	t.Run("configure and reconfigure from file success", func(t *testing.T) {
+		configPath := filepath.Join(spiretest.TempDir(t), "plugin.conf")
+		require.NoError(t, os.WriteFile(configPath, []byte("GOOD1"), 0600))
+
+		testLoad(t, pluginPath, loadTest{
+			registerConfigService: true,
+			mutateConfig: func(config *catalog.Config) {
+				config.PluginConfigs[0].DataSource = catalog.FileData(configPath)
+			},
+			expectPluginClient:  true,
+			expectServiceClient: true,
+			expectLogEntries: []spiretest.LogEntry{
+				{
+					Level:   logrus.InfoLevel,
+					Message: "CONFIGURED",
+					Data: logrus.Fields{
+						"config": "GOOD1",
+					},
+				},
+				{
+					Level:   logrus.InfoLevel,
+					Message: "CONFIGURED",
+					Data: logrus.Fields{
+						"config": "GOOD2",
+					},
+				},
+			},
+			epilogue: func(t *testing.T, cat *catalog.Catalog) {
+				require.NoError(t, os.WriteFile(configPath, []byte("GOOD2"), 0600))
+				cat.Reconfigure(context.Background())
+			},
+		})
+	})
 	t.Run("configure failure", func(t *testing.T) {
 		testLoad(t, pluginPath, loadTest{
 			registerConfigService: true,
 			mutateConfig: func(config *catalog.Config) {
-				config.PluginConfigs[0].Data = "BAD"
+				config.PluginConfigs[0].DataSource = catalog.FixedData("BAD")
 			},
 			expectErr: `failed to configure plugin "test": rpc error: code = InvalidArgument desc = bad config`,
 		})
@@ -235,7 +271,7 @@ func testPlugin(t *testing.T, pluginPath string) {
 	t.Run("configure interface not registered but data supplied", func(t *testing.T) {
 		testLoad(t, pluginPath, loadTest{
 			mutateConfig: func(config *catalog.Config) {
-				config.PluginConfigs[0].Data = "GOOD"
+				config.PluginConfigs[0].DataSource = catalog.FixedData("GOOD")
 			},
 			expectErr: `failed to configure plugin "test": no supported configuration interface found`,
 		})
@@ -355,26 +391,53 @@ func testLoad(t *testing.T, pluginPath string, tt loadTest) {
 		tt.mutateServiceRepo(serviceRepo)
 	}
 
-	closer, err := catalog.Load(context.Background(), config, repo)
-	if closer != nil {
+	cat, err := catalog.Load(context.Background(), config, repo)
+	if cat != nil {
 		defer func() {
-			closer.Close()
+			cat.Close()
+
+			wantEntries := slices.Clone(tt.expectLogEntries)
 			if tt.expectPluginClient {
 				// Assert that the plugin io.Closer was invoked by looking at
 				// the logs. It's hard to use the full log entry since there
 				// is a bunch of unrelated, per-test-run type stuff in there,
 				// so just inspect the log messages.
-				assertContainsLogMessage(t, hook.AllEntries(), "CLOSED")
+
+				wantEntries = append(wantEntries, spiretest.LogEntry{
+					Level:   logrus.InfoLevel,
+					Message: "CLOSED",
+				})
 			}
+
+			// Prune out data that isn't contained in the wanted entries.
+			// Otherwise, the tests get pretty coupled to the log fields, which
+			// isn't what these tests are particularly concerned with.
+			wantData := make(map[string]bool)
+			for _, wantEntry := range wantEntries {
+				for k := range wantEntry.Data {
+					wantData[k] = true
+				}
+			}
+			var allEntries []*logrus.Entry
+			for _, entry := range hook.AllEntries() {
+				// Only keep fields that are present in the wanted entries
+				for k := range entry.Data {
+					if !wantData[k] {
+						delete(entry.Data, k)
+					}
+				}
+				allEntries = append(allEntries, entry)
+			}
+			spiretest.AssertLogsContainEntries(t, allEntries, wantEntries)
 		}()
 	}
 
 	if tt.expectErr != "" {
 		require.ErrorContains(t, err, tt.expectErr, "load should have failed")
-		assert.Nil(t, closer, "closer should have been nil")
+		assert.Nil(t, cat, "catalog should have been nil")
 	} else {
 		require.NoError(t, err, "load should not have failed")
-		assert.NotNil(t, closer, "closer should not have been nil")
+		assert.NotNil(t, cat, "catalog should not have been nil")
 	}
 
 	if tt.expectPluginClient {
@@ -410,6 +473,10 @@ func testLoad(t *testing.T, pluginPath string, tt loadTest) {
 	} else {
 		assert.Nil(t, someService, "service client should not have been initialized")
 	}
+
+	if tt.epilogue != nil {
+		tt.epilogue(t, cat)
+	}
 }
 
 func buildTestPlugin(t *testing.T, srcPath string) string {
@@ -566,11 +633,3 @@ func (badFacade) GRPCServiceName() string                 { return "bad" }
 func (badFacade) InitClient(grpc.ClientConnInterface) any { return nil }
 func (badFacade) InitInfo(catalog.PluginInfo)             {}
 func (badFacade) InitLog(logrus.FieldLogger)              {}
-
-func assertContainsLogMessage(t *testing.T, entries []*logrus.Entry, message string) {
-	messages := make([]string, 0, len(entries))
-	for _, entry := range entries {
-		messages = append(messages, entry.Message)
-	}
-	assert.Contains(t, messages, message)
-}
