@@ -175,6 +175,12 @@ const (
 	// BySelectors tags selectors used when filtering
 	BySelectors = "by_selectors"
 
+	// CAJournal is a CA journal record
+	CAJournal = "ca_journal"
+
+	// CAJournalID tags a CA journal ID
+	CAJournalID = "ca_journal_id"
+
 	// CallerAddr labels an API caller address
 	CallerAddr = "caller_addr"
 
@@ -263,10 +269,10 @@ const (
 	// ElapsedTime tags some duration of time.
 	ElapsedTime = "elapsed_time"
 
-	// EntryAdded is the counter key for when a entry is added to LRU cache
+	// EntryAdded is the counter key for when an entry is added to LRU cache
 	EntryAdded = "lru_cache_entry_add"
 
-	// EntryRemoved is the counter key for when a entry is removed from LRU cache
+	// EntryRemoved is the counter key for when an entry is removed from LRU cache
 	EntryRemoved = "lru_cache_entry_remove"
 
 	// EntryUpdated is the counter key for when an LRU cache entry is updated
@@ -279,6 +285,9 @@ const (
 	// non-error level.
 	Error = "error"
 
+	// EventID tags an event ID
+	EventID = "event_id"
+
 	// Expect tags an expected value, as opposed to the one received. Message should clarify
 	// what kind of value was expected, and a different field should show the received value
 	Expect = "expect"
@@ -326,12 +335,18 @@ const (
 	// Generation represents an objection generation (i.e. version)
 	Generation = "generation"
 
+	// Hash tags a hash
+	Hash = "hash"
+
 	// Hint tags registration entry hint
 	Hint = "hint"
 
 	// IDType tags some type of ID (eg. registration ID, SPIFFE ID...)
 	IDType = "id_type"
 
+	// ImageID tags the image identifier in the format "repository@sha256:digest"
+	ImageID = "image_id"
+
 	// IssuedAt tags an issuance timestamp
 	IssuedAt = "issued_at"
 
@@ -341,9 +356,12 @@ const (
 	// JWTAuthorityExpiresAt tags a JWT Authority expiration
 	JWTAuthorityExpiresAt = "jwt_authority_expires_at"
 
-	// JWTAuthorityPublicKey tags a JWT authority key ID
+	// JWTAuthorityKeyID tags a JWT authority key ID
 	JWTAuthorityKeyID = "jwt_authority_key_id"
 
+	// JWTAuthorityKeyIDs tags a list of JWT authority key IDs
+	JWTAuthorityKeyIDs = "jwt_authority_key_ids"
+
 	// JWTAuthorityPublicKeySHA256 tags a JWT Authority public key
 	JWTAuthorityPublicKeySHA256 = "jwt_authority_public_key_sha256"
 
@@ -354,15 +372,24 @@ const (
 	// Kid tags some key ID
 	Kid = "kid"
 
+	// LaunchLogLevel log level when service started
+	LaunchLogLevel = "launch_log_level"
+
 	// LocalAuthorityID tags a local authority ID
 	LocalAuthorityID = "local_authority_id"
 
 	// Mode tags a bundle deletion mode
 	Mode = "mode"
 
+	// NewLogLevel tags a new log level
+	NewLogLevel = "new_log_level"
+
 	// Network tags some network name ("tcp", "udp")
 	Network = "network"
 
+	// NewHash tags a new hash
+	NewHash = "new_hash"
+
 	// NewSerialNumber tags a certificate new serial number
 	NewSerialNumber = "new_serial_num"
 
@@ -372,9 +399,15 @@ const (
 	// Nonce tags some nonce for communication
 	Nonce = "nonce"
 
+	// OldHash tags a hash
+	OldHash = "old_hash"
+
 	// ParentID tags parent ID for an entry
 	ParentID = "parent_id"
 
+	// PartialSelectors is a partial set of selectors for a workload
+	PartialSelectors = "partial_selectors"
+
 	// Path declares some logic path, likely on the file system
 	Path = "path"
 
@@ -412,7 +445,7 @@ const (
 	Reason = "reason"
 
 	// Reattestable declares if the agent should reattest when its SVID expires
-	Reattestable = "rettestable"
+	Reattestable = "reattestable"
 
 	// Received tags a received value, as opposed to the one that is expected. Message should clarify
 	// what kind of value was received, and a different field should show the expected value.
@@ -430,6 +463,9 @@ const (
 	// RecordMapSize is the gauge key to hold the size of the LRU cache entries map
 	RecordMapSize = "lru_cache_record_map_size"
 
+	// Reconfigurable tags whether something is reconfigurable.
+	Reconfigurable = "reconfigurable"
+
 	// RefreshHint tags a bundle refresh hint
 	RefreshHint = "refresh_hint"
 
@@ -443,7 +479,7 @@ const (
 	// RegistrationEntry tags a registration entry
 	RegistrationEntry = "registration_entry"
 
-	// RegistrationEntryEvent is a notice a registration entry has been create, modified, or deleted
+	// RegistrationEntryEvent is a notice a registration entry has been created, modified, or deleted
 	RegistrationEntryEvent = "registration_entry_event"
 
 	// RequestID tags a request identifier
@@ -480,7 +516,7 @@ const (
 	// SelectorsRemoved labels some count of selectors that have been removed from an entity
 	SelectorsRemoved = "selectors_removed"
 
-	// SelfSigned tags whether or not some entity is self-signed
+	// SelfSigned tags whether some entity is self-signed
 	SelfSigned = "self_signed"
 
 	// SendJWTBundleLatency tags latency for sending JWT bundle
@@ -511,6 +547,12 @@ const (
 	// with other tags to add clarity
 	Subject = "subject"
 
+	// SubjectKeyID tags a certificate subject key ID
+	SubjectKeyID = "subject_key_id"
+
+	// SubjectKeyIDs tags a list of subject key ID
+	SubjectKeyIDs = "subject_key_ids"
+
 	// SVIDMapSize is the gauge key for the size of the LRU cache SVID map
 	SVIDMapSize = "lru_cache_svid_map_size"
 
@@ -569,6 +611,9 @@ const (
 	// with other tags to add clarity
 	Updated = "updated"
 
+	// UpstreamAuthorityID tags a signing authority ID
+	UpstreamAuthorityID = "upstream_authority_id"
+
 	// StoreSvid tags if entry is storable
 	StoreSvid = "store_svid"
 
@@ -624,6 +669,24 @@ const (
 	// Cache functionality related to a cache
 	Cache = "cache"
 
+	// AgentsByIDCache functionality related to the agent btree cache indexed by ID
+	AgentsByIDCache = "agents_by_id_cache"
+
+	// AgentsByExpiresAtCache functionality related to the agent btree cache indexed by ExpiresAt
+	AgentsByExpiresAtCache = "agents_by_expiresat_cache"
+
+	// NodeAliasesByEntryIDCache functionality related to the node-aliases btree cache indexed by EntryID
+	NodeAliasesByEntryIDCache = "nodealiases_by_entryid_cache"
+
+	// NodeAliasesBySelectorCache functionality related to the node-aliases btree cache indexed by Selector
+	NodeAliasesBySelectorCache = "nodealiases_by_selector_cache"
+
+	// EntriesByEntryIDCache functionality related to the entries btree cache indexed by EntryID
+	EntriesByEntryIDCache = "entries_by_entryid_cache"
+
+	// EntriesByParentIDCache functionality related to the entries btree cache indexed by ParentID
+	EntriesByParentIDCache = "entries_by_parentid_cache"
+
 	// Cache type tag
 	CacheType = "cache_type"
 
@@ -720,6 +783,12 @@ const (
 	// RegistrationManager functionality related to a registration manager
 	RegistrationManager = "registration_manager"
 
+	// TaintedJWTSVIDs tags tainted JWT SVID count/list
+	TaintedJWTSVIDs = "tainted_jwt_svids"
+
+	// TaintedX509SVIDs tags tainted X.509 SVID count/list
+	TaintedX509SVIDs = "tainted_x509_svids"
+
 	// Telemetry tags a telemetry module
 	Telemetry = "telemetry"
 
@@ -822,8 +891,11 @@ const (
 	// ListAgents functionality related to listing agents
 	ListAgents = "list_agents"
 
-	// CountEntries functionality related to counting all registration entries
-	CountEntries = "count_entries"
+	// SkippedEntryEventIDs functionality related to counting missed entry event IDs
+	SkippedEntryEventIDs = "skipped_entry_event_ids"
+
+	// SkippedNodeEventIDs functionality related to counting missed node event IDs
+	SkippedNodeEventIDs = "skipped_node_event_ids"
 
 	// ListAllEntriesWithPages functionality related to listing all registration entries with pagination
 	ListAllEntriesWithPages = "list_all_entries_with_pages"
@@ -855,6 +927,12 @@ const (
 	// PushJWTKeyUpstream functionality related to pushing a public JWT Key to an upstream server.
 	PushJWTKeyUpstream = "push_jwtkey_upstream"
 
+	// ProcessTaintedX509SVIDs functionality related to processing tainted X.509 SVIDs.
+	ProcessTaintedX509SVIDs = "process_tainted_x509_svids"
+
+	// ProcessTaintedJWTSVIDs functionality related to processing tainted JWT SVIDs.
+	ProcessTaintedJWTSVIDs = "process_tainted_jwt_svids"
+
 	// SDSAPI functionality related to SDS; should be used with other tags
 	// to add clarity
 	SDSAPI = "sds_api"
