@@ -14,7 +14,6 @@ import (
 	agentv1 "github.com/spiffe/spire-api-sdk/proto/spire/api/server/agent/v1"
 	"github.com/spiffe/spire-api-sdk/proto/spire/api/types"
 	"github.com/spiffe/spire/pkg/common/errorutil"
-	"github.com/spiffe/spire/pkg/common/fflag"
 	"github.com/spiffe/spire/pkg/common/idutil"
 	"github.com/spiffe/spire/pkg/common/nodeutil"
 	"github.com/spiffe/spire/pkg/common/selector"
@@ -66,13 +65,49 @@ func New(config Config) *Service {
 }
 
 // RegisterService registers the agent service on the gRPC server/
-func RegisterService(s *grpc.Server, service *Service) {
+func RegisterService(s grpc.ServiceRegistrar, service *Service) {
 	agentv1.RegisterAgentServer(s, service)
 }
 
 // CountAgents returns the total number of agents.
-func (s *Service) CountAgents(ctx context.Context, _ *agentv1.CountAgentsRequest) (*agentv1.CountAgentsResponse, error) {
-	count, err := s.ds.CountAttestedNodes(ctx)
+func (s *Service) CountAgents(ctx context.Context, req *agentv1.CountAgentsRequest) (*agentv1.CountAgentsResponse, error) {
+	log := rpccontext.Logger(ctx)
+
+	countReq := &datastore.CountAttestedNodesRequest{}
+
+	// Parse proto filter into datastore request
+	if req.Filter != nil {
+		filter := req.Filter
+		rpccontext.AddRPCAuditFields(ctx, fieldsFromCountAgentsRequest(filter))
+
+		if filter.ByBanned != nil {
+			countReq.ByBanned = &req.Filter.ByBanned.Value
+		}
+		if filter.ByCanReattest != nil {
+			countReq.ByCanReattest = &req.Filter.ByCanReattest.Value
+		}
+
+		if filter.ByAttestationType != "" {
+			countReq.ByAttestationType = filter.ByAttestationType
+		}
+
+		if filter.ByExpiresBefore != "" {
+			countReq.ByExpiresBefore, _ = time.Parse("2006-01-02 15:04:05 -0700 -07", filter.ByExpiresBefore)
+		}
+
+		if filter.BySelectorMatch != nil {
+			selectors, err := api.SelectorsFromProto(filter.BySelectorMatch.Selectors)
+			if err != nil {
+				return nil, api.MakeErr(log, codes.InvalidArgument, "failed to parse selectors", err)
+			}
+			countReq.BySelectorMatch = &datastore.BySelectors{
+				Match:     datastore.MatchBehavior(filter.BySelectorMatch.Match),
+				Selectors: selectors,
+			}
+		}
+	}
+
+	count, err := s.ds.CountAttestedNodes(ctx, countReq)
 	if err != nil {
 		log := rpccontext.Logger(ctx)
 		return nil, api.MakeErr(log, codes.Internal, "failed to count agents", err)
@@ -94,20 +129,22 @@ func (s *Service) ListAgents(ctx context.Context, req *agentv1.ListAgentsRequest
 	// Parse proto filter into datastore request
 	if req.Filter != nil {
 		filter := req.Filter
-		rpccontext.AddRPCAuditFields(ctx, fieldsFromFilterRequest(filter))
+		rpccontext.AddRPCAuditFields(ctx, fieldsFromListAgentsRequest(filter))
 
-		var byBanned *bool
 		if filter.ByBanned != nil {
-			byBanned = &filter.ByBanned.Value
+			listReq.ByBanned = &req.Filter.ByBanned.Value
 		}
-		var byCanReattest *bool
 		if filter.ByCanReattest != nil {
-			byCanReattest = &filter.ByCanReattest.Value
+			listReq.ByCanReattest = &req.Filter.ByCanReattest.Value
 		}
 
-		listReq.ByAttestationType = filter.ByAttestationType
-		listReq.ByBanned = byBanned
-		listReq.ByCanReattest = byCanReattest
+		if filter.ByAttestationType != "" {
+			listReq.ByAttestationType = filter.ByAttestationType
+		}
+
+		if filter.ByExpiresBefore != "" {
+			listReq.ByExpiresBefore, _ = time.Parse("2006-01-02 15:04:05 -0700 -07", filter.ByExpiresBefore)
+		}
 
 		if filter.BySelectorMatch != nil {
 			selectors, err := api.SelectorsFromProto(filter.BySelectorMatch.Selectors)
@@ -397,8 +434,8 @@ func (s *Service) RenewAgent(ctx context.Context, req *agentv1.RenewAgentRequest
 	}
 
 	// Agent attempted to renew when it should've been reattesting
-	if attestedNode.CanReattest && fflag.IsSet(fflag.FlagReattestToRenew) {
-		return nil, errorutil.PermissionDenied(types.PermissionDeniedDetails_AGENT_MUST_REATTEST, "agent can't renew SVID, must reattest")
+	if attestedNode.CanReattest {
+		return nil, errorutil.PermissionDenied(types.PermissionDeniedDetails_AGENT_MUST_REATTEST, "agent must reattest instead of renew")
 	}
 
 	log.Info("Renewing agent SVID")
@@ -687,7 +724,30 @@ func getAttestAgentResponse(spiffeID spiffeid.ID, certificates []*x509.Certifica
 	}
 }
 
-func fieldsFromFilterRequest(filter *agentv1.ListAgentsRequest_Filter) logrus.Fields {
+func fieldsFromListAgentsRequest(filter *agentv1.ListAgentsRequest_Filter) logrus.Fields {
+	fields := logrus.Fields{}
+
+	if filter.ByAttestationType != "" {
+		fields[telemetry.NodeAttestorType] = filter.ByAttestationType
+	}
+
+	if filter.ByBanned != nil {
+		fields[telemetry.ByBanned] = filter.ByBanned.Value
+	}
+
+	if filter.ByCanReattest != nil {
+		fields[telemetry.ByCanReattest] = filter.ByCanReattest.Value
+	}
+
+	if filter.BySelectorMatch != nil {
+		fields[telemetry.BySelectorMatch] = filter.BySelectorMatch.Match.String()
+		fields[telemetry.BySelectors] = api.SelectorFieldFromProto(filter.BySelectorMatch.Selectors)
+	}
+
+	return fields
+}
+
+func fieldsFromCountAgentsRequest(filter *agentv1.CountAgentsRequest_Filter) logrus.Fields {
 	fields := logrus.Fields{}
 
 	if filter.ByAttestationType != "" {
