@@ -56,13 +56,75 @@ func New(config Config) *Service {
 }
 
 // RegisterService registers the entry service on the gRPC server.
-func RegisterService(s *grpc.Server, service *Service) {
+func RegisterService(s grpc.ServiceRegistrar, service *Service) {
 	entryv1.RegisterEntryServer(s, service)
 }
 
 // CountEntries returns the total number of entries.
-func (s *Service) CountEntries(ctx context.Context, _ *entryv1.CountEntriesRequest) (*entryv1.CountEntriesResponse, error) {
-	count, err := s.ds.CountRegistrationEntries(ctx)
+func (s *Service) CountEntries(ctx context.Context, req *entryv1.CountEntriesRequest) (*entryv1.CountEntriesResponse, error) {
+	log := rpccontext.Logger(ctx)
+	countReq := &datastore.CountRegistrationEntriesRequest{}
+
+	if req.Filter != nil {
+		rpccontext.AddRPCAuditFields(ctx, fieldsFromCountEntryFilter(ctx, s.td, req.Filter))
+		if req.Filter.ByHint != nil {
+			countReq.ByHint = req.Filter.ByHint.GetValue()
+		}
+
+		if req.Filter.ByParentId != nil {
+			parentID, err := api.TrustDomainMemberIDFromProto(ctx, s.td, req.Filter.ByParentId)
+			if err != nil {
+				return nil, api.MakeErr(log, codes.InvalidArgument, "malformed parent ID filter", err)
+			}
+			countReq.ByParentID = parentID.String()
+		}
+
+		if req.Filter.BySpiffeId != nil {
+			spiffeID, err := api.TrustDomainWorkloadIDFromProto(ctx, s.td, req.Filter.BySpiffeId)
+			if err != nil {
+				return nil, api.MakeErr(log, codes.InvalidArgument, "malformed SPIFFE ID filter", err)
+			}
+			countReq.BySpiffeID = spiffeID.String()
+		}
+
+		if req.Filter.BySelectors != nil {
+			dsSelectors, err := api.SelectorsFromProto(req.Filter.BySelectors.Selectors)
+			if err != nil {
+				return nil, api.MakeErr(log, codes.InvalidArgument, "malformed selectors filter", err)
+			}
+			if len(dsSelectors) == 0 {
+				return nil, api.MakeErr(log, codes.InvalidArgument, "malformed selectors filter", errors.New("empty selector set"))
+			}
+			countReq.BySelectors = &datastore.BySelectors{
+				Match:     datastore.MatchBehavior(req.Filter.BySelectors.Match),
+				Selectors: dsSelectors,
+			}
+		}
+
+		if req.Filter.ByFederatesWith != nil {
+			trustDomains := make([]string, 0, len(req.Filter.ByFederatesWith.TrustDomains))
+			for _, tdStr := range req.Filter.ByFederatesWith.TrustDomains {
+				td, err := spiffeid.TrustDomainFromString(tdStr)
+				if err != nil {
+					return nil, api.MakeErr(log, codes.InvalidArgument, "malformed federates with filter", err)
+				}
+				trustDomains = append(trustDomains, td.IDString())
+			}
+			if len(trustDomains) == 0 {
+				return nil, api.MakeErr(log, codes.InvalidArgument, "malformed federates with filter", errors.New("empty trust domain set"))
+			}
+			countReq.ByFederatesWith = &datastore.ByFederatesWith{
+				Match:        datastore.MatchBehavior(req.Filter.ByFederatesWith.Match),
+				TrustDomains: trustDomains,
+			}
+		}
+
+		if req.Filter.ByDownstream != nil {
+			countReq.ByDownstream = &req.Filter.ByDownstream.Value
+		}
+	}
+
+	count, err := s.ds.CountRegistrationEntries(ctx, countReq)
 	if err != nil {
 		log := rpccontext.Logger(ctx)
 		return nil, api.MakeErr(log, codes.Internal, "failed to count entries", err)
@@ -139,6 +201,10 @@ func (s *Service) ListEntries(ctx context.Context, req *entryv1.ListEntriesReque
 				TrustDomains: trustDomains,
 			}
 		}
+
+		if req.Filter.ByDownstream != nil {
+			listReq.ByDownstream = &req.Filter.ByDownstream.Value
+		}
 	}
 
 	dsResp, err := s.ds.ListRegistrationEntries(ctx, listReq)
@@ -225,8 +291,12 @@ func (s *Service) createEntry(ctx context.Context, e *types.Entry, outputMask *t
 	regEntry, existing, err := s.ds.CreateOrReturnRegistrationEntry(ctx, cEntry)
 	switch {
 	case err != nil:
+		statusCode := status.Code(err)
+		if statusCode == codes.Unknown {
+			statusCode = codes.Internal
+		}
 		return &entryv1.BatchCreateEntryResponse_Result{
-			Status: api.MakeStatus(log, codes.Internal, "failed to create entry", err),
+			Status: api.MakeStatus(log, statusCode, "failed to create entry", err),
 		}
 	case existing:
 		resultStatus = api.CreateStatus(codes.AlreadyExists, "similar entry already exists")
@@ -445,7 +515,7 @@ func SyncAuthorizedEntries(stream entryv1.Entry_SyncAuthorizedEntriesServer, ent
 		}
 
 		// Sort the requested IDs for efficient lookups into the sorted entry
-		// list. Agents SHOULD already send the list sorted but we need to
+		// list. Agents SHOULD already send the list sorted, but we need to
 		// make sure they are sorted for correctness of the search loop below.
 		// The go stdlib sorting algorithm performs well on pre-sorted data.
 		slices.Sort(req.Ids)
@@ -594,8 +664,12 @@ func (s *Service) updateEntry(ctx context.Context, e *types.Entry, inputMask *ty
 	}
 	dsEntry, err := s.ds.UpdateRegistrationEntry(ctx, convEntry, mask)
 	if err != nil {
+		statusCode := status.Code(err)
+		if statusCode == codes.Unknown {
+			statusCode = codes.Internal
+		}
 		return &entryv1.BatchUpdateEntryResponse_Result{
-			Status: api.MakeStatus(log, codes.Internal, "failed to update entry", err),
+			Status: api.MakeStatus(log, statusCode, "failed to update entry", err),
 		}
 	}
 
@@ -725,6 +799,46 @@ func fieldsFromListEntryFilter(ctx context.Context, td spiffeid.TrustDomain, fil
 		fields[telemetry.FederatesWith] = strings.Join(filter.ByFederatesWith.TrustDomains, ",")
 	}
 
+	if filter.ByDownstream != nil {
+		fields[telemetry.Downstream] = &filter.ByDownstream.Value
+	}
+
+	return fields
+}
+
+func fieldsFromCountEntryFilter(ctx context.Context, td spiffeid.TrustDomain, filter *entryv1.CountEntriesRequest_Filter) logrus.Fields {
+	fields := logrus.Fields{}
+
+	if filter.ByHint != nil {
+		fields[telemetry.Hint] = filter.ByHint.Value
+	}
+
+	if filter.ByParentId != nil {
+		if parentID, err := api.TrustDomainMemberIDFromProto(ctx, td, filter.ByParentId); err == nil {
+			fields[telemetry.ParentID] = parentID.String()
+		}
+	}
+
+	if filter.BySpiffeId != nil {
+		if id, err := api.TrustDomainWorkloadIDFromProto(ctx, td, filter.BySpiffeId); err == nil {
+			fields[telemetry.SPIFFEID] = id.String()
+		}
+	}
+
+	if filter.BySelectors != nil {
+		fields[telemetry.BySelectorMatch] = filter.BySelectors.Match.String()
+		fields[telemetry.BySelectors] = api.SelectorFieldFromProto(filter.BySelectors.Selectors)
+	}
+
+	if filter.ByFederatesWith != nil {
+		fields[telemetry.FederatesWithMatch] = filter.ByFederatesWith.Match.String()
+		fields[telemetry.FederatesWith] = strings.Join(filter.ByFederatesWith.TrustDomains, ",")
+	}
+
+	if filter.ByDownstream != nil {
+		fields[telemetry.Downstream] = &filter.ByDownstream.Value
+	}
+
 	return fields
 }
 
