@@ -10,6 +10,7 @@ import (
 	"github.com/spiffe/go-spiffe/v2/spiffeid"
 	svidv1 "github.com/spiffe/spire-api-sdk/proto/spire/api/server/svid/v1"
 	"github.com/spiffe/spire-api-sdk/proto/spire/api/types"
+	"github.com/spiffe/spire/pkg/common/idutil"
 	"github.com/spiffe/spire/pkg/common/jwtsvid"
 	"github.com/spiffe/spire/pkg/common/telemetry"
 	"github.com/spiffe/spire/pkg/common/x509util"
@@ -23,25 +24,27 @@ import (
 )
 
 // RegisterService registers the service on the gRPC server.
-func RegisterService(s *grpc.Server, service *Service) {
+func RegisterService(s grpc.ServiceRegistrar, service *Service) {
 	svidv1.RegisterSVIDServer(s, service)
 }
 
 // Config is the service configuration
 type Config struct {
-	EntryFetcher api.AuthorizedEntryFetcher
-	ServerCA     ca.ServerCA
-	TrustDomain  spiffeid.TrustDomain
-	DataStore    datastore.DataStore
+	EntryFetcher                 api.AuthorizedEntryFetcher
+	ServerCA                     ca.ServerCA
+	TrustDomain                  spiffeid.TrustDomain
+	DataStore                    datastore.DataStore
+	UseLegacyDownstreamX509CATTL bool
 }
 
 // New creates a new SVID service
 func New(config Config) *Service {
 	return &Service{
-		ca: config.ServerCA,
-		ef: config.EntryFetcher,
-		td: config.TrustDomain,
-		ds: config.DataStore,
+		ca:                           config.ServerCA,
+		ef:                           config.EntryFetcher,
+		td:                           config.TrustDomain,
+		ds:                           config.DataStore,
+		useLegacyDownstreamX509CATTL: config.UseLegacyDownstreamX509CATTL,
 	}
 }
 
@@ -49,10 +52,11 @@ func New(config Config) *Service {
 type Service struct {
 	svidv1.UnsafeSVIDServer
 
-	ca ca.ServerCA
-	ef api.AuthorizedEntryFetcher
-	td spiffeid.TrustDomain
-	ds datastore.DataStore
+	ca                           ca.ServerCA
+	ef                           api.AuthorizedEntryFetcher
+	td                           spiffeid.TrustDomain
+	ds                           datastore.DataStore
+	useLegacyDownstreamX509CATTL bool
 }
 
 func (s *Service) MintX509SVID(ctx context.Context, req *svidv1.MintX509SVIDRequest) (*svidv1.MintX509SVIDResponse, error) {
@@ -91,17 +95,24 @@ func (s *Service) MintX509SVID(ctx context.Context, req *svidv1.MintX509SVIDRequ
 		return nil, api.MakeErr(log, codes.InvalidArgument, "CSR URI SAN is invalid", err)
 	}
 
+	dnsNames := make([]string, 0, len(csr.DNSNames))
 	for _, dnsName := range csr.DNSNames {
-		if err := x509util.ValidateDNS(dnsName); err != nil {
+		err := x509util.ValidateLabel(dnsName)
+		if err != nil {
 			return nil, api.MakeErr(log, codes.InvalidArgument, "CSR DNS name is invalid", err)
 		}
+		dnsNames = append(dnsNames, dnsName)
+	}
+
+	if err := x509util.CheckForWildcardOverlap(dnsNames); err != nil {
+		return nil, api.MakeErr(log, codes.InvalidArgument, "CSR DNS name contains a wildcard that covers another non-wildcard name", err)
 	}
 
 	x509SVID, err := s.ca.SignWorkloadX509SVID(ctx, ca.WorkloadX509SVIDParams{
 		SPIFFEID:  id,
 		PublicKey: csr.PublicKey,
 		TTL:       time.Duration(req.Ttl) * time.Second,
-		DNSNames:  csr.DNSNames,
+		DNSNames:  dnsNames,
 		Subject:   csr.Subject,
 	})
 	if err != nil {
@@ -168,10 +179,19 @@ func (s *Service) BatchNewX509SVID(ctx context.Context, req *svidv1.BatchNewX509
 		//  Create new SVID
 		r := s.newX509SVID(ctx, svidParam, entriesMap)
 		results = append(results, r)
+		spiffeID := ""
+		if r.Svid != nil {
+			id, err := idutil.IDProtoString(r.Svid.Id)
+			if err == nil {
+				spiffeID = id
+			}
+		}
+
 		rpccontext.AuditRPCWithTypesStatus(ctx, r.Status, func() logrus.Fields {
 			fields := logrus.Fields{
 				telemetry.Csr:            api.HashByte(svidParam.Csr),
 				telemetry.RegistrationID: svidParam.EntryId,
+				telemetry.SPIFFEID:       spiffeID,
 			}
 
 			if r.Svid != nil {
@@ -377,9 +397,20 @@ func (s *Service) NewDownstreamX509CA(ctx context.Context, req *svidv1.NewDownst
 		return nil, err
 	}
 
+	// Use the TTL offered by the downstream server (if any), unless we are
+	// configured to use the legacy TTL.
+	ttl := req.PreferredTtl
+	if s.useLegacyDownstreamX509CATTL {
+		// Legacy downstream TTL prefers the downstream workload entry
+		// TTL (if any) and then the default workload TTL. We'll handle the
+		// latter inside of the credbuilder package, which already has
+		// knowledge of the default.
+		ttl = entry.X509SvidTtl
+	}
+
 	x509CASvid, err := s.ca.SignDownstreamX509CA(ctx, ca.DownstreamX509CAParams{
 		PublicKey: csr.PublicKey,
-		TTL:       time.Duration(entry.X509SvidTtl) * time.Second,
+		TTL:       time.Duration(ttl) * time.Second,
 	})
 	if err != nil {
 		return nil, api.MakeErr(log, codes.Internal, "failed to sign downstream X.509 CA", err)
@@ -418,7 +449,7 @@ func (s Service) fieldsFromJWTSvidParams(ctx context.Context, protoID *types.SPI
 		telemetry.TTL: ttl,
 	}
 	if protoID != nil {
-		// Dont care about parsing error
+		// Don't care about parsing error
 		id, err := api.TrustDomainWorkloadIDFromProto(ctx, s.td, protoID)
 		if err == nil {
 			fields[telemetry.SPIFFEID] = id.String()
