@@ -19,8 +19,8 @@ import (
 	svidv1 "github.com/spiffe/spire-api-sdk/proto/spire/api/server/svid/v1"
 	"github.com/spiffe/spire-api-sdk/proto/spire/api/types"
 	"github.com/spiffe/spire/pkg/common/idutil"
+	"github.com/spiffe/spire/pkg/common/jwtsvid"
 	"github.com/spiffe/spire/pkg/common/telemetry"
-	"github.com/spiffe/spire/pkg/common/x509svid"
 	"github.com/spiffe/spire/pkg/common/x509util"
 	"github.com/spiffe/spire/pkg/server/api"
 	"github.com/spiffe/spire/pkg/server/api/middleware"
@@ -30,11 +30,12 @@ import (
 	"github.com/spiffe/spire/proto/spire/common"
 	"github.com/spiffe/spire/test/fakes/fakedatastore"
 	"github.com/spiffe/spire/test/fakes/fakeserverca"
+	"github.com/spiffe/spire/test/grpctest"
 	"github.com/spiffe/spire/test/spiretest"
 	"github.com/spiffe/spire/test/testkey"
 	"github.com/stretchr/testify/require"
 
-	"github.com/go-jose/go-jose/v3/jwt"
+	"github.com/go-jose/go-jose/v4/jwt"
 	"google.golang.org/grpc"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
@@ -76,7 +77,7 @@ func TestServiceMintX509SVID(t *testing.T) {
 				URIs: []*url.URL{workloadID.URL()},
 			},
 			expiredAt: expiredAt,
-			subject:   "O=SPIRE,C=US,2.5.4.45=#13203835323763353230323837636461376436323561613834373664386538336561",
+			subject:   "O=SPIRE,C=US",
 			expectLogs: func(csr []byte) []spiretest.LogEntry {
 				return []spiretest.LogEntry{
 					{
@@ -102,7 +103,7 @@ func TestServiceMintX509SVID(t *testing.T) {
 				URIs: []*url.URL{workloadID.URL()},
 			},
 			expiredAt: customExpiresAt,
-			subject:   "O=SPIRE,C=US,2.5.4.45=#13203835323763353230323837636461376436323561613834373664386538336561",
+			subject:   "O=SPIRE,C=US",
 			ttl:       10 * time.Second,
 			expectLogs: func(csr []byte) []spiretest.LogEntry {
 				return []spiretest.LogEntry{
@@ -131,7 +132,7 @@ func TestServiceMintX509SVID(t *testing.T) {
 			},
 			dns:       []string{"dns1", "dns2"},
 			expiredAt: expiredAt,
-			subject:   "CN=dns1,O=SPIRE,C=US,2.5.4.45=#13203835323763353230323837636461376436323561613834373664386538336561",
+			subject:   "CN=dns1,O=SPIRE,C=US",
 			expectLogs: func(csr []byte) []spiretest.LogEntry {
 				return []spiretest.LogEntry{
 					{
@@ -161,7 +162,7 @@ func TestServiceMintX509SVID(t *testing.T) {
 				},
 			},
 			expiredAt: expiredAt,
-			subject:   "O=ORG,C=EN+C=US,2.5.4.45=#13203835323763353230323837636461376436323561613834373664386538336561",
+			subject:   "O=ORG,C=EN+C=US",
 			expectLogs: func(csr []byte) []spiretest.LogEntry {
 				return []spiretest.LogEntry{
 					{
@@ -193,7 +194,7 @@ func TestServiceMintX509SVID(t *testing.T) {
 			},
 			dns:       []string{"dns1", "dns2"},
 			expiredAt: expiredAt,
-			subject:   "CN=dns1,O=ORG,C=EN+C=US,2.5.4.45=#13203835323763353230323837636461376436323561613834373664386538336561",
+			subject:   "CN=dns1,O=ORG,C=EN+C=US",
 			expectLogs: func(csr []byte) []spiretest.LogEntry {
 				return []spiretest.LogEntry{
 					{
@@ -469,14 +470,14 @@ func TestServiceMintX509SVID(t *testing.T) {
 				DNSNames: []string{"abc-"},
 			},
 			code: codes.InvalidArgument,
-			err:  "CSR DNS name is invalid: label does not match regex: abc-",
+			err:  "CSR DNS name is invalid: idna error",
 			expectLogs: func(csr []byte) []spiretest.LogEntry {
 				return []spiretest.LogEntry{
 					{
 						Level:   logrus.ErrorLevel,
 						Message: "Invalid argument: CSR DNS name is invalid",
 						Data: logrus.Fields{
-							logrus.ErrorKey: "label does not match regex: abc-",
+							logrus.ErrorKey: "idna error\nidna: invalid label \"abc-\"",
 						},
 					},
 					{
@@ -486,7 +487,7 @@ func TestServiceMintX509SVID(t *testing.T) {
 							telemetry.Status:        "error",
 							telemetry.Type:          "audit",
 							telemetry.StatusCode:    "InvalidArgument",
-							telemetry.StatusMessage: "CSR DNS name is invalid: label does not match regex: abc-",
+							telemetry.StatusMessage: "CSR DNS name is invalid: idna error\nidna: invalid label \"abc-\"",
 							telemetry.Csr:           api.HashByte(csr),
 							telemetry.TTL:           "0",
 						},
@@ -1213,6 +1214,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.RegistrationID: "workload",
 							telemetry.Csr:            api.HashByte(m["workload"]),
 							telemetry.ExpiresAt:      expiresAtFromCAStr,
+							telemetry.SPIFFEID:       "spiffe://example.org/workload1",
 						},
 					},
 				}
@@ -1236,6 +1238,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.RegistrationID: "ttl",
 							telemetry.Csr:            api.HashByte(m["ttl"]),
 							telemetry.ExpiresAt:      expiresAtFromTTLEntryStr,
+							telemetry.SPIFFEID:       "spiffe://example.org/ttl",
 						},
 					},
 				}
@@ -1259,6 +1262,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.RegistrationID: "x509ttl",
 							telemetry.Csr:            api.HashByte(m["x509ttl"]),
 							telemetry.ExpiresAt:      expiresAtFromX509TTLEntryStr,
+							telemetry.SPIFFEID:       "spiffe://example.org/ttl",
 						},
 					},
 				}
@@ -1282,6 +1286,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.RegistrationID: "dns",
 							telemetry.Csr:            api.HashByte(m["dns"]),
 							telemetry.ExpiresAt:      expiresAtFromCAStr,
+							telemetry.SPIFFEID:       "spiffe://example.org/dns",
 						},
 					},
 				}
@@ -1314,6 +1319,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.RegistrationID: "workload",
 							telemetry.Csr:            api.HashByte(m["workload"]),
 							telemetry.ExpiresAt:      expiresAtFromCAStr,
+							telemetry.SPIFFEID:       "spiffe://example.org/workload1",
 						},
 					},
 					{
@@ -1334,6 +1340,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.Csr:            api.HashByte(m["invalid"]),
 							telemetry.StatusCode:     "Internal",
 							telemetry.StatusMessage:  "entry has malformed SPIFFE ID: request must specify SPIFFE ID",
+							telemetry.SPIFFEID:       "",
 						},
 					},
 					{
@@ -1345,6 +1352,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.RegistrationID: "dns",
 							telemetry.Csr:            api.HashByte(m["dns"]),
 							telemetry.ExpiresAt:      expiresAtFromCAStr,
+							telemetry.SPIFFEID:       "spiffe://example.org/dns",
 						},
 					},
 				}
@@ -1477,6 +1485,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.Csr:            api.HashByte(m[""]),
 							telemetry.StatusCode:     "InvalidArgument",
 							telemetry.StatusMessage:  "missing entry ID",
+							telemetry.SPIFFEID:       "",
 						},
 					},
 				}
@@ -1511,6 +1520,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.Csr:            "",
 							telemetry.StatusCode:     "InvalidArgument",
 							telemetry.StatusMessage:  "missing CSR",
+							telemetry.SPIFFEID:       "",
 						},
 					},
 				}
@@ -1545,6 +1555,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.Csr:            api.HashByte(m["invalid entry"]),
 							telemetry.StatusCode:     "NotFound",
 							telemetry.StatusMessage:  "entry not found or not authorized",
+							telemetry.SPIFFEID:       "",
 						},
 					},
 				}
@@ -1583,6 +1594,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.Csr:            api.HashByte(m["workload"]),
 							telemetry.StatusCode:     "InvalidArgument",
 							telemetry.StatusMessage:  fmt.Sprintf("malformed CSR: %v", invalidCsrErr),
+							telemetry.SPIFFEID:       "",
 						},
 					},
 				}
@@ -1624,6 +1636,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.Csr:            api.HashByte(m["workload"]),
 							telemetry.StatusCode:     "InvalidArgument",
 							telemetry.StatusMessage:  "invalid CSR signature: x509: ECDSA verification failure",
+							telemetry.SPIFFEID:       "",
 						},
 					},
 				}
@@ -1659,6 +1672,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.Csr:            api.HashByte(m["invalid"]),
 							telemetry.StatusCode:     "Internal",
 							telemetry.StatusMessage:  "entry has malformed SPIFFE ID: request must specify SPIFFE ID",
+							telemetry.SPIFFEID:       "",
 						},
 					},
 				}
@@ -1696,6 +1710,7 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 							telemetry.Csr:            api.HashByte(m["workload"]),
 							telemetry.StatusCode:     "Internal",
 							telemetry.StatusMessage:  "failed to sign X509-SVID: oh no",
+							telemetry.SPIFFEID:       "",
 						},
 					},
 				}
@@ -1791,9 +1806,6 @@ func TestServiceBatchNewX509SVID(t *testing.T) {
 				expectedSubject := &pkix.Name{
 					Organization: []string{"SPIRE"},
 					Country:      []string{"US"},
-					Names: []pkix.AttributeTypeAndValue{
-						x509svid.UniqueIDAttribute(entrySPIFFEID),
-					},
 				}
 				if len(entry.DnsNames) > 0 {
 					expectedSubject.CommonName = entry.DnsNames[0]
@@ -1832,7 +1844,7 @@ func TestNewDownstreamX509CA(t *testing.T) {
 	_, csrErr := x509.ParseCertificateRequest([]byte{1, 2, 3})
 
 	now := test.ca.Clock().Now().UTC()
-	expiresAtFromCA := now.Add(test.ca.X509SVIDTTL()).Unix()
+	expiresAtFromCA := now.Add(test.ca.X509CATTL()).Unix()
 
 	for _, tt := range []downstreamCaTest{
 		{
@@ -2088,10 +2100,6 @@ func setupServiceTest(t *testing.T) *serviceTest {
 	})
 
 	log, logHook := test.NewNullLogger()
-	registerFn := func(s *grpc.Server) {
-		svid.RegisterService(s, service)
-	}
-
 	test := &serviceTest{
 		ca:          ca,
 		ef:          ef,
@@ -2100,8 +2108,7 @@ func setupServiceTest(t *testing.T) *serviceTest {
 		logHook:     logHook,
 		rateLimiter: rateLimiter,
 	}
-
-	ppMiddleware := middleware.Preprocess(func(ctx context.Context, fullMethod string, req any) (context.Context, error) {
+	overrideContext := func(ctx context.Context) context.Context {
 		ctx = rpccontext.WithLogger(ctx, log)
 		ctx = rpccontext.WithRateLimiter(ctx, rateLimiter)
 		if test.withCallerID {
@@ -2110,23 +2117,20 @@ func setupServiceTest(t *testing.T) *serviceTest {
 		if test.downstream.entries != nil {
 			ctx = rpccontext.WithCallerDownstreamEntries(ctx, downstream.entries)
 		}
-		return ctx, nil
-	})
+		return ctx
+	}
 
-	unaryInterceptor, streamInterceptor := middleware.Interceptors(middleware.Chain(
-		ppMiddleware,
-		// Add audit log with local tracking disabled
-		middleware.WithAuditLog(false),
-	))
-	server := grpc.NewServer(
-		grpc.UnaryInterceptor(unaryInterceptor),
-		grpc.StreamInterceptor(streamInterceptor),
+	server := grpctest.StartServer(t, func(s grpc.ServiceRegistrar) {
+		svid.RegisterService(s, service)
+	},
+		grpctest.OverrideContext(overrideContext),
+		grpctest.Middleware(middleware.WithAuditLog(false)),
 	)
 
-	// Set create client and add to test
-	conn, done := spiretest.NewAPIServerWithMiddleware(t, registerFn, server)
+	conn := server.Dial(t)
+
 	test.client = svidv1.NewSVIDClient(conn)
-	test.done = done
+	test.done = server.Stop
 
 	return test
 }
@@ -2137,18 +2141,18 @@ func createCSR(tb testing.TB, template *x509.CertificateRequest) []byte {
 	return csr
 }
 
-func verifyJWTSVIDResponse(t *testing.T, jwtsvid *types.JWTSVID, id spiffeid.ID, audience []string, issuedAt, expiresAt, defaultExpiresAt time.Time, ttl time.Duration) {
-	require.NotNil(t, jwtsvid)
-	require.NotEmpty(t, jwtsvid.Token)
+func verifyJWTSVIDResponse(t *testing.T, svid *types.JWTSVID, id spiffeid.ID, audience []string, issuedAt, expiresAt, defaultExpiresAt time.Time, ttl time.Duration) {
+	require.NotNil(t, svid)
+	require.NotEmpty(t, svid.Token)
 
-	token, err := jwt.ParseSigned(jwtsvid.Token)
+	token, err := jwt.ParseSigned(svid.Token, jwtsvid.AllowedSignatureAlgorithms)
 	require.NoError(t, err)
 
 	var claims jwt.Claims
 	err = token.UnsafeClaimsWithoutVerification(&claims)
 	require.NoError(t, err)
 
-	jwtsvidID, err := api.TrustDomainWorkloadIDFromProto(context.Background(), td, jwtsvid.Id)
+	jwtsvidID, err := api.TrustDomainWorkloadIDFromProto(context.Background(), td, svid.Id)
 	require.NoError(t, err)
 	require.Equal(t, id, jwtsvidID)
 	require.Equal(t, id.String(), claims.Subject)
@@ -2156,15 +2160,15 @@ func verifyJWTSVIDResponse(t *testing.T, jwtsvid *types.JWTSVID, id spiffeid.ID,
 	require.Equal(t, jwt.Audience(audience), claims.Audience)
 
 	require.NotNil(t, claims.IssuedAt)
-	require.Equal(t, issuedAt.Unix(), jwtsvid.IssuedAt)
+	require.Equal(t, issuedAt.Unix(), svid.IssuedAt)
 	require.Equal(t, issuedAt.Unix(), int64(*claims.IssuedAt))
 
 	require.NotNil(t, claims.Expiry)
 	if ttl == 0 {
-		require.Equal(t, defaultExpiresAt.Unix(), jwtsvid.ExpiresAt)
+		require.Equal(t, defaultExpiresAt.Unix(), svid.ExpiresAt)
 		require.Equal(t, defaultExpiresAt.Unix(), int64(*claims.Expiry))
 	} else {
-		require.Equal(t, expiresAt.Unix(), jwtsvid.ExpiresAt)
+		require.Equal(t, expiresAt.Unix(), svid.ExpiresAt)
 		require.Equal(t, expiresAt.Unix(), int64(*claims.Expiry))
 	}
 }
