@@ -6,6 +6,7 @@ import (
 	"crypto"
 	"crypto/rand"
 	"crypto/x509"
+	"errors"
 	"fmt"
 	"sync"
 	"time"
@@ -13,6 +14,8 @@ import (
 	"github.com/andres-erbsen/clock"
 	"github.com/sirupsen/logrus"
 	"github.com/spiffe/go-spiffe/v2/spiffeid"
+	"github.com/spiffe/spire/pkg/common/backoff"
+	"github.com/spiffe/spire/pkg/common/coretypes/x509certificate"
 	"github.com/spiffe/spire/pkg/common/telemetry"
 	telemetry_server "github.com/spiffe/spire/pkg/common/telemetry/server"
 	"github.com/spiffe/spire/pkg/common/x509util"
@@ -31,8 +34,9 @@ import (
 )
 
 const (
-	publishJWKTimeout = 5 * time.Second
-	safetyThreshold   = 24 * time.Hour
+	publishJWKTimeout         = 5 * time.Second
+	safetyThresholdBundle     = 24 * time.Hour
+	safetyThresholdCAJournals = time.Hour * 24 * 14 // Two weeks
 
 	thirtyDays                  = 30 * 24 * time.Hour
 	preparationThresholdCap     = thirtyDays
@@ -41,17 +45,35 @@ const (
 	sevenDays                  = 7 * 24 * time.Hour
 	activationThresholdCap     = sevenDays
 	activationThresholdDivisor = 6
+
+	taintBackoffInterval       = 5 * time.Second
+	taintBackoffMaxElapsedTime = 1 * time.Minute
 )
 
 type ManagedCA interface {
 	SetX509CA(*ca.X509CA)
 	SetJWTKey(*ca.JWTKey)
+	NotifyTaintedX509Authorities([]*x509.Certificate)
 }
 
 type JwtKeyPublisher interface {
 	PublishJWTKey(ctx context.Context, jwtKey *common.PublicKey) ([]*common.PublicKey, error)
 }
 
+type AuthorityManager interface {
+	GetCurrentJWTKeySlot() Slot
+	GetNextJWTKeySlot() Slot
+	PrepareJWTKey(ctx context.Context) error
+	RotateJWTKey(ctx context.Context)
+	GetCurrentX509CASlot() Slot
+	GetNextX509CASlot() Slot
+	PrepareX509CA(ctx context.Context) error
+	RotateX509CA(ctx context.Context)
+	IsUpstreamAuthority() bool
+	PublishJWTKey(ctx context.Context, jwtKey *common.PublicKey) ([]*common.PublicKey, error)
+	NotifyTaintedX509Authority(ctx context.Context, authorityID string) error
+}
+
 type Config struct {
 	CredBuilder   *credtemplate.Builder
 	CredValidator *credvalidator.Validator
@@ -67,24 +89,28 @@ type Config struct {
 }
 
 type Manager struct {
-	c                  Config
-	caTTL              time.Duration
-	bundleUpdatedCh    chan struct{}
-	upstreamClient     *ca.UpstreamClient
-	upstreamPluginName string
+	c                            Config
+	caTTL                        time.Duration
+	bundleUpdatedCh              chan struct{}
+	taintedUpstreamAuthoritiesCh chan []*x509.Certificate
+	upstreamClient               *ca.UpstreamClient
+	upstreamPluginName           string
 
-	currentX509CA *X509CASlot
-	nextX509CA    *X509CASlot
+	currentX509CA *x509CASlot
+	nextX509CA    *x509CASlot
 	x509CAMutex   sync.RWMutex
 
-	currentJWTKey *JwtKeySlot
-	nextJWTKey    *JwtKeySlot
+	currentJWTKey *jwtKeySlot
+	nextJWTKey    *jwtKeySlot
 	jwtKeyMutex   sync.RWMutex
 
 	journal *Journal
 
 	// Used to log a warning only once when the UpstreamAuthority does not support JWT-SVIDs.
 	jwtUnimplementedWarnOnce sync.Once
+
+	// Used for testing backoff, must not be set in regular code
+	triggerBackOffCh chan error
 }
 
 func NewManager(ctx context.Context, c Config) (*Manager, error) {
@@ -93,19 +119,22 @@ func NewManager(ctx context.Context, c Config) (*Manager, error) {
 	}
 
 	m := &Manager{
-		c:               c,
-		caTTL:           c.CredBuilder.Config().X509CATTL,
-		bundleUpdatedCh: make(chan struct{}, 1),
+		c:                            c,
+		caTTL:                        c.CredBuilder.Config().X509CATTL,
+		bundleUpdatedCh:              make(chan struct{}, 1),
+		taintedUpstreamAuthoritiesCh: make(chan []*x509.Certificate, 1),
 	}
 
 	if upstreamAuthority, ok := c.Catalog.GetUpstreamAuthority(); ok {
 		m.upstreamClient = ca.NewUpstreamClient(ca.UpstreamClientConfig{
 			UpstreamAuthority: upstreamAuthority,
 			BundleUpdater: &bundleUpdater{
-				log:           c.Log,
-				trustDomainID: c.TrustDomain.IDString(),
-				ds:            c.Catalog.GetDataStore(),
-				updated:       m.bundleUpdated,
+				log:                         c.Log,
+				trustDomainID:               c.TrustDomain.IDString(),
+				ds:                          c.Catalog.GetDataStore(),
+				updated:                     m.bundleUpdated,
+				upstreamAuthoritiesTainted:  m.notifyUpstreamAuthoritiesTainted,
+				processedTaintedAuthorities: map[string]struct{}{},
 			},
 		})
 		m.upstreamPluginName = upstreamAuthority.Name()
@@ -119,7 +148,7 @@ func NewManager(ctx context.Context, c Config) (*Manager, error) {
 		UpstreamClient: m.upstreamClient,
 	}
 
-	journal, slots, err := loader.Load(ctx)
+	journal, slots, err := loader.load(ctx)
 	if err != nil {
 		return nil, err
 	}
@@ -127,21 +156,21 @@ func NewManager(ctx context.Context, c Config) (*Manager, error) {
 	now := m.c.Clock.Now()
 	m.journal = journal
 	if currentX509CA, ok := slots[CurrentX509CASlot]; ok {
-		m.currentX509CA = currentX509CA.(*X509CASlot)
+		m.currentX509CA = currentX509CA.(*x509CASlot)
 
 		if !currentX509CA.IsEmpty() && !currentX509CA.ShouldActivateNext(now) {
 			// activate the X509CA immediately if it is set and not within
 			// activation time of the next X509CA.
-			m.activateX509CA()
+			m.activateX509CA(ctx)
 		}
 	}
 
 	if nextX509CA, ok := slots[NextX509CASlot]; ok {
-		m.nextX509CA = nextX509CA.(*X509CASlot)
+		m.nextX509CA = nextX509CA.(*x509CASlot)
 	}
 
 	if currentJWTKey, ok := slots[CurrentJWTKeySlot]; ok {
-		m.currentJWTKey = currentJWTKey.(*JwtKeySlot)
+		m.currentJWTKey = currentJWTKey.(*jwtKeySlot)
 
 		// TODO: Activation on journal depends on dates, it will need to be
 		// refactored to allow to set a status, because when forcing a rotation,
@@ -149,12 +178,12 @@ func NewManager(ctx context.Context, c Config) (*Manager, error) {
 		if !currentJWTKey.IsEmpty() && !currentJWTKey.ShouldActivateNext(now) {
 			// activate the JWT key immediately if it is set and not within
 			// activation time of the next JWT key.
-			m.activateJWTKey()
+			m.activateJWTKey(ctx)
 		}
 	}
 
 	if nextJWTKey, ok := slots[NextJWTKeySlot]; ok {
-		m.nextJWTKey = nextJWTKey.(*JwtKeySlot)
+		m.nextJWTKey = nextJWTKey.(*jwtKeySlot)
 	}
 
 	return m, nil
@@ -166,6 +195,16 @@ func (m *Manager) Close() {
 	}
 }
 
+func (m *Manager) NotifyTaintedX509Authority(ctx context.Context, authorityID string) error {
+	taintedAuthority, err := m.fetchRootCAByAuthorityID(ctx, authorityID)
+	if err != nil {
+		return err
+	}
+
+	m.c.CA.NotifyTaintedX509Authorities([]*x509.Certificate{taintedAuthority})
+	return nil
+}
+
 func (m *Manager) GetCurrentX509CASlot() Slot {
 	m.x509CAMutex.RLock()
 	defer m.x509CAMutex.RUnlock()
@@ -226,41 +265,47 @@ func (m *Manager) PrepareX509CA(ctx context.Context) (err error) {
 	// Set key from new CA, to be able to get it after
 	// slot moved to old state
 	slot.authorityID = x509util.SubjectKeyIDToString(x509CA.Certificate.SubjectKeyId)
+	slot.upstreamAuthorityID = x509util.SubjectKeyIDToString(x509CA.Certificate.AuthorityKeyId)
 	slot.publicKey = slot.x509CA.Certificate.PublicKey
 	slot.notAfter = slot.x509CA.Certificate.NotAfter
 
-	if err := m.journal.AppendX509CA(slot.id, slot.issuedAt, slot.x509CA); err != nil {
+	if err := m.journal.AppendX509CA(ctx, slot.id, slot.issuedAt, slot.x509CA); err != nil {
 		log.WithError(err).Error("Unable to append X509 CA to journal")
 	}
 
 	m.c.Log.WithFields(logrus.Fields{
-		telemetry.Slot:             slot.id,
-		telemetry.IssuedAt:         slot.issuedAt,
-		telemetry.Expiration:       slot.x509CA.Certificate.NotAfter,
-		telemetry.SelfSigned:       m.upstreamClient == nil,
-		telemetry.LocalAuthorityID: slot.authorityID,
+		telemetry.Slot:                slot.id,
+		telemetry.IssuedAt:            slot.issuedAt,
+		telemetry.Expiration:          slot.x509CA.Certificate.NotAfter,
+		telemetry.SelfSigned:          m.upstreamClient == nil,
+		telemetry.LocalAuthorityID:    slot.authorityID,
+		telemetry.UpstreamAuthorityID: slot.upstreamAuthorityID,
 	}).Info("X509 CA prepared")
 	return nil
 }
 
-func (m *Manager) ActivateX509CA() {
+func (m *Manager) IsUpstreamAuthority() bool {
+	return m.upstreamClient != nil
+}
+
+func (m *Manager) ActivateX509CA(ctx context.Context) {
 	m.x509CAMutex.RLock()
 	defer m.x509CAMutex.RUnlock()
 
-	m.activateX509CA()
+	m.activateX509CA(ctx)
 }
 
-func (m *Manager) RotateX509CA() {
+func (m *Manager) RotateX509CA(ctx context.Context) {
 	m.x509CAMutex.Lock()
 	defer m.x509CAMutex.Unlock()
 
 	m.currentX509CA, m.nextX509CA = m.nextX509CA, m.currentX509CA
 	m.nextX509CA.Reset()
-	if err := m.journal.UpdateX509CAStatus(m.nextX509CA.issuedAt, journal.Status_OLD); err != nil {
+	if err := m.journal.UpdateX509CAStatus(ctx, m.nextX509CA.AuthorityID(), journal.Status_OLD); err != nil {
 		m.c.Log.WithError(err).Error("Failed to update status on X509CA journal entry")
 	}
 
-	m.activateX509CA()
+	m.activateX509CA(ctx)
 }
 
 func (m *Manager) GetCurrentJWTKeySlot() Slot {
@@ -324,7 +369,7 @@ func (m *Manager) PrepareJWTKey(ctx context.Context) (err error) {
 	slot.authorityID = jwtKey.Kid
 	slot.notAfter = jwtKey.NotAfter
 
-	if err := m.journal.AppendJWTKey(slot.id, slot.issuedAt, slot.jwtKey); err != nil {
+	if err := m.journal.AppendJWTKey(ctx, slot.id, slot.issuedAt, slot.jwtKey); err != nil {
 		log.WithError(err).Error("Unable to append JWT key to journal")
 	}
 
@@ -337,25 +382,25 @@ func (m *Manager) PrepareJWTKey(ctx context.Context) (err error) {
 	return nil
 }
 
-func (m *Manager) ActivateJWTKey() {
+func (m *Manager) ActivateJWTKey(ctx context.Context) {
 	m.jwtKeyMutex.RLock()
 	defer m.jwtKeyMutex.RUnlock()
 
-	m.activateJWTKey()
+	m.activateJWTKey(ctx)
 }
 
-func (m *Manager) RotateJWTKey() {
+func (m *Manager) RotateJWTKey(ctx context.Context) {
 	m.jwtKeyMutex.Lock()
 	defer m.jwtKeyMutex.Unlock()
 
 	m.currentJWTKey, m.nextJWTKey = m.nextJWTKey, m.currentJWTKey
 	m.nextJWTKey.Reset()
 
-	if err := m.journal.UpdateJWTKeyStatus(m.nextJWTKey.issuedAt, journal.Status_OLD); err != nil {
+	if err := m.journal.UpdateJWTKeyStatus(ctx, m.nextJWTKey.AuthorityID(), journal.Status_OLD); err != nil {
 		m.c.Log.WithError(err).Error("Failed to update status on JWTKey journal entry")
 	}
 
-	m.activateJWTKey()
+	m.activateJWTKey(ctx)
 }
 
 // PublishJWTKey publishes the passed JWK to the upstream server using the configured
@@ -408,7 +453,7 @@ func (m *Manager) PruneBundle(ctx context.Context) (err error) {
 	defer counter.Done(&err)
 
 	ds := m.c.Catalog.GetDataStore()
-	expiresBefore := m.c.Clock.Now().Add(-safetyThreshold)
+	expiresBefore := m.c.Clock.Now().Add(-safetyThresholdBundle)
 
 	changed, err := ds.PruneBundle(ctx, m.c.TrustDomain.IDString(), expiresBefore)
 
@@ -425,13 +470,34 @@ func (m *Manager) PruneBundle(ctx context.Context) (err error) {
 	return nil
 }
 
-func (m *Manager) NotifyOnBundleUpdate(ctx context.Context) {
+func (m *Manager) PruneCAJournals(ctx context.Context) (err error) {
+	counter := telemetry_server.StartCAManagerPruneBundleCall(m.c.Metrics)
+	defer counter.Done(&err)
+
+	ds := m.c.Catalog.GetDataStore()
+	expiresBefore := m.c.Clock.Now().Add(-safetyThresholdCAJournals)
+
+	err = ds.PruneCAJournals(ctx, expiresBefore.Unix())
+
+	if err != nil {
+		return fmt.Errorf("unable to prune CA journals: %w", err)
+	}
+	return nil
+}
+
+// ProcessBundleUpdates Notify any bundle update, or process tainted authorities
+func (m *Manager) ProcessBundleUpdates(ctx context.Context) {
 	for {
 		select {
 		case <-m.bundleUpdatedCh:
 			if err := m.notifyBundleUpdated(ctx); err != nil {
 				m.c.Log.WithError(err).Warn("Failed to notify on bundle update")
 			}
+		case taintedAuthorities := <-m.taintedUpstreamAuthoritiesCh:
+			if err := m.notifyTaintedAuthorities(ctx, taintedAuthorities); err != nil {
+				m.c.Log.WithError(err).Error("Failed to force intermediate bundle rotation")
+				return
+			}
 		case <-ctx.Done():
 			return
 		}
@@ -457,7 +523,7 @@ func (m *Manager) NotifyBundleLoaded(ctx context.Context) error {
 	)
 }
 
-func (m *Manager) activateJWTKey() {
+func (m *Manager) activateJWTKey(ctx context.Context) {
 	log := m.c.Log.WithFields(logrus.Fields{
 		telemetry.Slot:             m.currentJWTKey.id,
 		telemetry.IssuedAt:         m.currentJWTKey.issuedAt,
@@ -468,25 +534,26 @@ func (m *Manager) activateJWTKey() {
 	telemetry_server.IncrActivateJWTKeyManagerCounter(m.c.Metrics)
 
 	m.currentJWTKey.status = journal.Status_ACTIVE
-	if err := m.journal.UpdateJWTKeyStatus(m.currentJWTKey.issuedAt, journal.Status_ACTIVE); err != nil {
+	if err := m.journal.UpdateJWTKeyStatus(ctx, m.currentJWTKey.AuthorityID(), journal.Status_ACTIVE); err != nil {
 		log.WithError(err).Error("Failed to update to activated status on JWTKey journal entry")
 	}
 
 	m.c.CA.SetJWTKey(m.currentJWTKey.jwtKey)
 }
 
-func (m *Manager) activateX509CA() {
+func (m *Manager) activateX509CA(ctx context.Context) {
 	log := m.c.Log.WithFields(logrus.Fields{
-		telemetry.Slot:             m.currentX509CA.id,
-		telemetry.IssuedAt:         m.currentX509CA.issuedAt,
-		telemetry.Expiration:       m.currentX509CA.x509CA.Certificate.NotAfter,
-		telemetry.LocalAuthorityID: m.currentX509CA.authorityID,
+		telemetry.Slot:                m.currentX509CA.id,
+		telemetry.IssuedAt:            m.currentX509CA.issuedAt,
+		telemetry.Expiration:          m.currentX509CA.x509CA.Certificate.NotAfter,
+		telemetry.LocalAuthorityID:    m.currentX509CA.authorityID,
+		telemetry.UpstreamAuthorityID: m.currentX509CA.upstreamAuthorityID,
 	})
 	log.Info("X509 CA activated")
 	telemetry_server.IncrActivateX509CAManagerCounter(m.c.Metrics)
 
 	m.currentX509CA.status = journal.Status_ACTIVE
-	if err := m.journal.UpdateX509CAStatus(m.currentX509CA.issuedAt, journal.Status_ACTIVE); err != nil {
+	if err := m.journal.UpdateX509CAStatus(ctx, m.currentX509CA.AuthorityID(), journal.Status_ACTIVE); err != nil {
 		log.WithError(err).Error("Failed to update to activated status on X509CA journal entry")
 	}
 
@@ -513,6 +580,114 @@ func (m *Manager) dropBundleUpdated() {
 	default:
 	}
 }
+
+func (m *Manager) notifyUpstreamAuthoritiesTainted(taintedAuthorities []*x509.Certificate) {
+	select {
+	case m.taintedUpstreamAuthoritiesCh <- taintedAuthorities:
+	default:
+	}
+}
+
+func (m *Manager) fetchRootCAByAuthorityID(ctx context.Context, authorityID string) (*x509.Certificate, error) {
+	bundle, err := m.fetchRequiredBundle(ctx)
+	if err != nil {
+		return nil, err
+	}
+
+	for _, rootCA := range bundle.RootCas {
+		if rootCA.TaintedKey {
+			cert, err := x509.ParseCertificate(rootCA.DerBytes)
+			if err != nil {
+				return nil, fmt.Errorf("failed to parse RootCA: %w", err)
+			}
+
+			skID := x509util.SubjectKeyIDToString(cert.SubjectKeyId)
+			if authorityID == skID {
+				return cert, nil
+			}
+		}
+	}
+
+	return nil, fmt.Errorf("no tainted root CA found with authority ID: %q", authorityID)
+}
+
+func (m *Manager) notifyTaintedAuthorities(ctx context.Context, taintedAuthorities []*x509.Certificate) error {
+	taintBackoff := backoff.NewBackoff(
+		m.c.Clock,
+		taintBackoffInterval,
+		backoff.WithMaxElapsedTime(taintBackoffMaxElapsedTime),
+	)
+
+	for {
+		err := m.processTaintedUpstreamAuthorities(ctx, taintedAuthorities)
+		if err == nil {
+			break
+		}
+
+		nextDuration := taintBackoff.NextBackOff()
+		if nextDuration == backoff.Stop {
+			return err
+		}
+		m.c.Log.WithError(err).Warn("Failed to process tainted keys on upstream authority")
+		if m.triggerBackOffCh != nil {
+			m.triggerBackOffCh <- err
+		}
+
+		select {
+		case <-ctx.Done():
+			return ctx.Err()
+		case <-m.c.Clock.After(nextDuration):
+			continue
+		}
+	}
+
+	return nil
+}
+
+func (m *Manager) processTaintedUpstreamAuthorities(ctx context.Context, taintedAuthorities []*x509.Certificate) error {
+	// Nothing to rotate if no upstream authority is used
+	if m.upstreamClient == nil {
+		return errors.New("processing of tainted upstream authorities must not be reached when not using an upstream authority; please report this bug")
+	}
+
+	if len(taintedAuthorities) == 0 {
+		// No tainted keys found
+		return nil
+	}
+
+	m.c.Log.Debug("Processing tainted keys on upstream authority")
+
+	currentSlotCA := m.currentX509CA.x509CA
+	if ok := isX509AuthorityTainted(currentSlotCA, taintedAuthorities); ok {
+		m.c.Log.Info("Current root CA is signed by a tainted upstream authority, preparing rotation")
+		if ok := m.shouldPrepareX509CA(taintedAuthorities); ok {
+			if err := m.PrepareX509CA(ctx); err != nil {
+				return fmt.Errorf("failed to prepare x509 authority: %w", err)
+			}
+		}
+
+		// Activate the prepared X.509 authority
+		m.RotateX509CA(ctx)
+	}
+
+	// Now that we have rotated the intermediate, we can notify about the
+	// tainted authorities, so agents and downstream servers can start forcing
+	// the rotation of their SVIDs.
+	ds := m.c.Catalog.GetDataStore()
+	for _, each := range taintedAuthorities {
+		skID := x509util.SubjectKeyIDToString(each.SubjectKeyId)
+		if err := ds.TaintX509CA(ctx, m.c.TrustDomain.IDString(), skID); err != nil {
+			return fmt.Errorf("could not taint X509 CA in datastore: %w", err)
+		}
+	}
+
+	// Intermediate is safe. Notify rotator to force rotation
+	// of tainted X.509 SVID.
+	m.c.CA.NotifyTaintedX509Authorities(taintedAuthorities)
+
+	return nil
+}
+
 func (m *Manager) notifyBundleUpdated(ctx context.Context) error {
 	var bundle *common.Bundle
 	return m.notify(ctx, "bundle updated", false,
@@ -675,6 +850,20 @@ func (m *Manager) appendBundle(ctx context.Context, caChain []*x509.Certificate,
 	return res, nil
 }
 
+func (m *Manager) shouldPrepareX509CA(taintedAuthorities []*x509.Certificate) bool {
+	slot := m.nextX509CA
+	switch {
+	case slot.IsEmpty():
+		return true
+	case slot.Status() == journal.Status_PREPARED:
+		isTainted := isX509AuthorityTainted(slot.x509CA, taintedAuthorities)
+		m.c.Log.Info("Next authority is tainted, prepare new X.509 authority")
+		return isTainted
+	default:
+		return false
+	}
+}
+
 // MaxSVIDTTL returns the maximum SVID lifetime that can be guaranteed to not
 // be cut artificially short by a scheduled rotation.
 func MaxSVIDTTL() time.Duration {
@@ -703,23 +892,76 @@ func MinCATTLForSVIDTTL(svidTTL time.Duration) time.Duration {
 }
 
 type bundleUpdater struct {
-	log           logrus.FieldLogger
-	trustDomainID string
-	ds            datastore.DataStore
-	updated       func()
+	log                         logrus.FieldLogger
+	trustDomainID               string
+	ds                          datastore.DataStore
+	updated                     func()
+	upstreamAuthoritiesTainted  func([]*x509.Certificate)
+	processedTaintedAuthorities map[string]struct{}
 }
 
-func (u *bundleUpdater) AppendX509Roots(ctx context.Context, roots []*x509.Certificate) error {
+func (u *bundleUpdater) SyncX509Roots(ctx context.Context, roots []*x509certificate.X509Authority) error {
 	bundle := &common.Bundle{
 		TrustDomainId: u.trustDomainID,
 		RootCas:       make([]*common.Certificate, 0, len(roots)),
 	}
 
+	x509Authorities, err := u.fetchX509Authorities(ctx)
+	if err != nil {
+		return err
+	}
+
+	newAuthorities := make(map[string]struct{}, len(roots))
+	var taintedAuthorities []*x509.Certificate
 	for _, root := range roots {
+		skID := x509util.SubjectKeyIDToString(root.Certificate.SubjectKeyId)
+		// Collect all skIDs
+		newAuthorities[skID] = struct{}{}
+
+		// Verify if new root ca is tainted
+		if root.Tainted {
+			// Taint x.509 authority, if required
+			if found, ok := x509Authorities[skID]; ok && !found.Tainted {
+				_, alreadyProcessed := u.processedTaintedAuthorities[skID]
+				if !alreadyProcessed {
+					u.processedTaintedAuthorities[skID] = struct{}{}
+					// Add to the list of new tainted authorities
+					taintedAuthorities = append(taintedAuthorities, found.Certificate)
+					u.log.WithField(telemetry.SubjectKeyID, skID).Info("X.509 authority tainted")
+				}
+				// Prevent to add tainted keys, since status is updated before
+				continue
+			}
+		}
+
 		bundle.RootCas = append(bundle.RootCas, &common.Certificate{
-			DerBytes: root.Raw,
+			DerBytes:   root.Certificate.Raw,
+			TaintedKey: root.Tainted,
 		})
 	}
+
+	// Notify about tainted authorities to force the rotation of
+	// intermediates and update the database. This is done in a separate thread
+	// to prevent agents and downstream servers to start the rotation before the
+	// current server starts the rotation of the intermediate.
+	if len(taintedAuthorities) > 0 {
+		u.upstreamAuthoritiesTainted(taintedAuthorities)
+	}
+
+	for skID, authority := range x509Authorities {
+		// Only tainted keys can ke revoked
+		if authority.Tainted {
+			// In case a stored tainted authority is not found,
+			// from latest bundle update, then revoke it
+			if _, found := newAuthorities[skID]; !found {
+				if err := u.ds.RevokeX509CA(ctx, u.trustDomainID, skID); err != nil {
+					return fmt.Errorf("failed to revoke a tainted key %q: %w", skID, err)
+				}
+				u.log.WithField(telemetry.SubjectKeyID, skID).Info("X.509 authority revoked")
+			}
+		}
+	}
+
 	if _, err := u.appendBundle(ctx, bundle); err != nil {
 		return err
 	}
@@ -741,6 +983,32 @@ func (u *bundleUpdater) LogError(err error, msg string) {
 	u.log.WithError(err).Error(msg)
 }
 
+func (u *bundleUpdater) fetchX509Authorities(ctx context.Context) (map[string]*x509certificate.X509Authority, error) {
+	bundle, err := u.ds.FetchBundle(ctx, u.trustDomainID)
+	if err != nil {
+		return nil, fmt.Errorf("failed to fetch bundle: %w", err)
+	}
+	// Bundle not found
+	if bundle == nil {
+		return nil, nil
+	}
+
+	authorities := map[string]*x509certificate.X509Authority{}
+	for _, eachRoot := range bundle.RootCas {
+		cert, err := x509.ParseCertificate(eachRoot.DerBytes)
+		if err != nil {
+			return nil, fmt.Errorf("failed to parse root certificate: %w", err)
+		}
+
+		authorities[x509util.SubjectKeyIDToString(cert.SubjectKeyId)] = &x509certificate.X509Authority{
+			Certificate: cert,
+			Tainted:     eachRoot.TaintedKey,
+		}
+	}
+
+	return authorities, nil
+}
+
 func (u *bundleUpdater) appendBundle(ctx context.Context, bundle *common.Bundle) (*common.Bundle, error) {
 	dsBundle, err := u.ds.AppendBundle(ctx, bundle)
 	if err != nil {
@@ -793,3 +1061,24 @@ func publicKeyFromJWTKey(jwtKey *ca.JWTKey) (*common.PublicKey, error) {
 		NotAfter:  jwtKey.NotAfter.Unix(),
 	}, nil
 }
+
+// isX509AuthorityTainted verifies if the provided X.509 authority is tainted
+func isX509AuthorityTainted(x509CA *ca.X509CA, taintedAuthorities []*x509.Certificate) bool {
+	rootPool := x509.NewCertPool()
+	for _, taintedKey := range taintedAuthorities {
+		rootPool.AddCert(taintedKey)
+	}
+
+	intermediatePool := x509.NewCertPool()
+	for _, intermediateCA := range x509CA.UpstreamChain {
+		intermediatePool.AddCert(intermediateCA)
+	}
+
+	// Verify certificate chain, using tainted authority as root
+	_, err := x509CA.Certificate.Verify(x509.VerifyOptions{
+		Intermediates: intermediatePool,
+		Roots:         rootPool,
+	})
+
+	return err == nil
+}
