@@ -10,8 +10,6 @@ import (
 	"errors"
 	"fmt"
 	"math/big"
-	"os"
-	"path/filepath"
 	"sync"
 	"testing"
 	"time"
@@ -19,10 +17,13 @@ import (
 	"github.com/sirupsen/logrus"
 	"github.com/sirupsen/logrus/hooks/test"
 	"github.com/spiffe/go-spiffe/v2/spiffeid"
+	"github.com/spiffe/spire/pkg/common/coretypes/x509certificate"
 	telemetry_server "github.com/spiffe/spire/pkg/common/telemetry/server"
+	"github.com/spiffe/spire/pkg/common/x509util"
 	"github.com/spiffe/spire/pkg/server/ca"
 	"github.com/spiffe/spire/pkg/server/credtemplate"
 	"github.com/spiffe/spire/pkg/server/credvalidator"
+	"github.com/spiffe/spire/pkg/server/datastore"
 	"github.com/spiffe/spire/pkg/server/plugin/keymanager"
 	"github.com/spiffe/spire/pkg/server/plugin/notifier"
 	"github.com/spiffe/spire/pkg/server/plugin/upstreamauthority"
@@ -36,9 +37,11 @@ import (
 	"github.com/spiffe/spire/test/fakes/fakeserverkeymanager"
 	"github.com/spiffe/spire/test/fakes/fakeupstreamauthority"
 	"github.com/spiffe/spire/test/spiretest"
+	"github.com/spiffe/spire/test/testca"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"google.golang.org/grpc/codes"
+	"google.golang.org/protobuf/proto"
 )
 
 const (
@@ -56,11 +59,12 @@ func TestGetCurrentJWTKeySlot(t *testing.T) {
 
 	test := setupTest(t)
 	test.initSelfSignedManager()
+	require.False(t, test.m.IsUpstreamAuthority())
 
 	t.Run("no authority created", func(t *testing.T) {
 		currentSlot := test.m.GetCurrentJWTKeySlot()
 
-		slot := currentSlot.(*JwtKeySlot)
+		slot := currentSlot.(*jwtKeySlot)
 
 		require.True(t, slot.IsEmpty())
 		require.Empty(t, slot.issuedAt)
@@ -75,7 +79,7 @@ func TestGetCurrentJWTKeySlot(t *testing.T) {
 		require.NoError(t, test.m.PrepareJWTKey(ctx))
 
 		currentSlot := test.m.GetCurrentJWTKeySlot()
-		slot := currentSlot.(*JwtKeySlot)
+		slot := currentSlot.(*jwtKeySlot)
 		require.NotNil(t, slot.jwtKey)
 		require.NotEmpty(t, slot.authorityID)
 		require.Equal(t, expectIssuedAt, slot.issuedAt)
@@ -91,7 +95,7 @@ func TestGetNextJWTKeySlot(t *testing.T) {
 
 	t.Run("no next created", func(t *testing.T) {
 		nextSlot := test.m.GetNextJWTKeySlot()
-		slot := nextSlot.(*JwtKeySlot)
+		slot := nextSlot.(*jwtKeySlot)
 
 		require.Nil(t, slot.jwtKey)
 		require.Empty(t, slot.issuedAt)
@@ -106,7 +110,7 @@ func TestGetNextJWTKeySlot(t *testing.T) {
 		require.NoError(t, test.m.PrepareJWTKey(ctx))
 
 		nextSlot := test.m.GetNextJWTKeySlot()
-		slot := nextSlot.(*JwtKeySlot)
+		slot := nextSlot.(*jwtKeySlot)
 		require.NotNil(t, slot.jwtKey)
 		require.NotEmpty(t, slot.authorityID)
 		require.Equal(t, expectIssuedAt, slot.issuedAt)
@@ -123,9 +127,10 @@ func TestGetCurrentX509CASlot(t *testing.T) {
 	t.Run("no authority created", func(t *testing.T) {
 		currentSlot := test.m.GetCurrentX509CASlot()
 
-		slot := currentSlot.(*X509CASlot)
+		slot := currentSlot.(*x509CASlot)
 		require.Nil(t, slot.x509CA)
 		require.Empty(t, slot.authorityID)
+		require.Empty(t, slot.upstreamAuthorityID)
 		require.Empty(t, slot.issuedAt)
 		require.Empty(t, slot.publicKey)
 		require.Empty(t, slot.notAfter)
@@ -138,9 +143,10 @@ func TestGetCurrentX509CASlot(t *testing.T) {
 		require.NoError(t, test.m.PrepareX509CA(ctx))
 
 		currentSlot := test.m.GetCurrentX509CASlot()
-		slot := currentSlot.(*X509CASlot)
+		slot := currentSlot.(*x509CASlot)
 		require.NotNil(t, slot.x509CA)
 		require.NotEmpty(t, slot.authorityID)
+		require.Empty(t, slot.upstreamAuthorityID)
 		require.NotNil(t, slot.publicKey)
 		require.Equal(t, expectIssuedAt, slot.issuedAt)
 		require.Equal(t, expectNotAfter, slot.notAfter)
@@ -155,10 +161,11 @@ func TestGetNextX509CASlot(t *testing.T) {
 
 	t.Run("no next created", func(t *testing.T) {
 		nextSlot := test.m.GetNextX509CASlot()
-		slot := nextSlot.(*X509CASlot)
+		slot := nextSlot.(*x509CASlot)
 
 		require.Nil(t, slot.x509CA)
 		require.Empty(t, slot.authorityID)
+		require.Empty(t, slot.upstreamAuthorityID)
 		require.Empty(t, slot.issuedAt)
 		require.Empty(t, slot.publicKey)
 		require.Empty(t, slot.notAfter)
@@ -171,9 +178,10 @@ func TestGetNextX509CASlot(t *testing.T) {
 		require.NoError(t, test.m.PrepareX509CA(ctx))
 
 		nextSlot := test.m.GetNextX509CASlot()
-		slot := nextSlot.(*X509CASlot)
+		slot := nextSlot.(*x509CASlot)
 		require.NotNil(t, slot.x509CA)
 		require.NotEmpty(t, slot.authorityID)
+		require.Empty(t, slot.upstreamAuthorityID)
 		require.NotNil(t, slot.publicKey)
 		require.Equal(t, expectIssuedAt, slot.issuedAt)
 		require.Equal(t, expectNotAfter, slot.notAfter)
@@ -192,9 +200,9 @@ func TestPersistence(t *testing.T) {
 
 	// Prepare authority and activate authority
 	require.NoError(t, test.m.PrepareJWTKey(ctx))
-	test.m.ActivateJWTKey()
+	test.m.ActivateJWTKey(ctx)
 	require.NoError(t, test.m.PrepareX509CA(ctx))
-	test.m.ActivateX509CA()
+	test.m.ActivateX509CA(ctx)
 
 	firstX509CA, firstJWTKey := test.currentX509CA(), test.currentJWTKey()
 
@@ -240,6 +248,65 @@ func TestSlotLoadedWhenJournalIsLost(t *testing.T) {
 	require.True(t, test.m.GetCurrentX509CASlot().IsEmpty())
 }
 
+func TestNotifyTaintedX509Authority(t *testing.T) {
+	ctx := context.Background()
+	test := setupTest(t)
+	test.initSelfSignedManager()
+
+	// Create a test CA
+	ca := testca.New(t, testTrustDomain)
+	cert := ca.X509Authorities()[0]
+	bundle, err := test.ds.CreateBundle(ctx, &common.Bundle{
+		TrustDomainId: testTrustDomain.IDString(),
+		RootCas: []*common.Certificate{
+			{
+				DerBytes:   cert.Raw,
+				TaintedKey: true,
+			},
+		},
+	})
+	require.NoError(t, err)
+
+	t.Run("notify tainted authority", func(t *testing.T) {
+		err = test.m.NotifyTaintedX509Authority(ctx, ca.GetSubjectKeyID())
+		require.NoError(t, err)
+		ctx, cancel := context.WithTimeout(ctx, time.Minute)
+		defer cancel()
+
+		expectedTaintedAuthorities := []*x509.Certificate{cert}
+		select {
+		case taintedAuthorities := <-test.ca.taintedAuthoritiesCh:
+			require.Equal(t, expectedTaintedAuthorities, taintedAuthorities)
+		case <-ctx.Done():
+			assert.Fail(t, "no notification received")
+		}
+	})
+
+	// Untaint authority
+	bundle.RootCas[0].TaintedKey = false
+	bundle, err = test.ds.UpdateBundle(ctx, bundle, nil)
+	require.NoError(t, err)
+
+	t.Run("no tainted authority", func(t *testing.T) {
+		err := test.m.NotifyTaintedX509Authority(ctx, ca.GetSubjectKeyID())
+
+		expectedErr := fmt.Sprintf("no tainted root CA found with authority ID: %q", ca.GetSubjectKeyID())
+		require.EqualError(t, err, expectedErr)
+	})
+
+	bundle.RootCas = append(bundle.RootCas, &common.Certificate{
+		DerBytes:   []byte("foh"),
+		TaintedKey: true,
+	})
+	_, err = test.ds.UpdateBundle(ctx, bundle, nil)
+	require.NoError(t, err)
+
+	t.Run("malformed root CA", func(t *testing.T) {
+		err := test.m.NotifyTaintedX509Authority(ctx, ca.GetSubjectKeyID())
+		require.EqualError(t, err, "failed to parse RootCA: x509: malformed certificate")
+	})
+}
+
 func TestSelfSigning(t *testing.T) {
 	ctx := context.Background()
 	test := setupTest(t)
@@ -262,19 +329,20 @@ func TestUpstreamSigned(t *testing.T) {
 	ctx := context.Background()
 	test := setupTest(t)
 
-	upstreamAuthority, fakeUA := fakeupstreamauthority.Load(t, fakeupstreamauthority.Config{
+	upstreamAuthority, fakeUA := test.newFakeUpstreamAuthority(t, fakeupstreamauthority.Config{
 		TrustDomain:           testTrustDomain,
 		DisallowPublishJWTKey: true,
 	})
 
 	test.initAndActivateUpstreamSignedManager(ctx, upstreamAuthority)
+	require.True(t, test.m.IsUpstreamAuthority())
 
 	// X509 CA should be set up to be an intermediate but only have itself
 	// in the chain since it was signed directly by the upstream root.
 	x509CA := test.currentX509CA()
 	assert.NotNil(t, x509CA.Signer)
 	if assert.NotNil(t, x509CA.Certificate) {
-		assert.Equal(t, fakeUA.X509Root().Subject, x509CA.Certificate.Issuer)
+		assert.Equal(t, fakeUA.X509Root().Certificate.Subject, x509CA.Certificate.Issuer)
 	}
 	if assert.Len(t, x509CA.UpstreamChain, 1) {
 		assert.Equal(t, x509CA.Certificate, x509CA.UpstreamChain[0])
@@ -290,12 +358,188 @@ func TestUpstreamSigned(t *testing.T) {
 			"by this server may have trouble communicating with workloads outside "+
 			"this cluster when using JWT-SVIDs."),
 	)
+
+	// Taint first root
+	err := fakeUA.TaintAuthority(0)
+	require.NoError(t, err)
+
+	// Get the roots again and verify that the first X.509 authority is tainted
+	x509Roots := fakeUA.X509Roots()
+	require.True(t, x509Roots[0].Tainted)
+
+	ctx, cancel := context.WithTimeout(ctx, time.Minute)
+	defer cancel()
+
+	select {
+	case taintedAuthorities := <-test.m.taintedUpstreamAuthoritiesCh:
+		expectedTaintedAuthorities := []*x509.Certificate{x509Roots[0].Certificate}
+		require.Equal(t, expectedTaintedAuthorities, taintedAuthorities)
+	case <-ctx.Done():
+		assert.Fail(t, "no notification received")
+	}
+}
+
+func TestUpstreamProcessTaintedAuthority(t *testing.T) {
+	ctx, cancel := context.WithTimeout(context.Background(), time.Minute)
+	defer cancel()
+
+	test := setupTest(t)
+
+	upstreamAuthority, fakeUA := fakeupstreamauthority.Load(t, fakeupstreamauthority.Config{
+		TrustDomain:           testTrustDomain,
+		DisallowPublishJWTKey: true,
+	})
+
+	test.initAndActivateUpstreamSignedManager(ctx, upstreamAuthority)
+	require.True(t, test.m.IsUpstreamAuthority())
+
+	go test.m.ProcessBundleUpdates(ctx)
+
+	// Taint first root
+	err := fakeUA.TaintAuthority(0)
+	require.NoError(t, err)
+
+	// Get the roots again and verify that the first X.509 authority is tainted
+	x509Roots := fakeUA.X509Roots()
+	require.True(t, x509Roots[0].Tainted)
+
+	expectedTaintedAuthorities := []*x509.Certificate{x509Roots[0].Certificate}
+	select {
+	case received := <-test.ca.taintedAuthoritiesCh:
+		require.Equal(t, expectedTaintedAuthorities, received)
+	case <-ctx.Done():
+		assert.Fail(t, "deadline reached")
+	}
+
+	bundle := test.fetchBundle(ctx)
+	expectRootCas := x509certificate.RequireToCommonProtos(x509Roots)
+	spiretest.AssertProtoListEqual(t, expectRootCas, bundle.RootCas)
+}
+
+func TestUpstreamProcessTaintedAuthorityBackoff(t *testing.T) {
+	ctx, cancel := context.WithTimeout(context.Background(), time.Minute)
+	defer cancel()
+
+	test := setupTest(t)
+
+	upstreamAuthority, fakeUA := fakeupstreamauthority.Load(t, fakeupstreamauthority.Config{
+		TrustDomain:           testTrustDomain,
+		DisallowPublishJWTKey: true,
+	})
+
+	test.initAndActivateUpstreamSignedManager(ctx, upstreamAuthority)
+	require.True(t, test.m.IsUpstreamAuthority())
+
+	test.m.triggerBackOffCh = make(chan error, 1)
+
+	// Prepared must be tainted too
+	go test.m.ProcessBundleUpdates(ctx)
+
+	// Set an invalid key type to make prepare fails
+	test.m.c.X509CAKeyType = 123
+	err := test.m.PrepareX509CA(ctx)
+	require.Error(t, err)
+
+	// Taint first root
+	err = fakeUA.TaintAuthority(0)
+	require.NoError(t, err)
+
+	// Get the roots again and verify that the first X.509 authority is tainted
+	x509Roots := fakeUA.X509Roots()
+	require.True(t, x509Roots[0].Tainted)
+
+	expectBackoffErr := func(t *testing.T) {
+		select {
+		case receivedErr := <-test.m.triggerBackOffCh:
+			require.EqualError(t, receivedErr, "failed to prepare x509 authority: rpc error: code = Internal desc = keymanager(fake): facade does not support key type \"UNKNOWN(123)\"")
+		case <-ctx.Done():
+			assert.Fail(t, "deadline reached")
+		}
+	}
+
+	// Must fail due to the invalid key type
+	expectBackoffErr(t)
+
+	// Try again; expect to fail
+	test.clock.Add(6 * time.Second)
+	expectBackoffErr(t)
+
+	// Restore to a valid key type, and advance time again
+	test.m.c.X509CAKeyType = keymanager.ECP256
+	test.clock.Add(10 * time.Second)
+
+	expectedTaintedAuthorities := []*x509.Certificate{x509Roots[0].Certificate}
+	select {
+	case received := <-test.ca.taintedAuthoritiesCh:
+		require.Equal(t, expectedTaintedAuthorities, received)
+	case <-ctx.Done():
+		assert.Fail(t, "deadline reached")
+	}
+
+	bundle := test.fetchBundle(ctx)
+	expectRootCas := x509certificate.RequireToCommonProtos(x509Roots)
+	spiretest.AssertProtoListEqual(t, expectRootCas, bundle.RootCas)
+}
+
+func TestGetCurrentX509CASlotUpstreamSigned(t *testing.T) {
+	ctx := context.Background()
+
+	test := setupTest(t)
+
+	upstreamAuthority, ua := test.newFakeUpstreamAuthority(t, fakeupstreamauthority.Config{
+		TrustDomain:           testTrustDomain,
+		DisallowPublishJWTKey: true,
+	})
+
+	test.initAndActivateUpstreamSignedManager(ctx, upstreamAuthority)
+
+	expectIssuedAt := test.clock.Now()
+	expectNotAfter := expectIssuedAt.Add(test.m.caTTL).UTC()
+	expectUpstreamAuthorityID := x509util.SubjectKeyIDToString(ua.X509Root().Certificate.SubjectKeyId)
+
+	require.NoError(t, test.m.PrepareX509CA(ctx))
+
+	currentSlot := test.m.GetCurrentX509CASlot()
+	slot := currentSlot.(*x509CASlot)
+	require.NotNil(t, slot.x509CA)
+	require.NotEmpty(t, slot.authorityID)
+	require.Equal(t, expectUpstreamAuthorityID, slot.upstreamAuthorityID)
+	require.NotNil(t, slot.publicKey)
+	require.Equal(t, expectIssuedAt, slot.issuedAt)
+	require.Equal(t, expectNotAfter, slot.notAfter)
+}
+
+func TestGetNextX509CASlotUpstreamSigned(t *testing.T) {
+	ctx := context.Background()
+
+	test := setupTest(t)
+	upstreamAuthority, ua := test.newFakeUpstreamAuthority(t, fakeupstreamauthority.Config{
+		TrustDomain:           testTrustDomain,
+		DisallowPublishJWTKey: true,
+	})
+
+	test.initAndActivateUpstreamSignedManager(ctx, upstreamAuthority)
+
+	expectIssuedAt := test.clock.Now()
+	expectNotAfter := expectIssuedAt.Add(test.m.caTTL).UTC()
+	expectUpstreamAuthorityID := x509util.SubjectKeyIDToString(ua.X509Root().Certificate.SubjectKeyId)
+
+	require.NoError(t, test.m.PrepareX509CA(ctx))
+
+	nextSlot := test.m.GetNextX509CASlot()
+	slot := nextSlot.(*x509CASlot)
+	require.NotNil(t, slot.x509CA)
+	require.NotEmpty(t, slot.authorityID)
+	require.Equal(t, expectUpstreamAuthorityID, slot.upstreamAuthorityID)
+	require.NotNil(t, slot.publicKey)
+	require.Equal(t, expectIssuedAt, slot.issuedAt)
+	require.Equal(t, expectNotAfter, slot.notAfter)
 }
 
 func TestUpstreamSignedProducesInvalidChain(t *testing.T) {
 	ctx := context.Background()
 	test := setupTest(t)
-	upstreamAuthority, _ := fakeupstreamauthority.Load(t, fakeupstreamauthority.Config{
+	upstreamAuthority, _ := test.newFakeUpstreamAuthority(t, fakeupstreamauthority.Config{
 		TrustDomain: testTrustDomain,
 		// The verification code relies on go-spiffe, which for compat reasons,
 		// does not currently validate SPIFFE conformance beyond the leaf
@@ -326,7 +570,7 @@ func TestUpstreamSignedProducesInvalidChain(t *testing.T) {
 func TestUpstreamIntermediateSigned(t *testing.T) {
 	ctx := context.Background()
 	test := setupTest(t)
-	upstreamAuthority, fakeUA := fakeupstreamauthority.Load(t, fakeupstreamauthority.Config{
+	upstreamAuthority, fakeUA := test.newFakeUpstreamAuthority(t, fakeupstreamauthority.Config{
 		TrustDomain:           testTrustDomain,
 		DisallowPublishJWTKey: true,
 		UseIntermediate:       true,
@@ -348,7 +592,7 @@ func TestUpstreamIntermediateSigned(t *testing.T) {
 	// The trust bundle should contain the upstream root
 	test.requireBundleRootCAs(ctx, t, fakeUA.X509Root())
 
-	// We expect this warning because the UpstreamAuthority doesn't implements PublishJWTKey
+	// We expect this warning because the UpstreamAuthority doesn't implement PublishJWTKey
 	assert.Equal(t,
 		1,
 		test.countLogEntries(logrus.WarnLevel, "UpstreamAuthority plugin does not support JWT-SVIDs. Workloads managed "+
@@ -363,7 +607,7 @@ func TestUpstreamAuthorityWithPublishJWTKeyImplemented(t *testing.T) {
 	bundle := test.createBundle(ctx)
 	require.Len(t, bundle.JwtSigningKeys, 0)
 
-	upstreamAuthority, ua := fakeupstreamauthority.Load(t, fakeupstreamauthority.Config{
+	upstreamAuthority, ua := test.newFakeUpstreamAuthority(t, fakeupstreamauthority.Config{
 		TrustDomain: testTrustDomain,
 	})
 	test.initAndActivateUpstreamSignedManager(ctx, upstreamAuthority)
@@ -392,14 +636,14 @@ func TestX509CARotation(t *testing.T) {
 	// kick off a goroutine to service bundle update notifications. This is
 	// typically handled by Run() but using it would complicate the test.
 	test.m.dropBundleUpdated()
-	go test.m.NotifyOnBundleUpdate(ctx)
+	go test.m.ProcessBundleUpdates(ctx)
 
 	// after initialization, we should have a current X509CA but no next.
 	first := test.currentX509CA()
 	require.Equal(t, journal.Status_ACTIVE, test.currentX509CAStatus())
 	assert.Nil(t, test.nextX509CA(), "second X509CA should not be prepared yet")
 	require.Equal(t, journal.Status_UNKNOWN, test.nextX509CAStatus())
-	test.requireBundleRootCAs(ctx, t, first.Certificate)
+	test.requireIntermediateRootCA(ctx, t, first.Certificate)
 
 	// Prepare new X509CA. the current X509CA should stay
 	// the same but the next X509CA should have been prepared and added to
@@ -411,14 +655,14 @@ func TestX509CARotation(t *testing.T) {
 	second := test.nextX509CA()
 	assert.NotNil(t, second, "second X509CA should have been prepared")
 	require.Equal(t, journal.Status_PREPARED, test.nextX509CAStatus())
-	test.requireBundleRootCAs(ctx, t, first.Certificate, second.Certificate)
+	test.requireIntermediateRootCA(ctx, t, first.Certificate, second.Certificate)
 
 	// we should now have a bundle update notification due to the preparation
 	test.waitForBundleUpdatedNotification(ctx, notifyCh)
 
 	// Rotate "next" should become "current" and
 	// "next" should be reset.
-	test.m.RotateX509CA()
+	test.m.RotateX509CA(ctx)
 	test.requireX509CAEqual(t, second, test.currentX509CA())
 	require.Equal(t, journal.Status_ACTIVE, test.currentX509CAStatus())
 	assert.Nil(t, test.nextX509CA())
@@ -433,14 +677,14 @@ func TestX509CARotation(t *testing.T) {
 	third := test.nextX509CA()
 	assert.NotNil(t, third, "third X509CA should have been prepared")
 	require.Equal(t, journal.Status_PREPARED, test.nextX509CAStatus())
-	test.requireBundleRootCAs(ctx, t, first.Certificate, second.Certificate, third.Certificate)
+	test.requireIntermediateRootCA(ctx, t, first.Certificate, second.Certificate, third.Certificate)
 
 	// we should now have another bundle update notification due to the preparation
 	test.waitForBundleUpdatedNotification(ctx, notifyCh)
 
 	// Rotate again, "next" should become "current" and
 	// "next" should be reset.
-	test.m.RotateX509CA()
+	test.m.RotateX509CA(ctx)
 	test.requireX509CAEqual(t, third, test.currentX509CA())
 	require.Equal(t, journal.Status_ACTIVE, test.currentX509CAStatus())
 	assert.Nil(t, test.nextX509CA())
@@ -458,7 +702,7 @@ func TestX509CARotationMetric(t *testing.T) {
 
 	// reset the metrics rotate CA to activate mark
 	test.metrics.Reset()
-	test.m.RotateX509CA()
+	test.m.RotateX509CA(ctx)
 
 	// create expected metrics with ttl from certificate
 	expected := fakemetrics.New()
@@ -482,7 +726,7 @@ func TestJWTKeyRotation(t *testing.T) {
 	// kick off a goroutine to service bundle update notifications. This is
 	// typically handled by Run() but using it would complicate the test.
 	test.m.dropBundleUpdated() // drop bundle update message produce by initialization
-	go test.m.NotifyOnBundleUpdate(ctx)
+	go test.m.ProcessBundleUpdates(ctx)
 
 	// after initialization, we should have a current JWTKey but no next.
 	first := test.currentJWTKey()
@@ -507,7 +751,7 @@ func TestJWTKeyRotation(t *testing.T) {
 
 	// rotate, "next" should become "current" and
 	// "next" should be reset.
-	test.m.RotateJWTKey()
+	test.m.RotateJWTKey(ctx)
 	test.requireJWTKeyEqual(t, second, test.currentJWTKey())
 	require.Equal(t, journal.Status_ACTIVE, test.currentJWTKeyStatus())
 	assert.Nil(t, test.nextJWTKey())
@@ -529,7 +773,7 @@ func TestJWTKeyRotation(t *testing.T) {
 
 	// rotate again. "next" should become "current" and
 	// "next" should be reset.
-	test.m.RotateJWTKey()
+	test.m.RotateJWTKey(ctx)
 	test.requireJWTKeyEqual(t, third, test.currentJWTKey())
 	require.Equal(t, journal.Status_ACTIVE, test.currentJWTKeyStatus())
 	assert.Nil(t, test.nextJWTKey())
@@ -562,24 +806,24 @@ func TestPruneBundle(t *testing.T) {
 	firstJWTKey := test.currentJWTKey()
 	secondX509CA := test.nextX509CA()
 	secondJWTKey := test.nextJWTKey()
-	test.requireBundleRootCAs(ctx, t, firstX509CA.Certificate, secondX509CA.Certificate)
+	test.requireIntermediateRootCA(ctx, t, firstX509CA.Certificate, secondX509CA.Certificate)
 	test.requireBundleJWTKeys(ctx, t, firstJWTKey, secondJWTKey)
 
 	// kick off a goroutine to service bundle update notifications. This is
 	// typically handled by Run() but using it would complicate the test.
 	test.m.dropBundleUpdated() // drop bundle update message produce by initialization
-	go test.m.NotifyOnBundleUpdate(ctx)
+	go test.m.ProcessBundleUpdates(ctx)
 
 	// advance just past the expiration time of the first and prune. nothing
 	// should change.
 	test.setTimeAndPrune(firstExpiresTime.Add(time.Minute))
-	test.requireBundleRootCAs(ctx, t, firstX509CA.Certificate, secondX509CA.Certificate)
+	test.requireIntermediateRootCA(ctx, t, firstX509CA.Certificate, secondX509CA.Certificate)
 	test.requireBundleJWTKeys(ctx, t, firstJWTKey, secondJWTKey)
 
 	// advance beyond the safety threshold of the first, prune, and assert that
 	// the first has been pruned
-	test.addTimeAndPrune(safetyThreshold)
-	test.requireBundleRootCAs(ctx, t, secondX509CA.Certificate)
+	test.addTimeAndPrune(safetyThresholdBundle)
+	test.requireIntermediateRootCA(ctx, t, secondX509CA.Certificate)
 	test.requireBundleJWTKeys(ctx, t, secondJWTKey)
 
 	// we should now have a bundle update notification due to the pruning
@@ -587,38 +831,158 @@ func TestPruneBundle(t *testing.T) {
 
 	// advance beyond the second expiration time, prune, and assert nothing
 	// changes because we can't prune out the whole bundle.
-	test.clock.Set(secondExpiresTime.Add(time.Minute + safetyThreshold))
+	test.clock.Set(secondExpiresTime.Add(time.Minute + safetyThresholdBundle))
 	require.EqualError(t, test.m.PruneBundle(context.Background()), "unable to prune bundle: rpc error: code = Unknown desc = prune failed: would prune all certificates")
-	test.requireBundleRootCAs(ctx, t, secondX509CA.Certificate)
+	test.requireIntermediateRootCA(ctx, t, secondX509CA.Certificate)
 	test.requireBundleJWTKeys(ctx, t, secondJWTKey)
 }
 
-func TestRunNotifiesBundleLoaded(t *testing.T) {
+func TestPruneCAJournals(t *testing.T) {
 	test := setupTest(t)
-	test.initAndActivateSelfSignedManager(context.Background())
+	test.initSelfSignedManager()
 
+	type testJournal struct {
+		Journal
+		shouldBePruned bool
+	}
+
+	timeNow := test.clock.Now()
+	now := timeNow.Unix()
+	tomorrow := timeNow.Add(time.Hour * 24).Unix()
+	beforeThreshold := timeNow.Add(-safetyThresholdCAJournals).Add(-time.Minute).Unix()
+
+	jc := &journalConfig{
+		cat: test.cat,
+		log: test.log,
+	}
+	testCases := []struct {
+		name         string
+		entries      *journal.Entries
+		testJournals []*testJournal
+	}{
+		{
+			name: "no journals with CAs expired before the threshold - no journals to be pruned",
+			testJournals: []*testJournal{
+				{
+					Journal: Journal{
+						config: jc,
+						entries: &journal.Entries{
+							X509CAs: []*journal.X509CAEntry{{NotAfter: now}, {NotAfter: tomorrow}},
+							JwtKeys: []*journal.JWTKeyEntry{{NotAfter: now}, {NotAfter: tomorrow}},
+						},
+					},
+				},
+				{
+					Journal: Journal{
+						config: jc,
+						entries: &journal.Entries{
+							X509CAs: []*journal.X509CAEntry{{NotAfter: now}, {NotAfter: tomorrow}},
+							JwtKeys: []*journal.JWTKeyEntry{{NotAfter: now}, {NotAfter: tomorrow}},
+						},
+					},
+				},
+			},
+		},
+		{
+			name: "some journals with CAs expired before the threshold, but not all - no journals to be pruned",
+			testJournals: []*testJournal{
+				{
+					Journal: Journal{
+						config: jc,
+						entries: &journal.Entries{
+							X509CAs: []*journal.X509CAEntry{{NotAfter: tomorrow}, {NotAfter: beforeThreshold}},
+							JwtKeys: []*journal.JWTKeyEntry{{NotAfter: beforeThreshold}, {NotAfter: tomorrow}},
+						},
+					},
+				},
+				{
+					Journal: Journal{
+						config: jc,
+						entries: &journal.Entries{
+							X509CAs: []*journal.X509CAEntry{{NotAfter: tomorrow}, {NotAfter: beforeThreshold}},
+							JwtKeys: []*journal.JWTKeyEntry{{NotAfter: beforeThreshold}, {NotAfter: tomorrow}},
+						},
+					},
+				},
+			},
+		},
+		{
+			name: "all CAs expired before the threshold in a journal - one journal to be pruned",
+			testJournals: []*testJournal{
+				{
+					shouldBePruned: true,
+					Journal: Journal{
+						config: jc,
+						entries: &journal.Entries{
+							X509CAs: []*journal.X509CAEntry{{NotAfter: beforeThreshold}, {NotAfter: beforeThreshold}},
+							JwtKeys: []*journal.JWTKeyEntry{{NotAfter: beforeThreshold}, {NotAfter: beforeThreshold}},
+						},
+					},
+				},
+				{
+					Journal: Journal{
+						config: jc,
+						entries: &journal.Entries{
+							X509CAs: []*journal.X509CAEntry{{NotAfter: tomorrow}, {NotAfter: beforeThreshold}},
+							JwtKeys: []*journal.JWTKeyEntry{{NotAfter: beforeThreshold}, {NotAfter: tomorrow}},
+						},
+					},
+				},
+			},
+		},
+	}
+
+	var expectedCAJournals []*datastore.CAJournal
+	for _, testCase := range testCases {
+		testCase := testCase
+		expectedCAJournals = []*datastore.CAJournal{}
+		t.Run(testCase.name, func(t *testing.T) {
+			// Have a fresh data store in each test case
+			test.ds = fakedatastore.New(t)
+			test.m.c.Catalog.(*fakeservercatalog.Catalog).SetDataStore(test.ds)
+
+			for _, j := range testCase.testJournals {
+				entriesBytes, err := proto.Marshal(j.entries)
+				require.NoError(t, err)
+				caJournal, err := test.ds.SetCAJournal(ctx, &datastore.CAJournal{
+					ActiveX509AuthorityID: "",
+					Data:                  entriesBytes,
+				})
+				require.NoError(t, err)
+
+				if !j.shouldBePruned {
+					expectedCAJournals = append(expectedCAJournals, caJournal)
+				}
+			}
+
+			require.NoError(t, test.m.PruneCAJournals(ctx))
+			caJournals, err := test.ds.ListCAJournalsForTesting(ctx)
+			require.NoError(t, err)
+			require.ElementsMatch(t, expectedCAJournals, caJournals)
+		})
+	}
+}
+
+func TestRunNotifiesBundleLoaded(t *testing.T) {
 	ctx, cancel := context.WithTimeout(context.Background(), time.Minute)
 	defer cancel()
-	// time out in a minute if the bundle loaded never happens
+
+	test := setupTest(t)
+	test.initAndActivateSelfSignedManager(ctx)
+
 	var actual *common.Bundle
 	test.setNotifier(fakenotifier.New(t, fakenotifier.Config{
 		OnNotifyAndAdviseBundleLoaded: func(bundle *common.Bundle) error {
 			actual = bundle
-			cancel()
 			return nil
 		},
 	}))
 
-	// The bundle loaded handler above will cancel the
-	// context. This will usually be observed as an error returned from the
-	// notifier, but not always, depending on the timing of the cancellation.
-	// When the notifier does not return an error, Run will return without an
-	// error when the canceled context is passed internally to run the tasks.
 	err := test.m.NotifyBundleLoaded(ctx)
-	require.EqualError(t, err, "one or more notifiers returned an error: rpc error: code = Canceled desc = notifier(fake): context canceled")
+	require.NoError(t, err)
 
 	// make sure the event contained the bundle
-	expected := test.fetchBundle(context.Background())
+	expected := test.fetchBundle(ctx)
 	spiretest.RequireProtoEqual(t, expected, actual)
 }
 
@@ -628,10 +992,6 @@ func TestRunFailsIfNotifierFails(t *testing.T) {
 
 	test := setupTest(t)
 	test.initAndActivateSelfSignedManager(ctx)
-	// manager, err := NewManager(ctx, test.selfSignedConfig())
-	// require.NoError(t, err)
-
-	// test.m = manager
 	test.setNotifier(fakenotifier.New(t, fakenotifier.Config{
 		OnNotifyAndAdviseBundleLoaded: func(bundle *common.Bundle) error {
 			return errors.New("ohno")
@@ -669,10 +1029,6 @@ func TestActivationThresholdCap(t *testing.T) {
 }
 
 func TestAlternateKeyTypes(t *testing.T) {
-	upstreamAuthority, _ := fakeupstreamauthority.Load(t, fakeupstreamauthority.Config{
-		TrustDomain: testTrustDomain,
-	})
-
 	expectRSA := func(t *testing.T, signer crypto.Signer, keySize int) {
 		publicKey, ok := signer.Public().(*rsa.PublicKey)
 		t.Logf("PUBLIC KEY TYPE: %T", signer.Public())
@@ -707,7 +1063,7 @@ func TestAlternateKeyTypes(t *testing.T) {
 
 	testCases := []struct {
 		name              string
-		upstreamAuthority upstreamauthority.UpstreamAuthority
+		upstreamAuthority bool
 		x509CAKeyType     keymanager.KeyType
 		jwtKeyType        keymanager.KeyType
 		checkX509CA       func(*testing.T, crypto.Signer)
@@ -750,7 +1106,7 @@ func TestAlternateKeyTypes(t *testing.T) {
 		},
 		{
 			name:              "upstream-signed with RSA 2048",
-			upstreamAuthority: upstreamAuthority,
+			upstreamAuthority: true,
 			x509CAKeyType:     keymanager.RSA2048,
 			jwtKeyType:        keymanager.RSA2048,
 			checkX509CA:       expectRSA2048,
@@ -758,7 +1114,7 @@ func TestAlternateKeyTypes(t *testing.T) {
 		},
 		{
 			name:              "upstream-signed with RSA 4096",
-			upstreamAuthority: upstreamAuthority,
+			upstreamAuthority: true,
 			x509CAKeyType:     keymanager.RSA4096,
 			jwtKeyType:        keymanager.RSA4096,
 			checkX509CA:       expectRSA4096,
@@ -766,7 +1122,7 @@ func TestAlternateKeyTypes(t *testing.T) {
 		},
 		{
 			name:              "upstream-signed with EC P256",
-			upstreamAuthority: upstreamAuthority,
+			upstreamAuthority: true,
 			x509CAKeyType:     keymanager.ECP256,
 			jwtKeyType:        keymanager.ECP256,
 			checkX509CA:       expectEC256,
@@ -774,7 +1130,7 @@ func TestAlternateKeyTypes(t *testing.T) {
 		},
 		{
 			name:              "upstream-signed with EC P384",
-			upstreamAuthority: upstreamAuthority,
+			upstreamAuthority: true,
 			x509CAKeyType:     keymanager.ECP384,
 			jwtKeyType:        keymanager.ECP384,
 			checkX509CA:       expectEC384,
@@ -788,6 +1144,7 @@ func TestAlternateKeyTypes(t *testing.T) {
 			ctx := context.Background()
 
 			test := setupTest(t)
+
 			c := test.selfSignedConfig()
 			c.X509CAKeyType = testCase.x509CAKeyType
 			c.JWTKeyType = testCase.jwtKeyType
@@ -797,7 +1154,12 @@ func TestAlternateKeyTypes(t *testing.T) {
 			test.cat.SetKeyManager(fakeserverkeymanager.New(t))
 
 			// Optionally provide an upstream authority
-			test.cat.SetUpstreamAuthority(testCase.upstreamAuthority)
+			if testCase.upstreamAuthority {
+				upstreamAuthority, _ := test.newFakeUpstreamAuthority(t, fakeupstreamauthority.Config{
+					TrustDomain: testTrustDomain,
+				})
+				test.cat.SetUpstreamAuthority(upstreamAuthority)
+			}
 
 			manager, err := NewManager(ctx, c)
 			require.NoError(t, err)
@@ -805,9 +1167,9 @@ func TestAlternateKeyTypes(t *testing.T) {
 
 			// Prepare and activate a bundle
 			require.NoError(t, test.m.PrepareJWTKey(ctx))
-			test.m.ActivateJWTKey()
+			test.m.ActivateJWTKey(ctx)
 			require.NoError(t, test.m.PrepareX509CA(ctx))
-			test.m.activateX509CA()
+			test.m.activateX509CA(ctx)
 
 			testCase.checkX509CA(t, test.currentX509CA().Signer)
 			testCase.checkJWTKey(t, test.currentJWTKey().Signer)
@@ -849,7 +1211,9 @@ type managerTest struct {
 
 func setupTest(t *testing.T) *managerTest {
 	clock := clock.NewMock(t)
-	ca := new(fakeCA)
+	ca := &fakeCA{
+		taintedAuthoritiesCh: make(chan []*x509.Certificate, 1),
+	}
 
 	log, logHook := test.NewNullLogger()
 	metrics := fakemetrics.New()
@@ -876,6 +1240,11 @@ func setupTest(t *testing.T) *managerTest {
 	}
 }
 
+func (m *managerTest) newFakeUpstreamAuthority(t *testing.T, config fakeupstreamauthority.Config) (upstreamauthority.UpstreamAuthority, *fakeupstreamauthority.UpstreamAuthority) {
+	config.Clock = m.clock
+	return fakeupstreamauthority.Load(t, config)
+}
+
 func (m *managerTest) initSelfSignedManager() {
 	m.cat.SetUpstreamAuthority(nil)
 	manager, err := NewManager(context.Background(), m.selfSignedConfig())
@@ -889,9 +1258,9 @@ func (m *managerTest) initAndActivateSelfSignedManager(ctx context.Context) {
 	require.NoError(m.t, err)
 
 	require.NoError(m.t, manager.PrepareJWTKey(ctx))
-	manager.ActivateJWTKey()
+	manager.ActivateJWTKey(ctx)
 	require.NoError(m.t, manager.PrepareX509CA(ctx))
-	manager.ActivateX509CA()
+	manager.ActivateX509CA(ctx)
 
 	m.m = manager
 }
@@ -914,9 +1283,9 @@ func (m *managerTest) initAndActivateUpstreamSignedManager(ctx context.Context,
 	m.initUpstreamSignedManager(upstreamAuthority)
 
 	require.NoError(m.t, m.m.PrepareJWTKey(ctx))
-	m.m.ActivateJWTKey()
+	m.m.ActivateJWTKey(ctx)
 	require.NoError(m.t, m.m.PrepareX509CA(ctx))
-	m.m.ActivateX509CA()
+	m.m.ActivateX509CA(ctx)
 }
 
 func (m *managerTest) selfSignedConfig() Config {
@@ -944,7 +1313,6 @@ func (m *managerTest) selfSignedConfigWithKeyTypes(x509CAKeyType, jwtKeyType key
 		TrustDomain:   testTrustDomain,
 		X509CAKeyType: x509CAKeyType,
 		JWTKeyType:    jwtKeyType,
-		Dir:           m.dir,
 		Metrics:       m.metrics,
 		Log:           m.log,
 		Clock:         m.clock,
@@ -995,7 +1363,7 @@ func (m *managerTest) getSignerInfo(signer crypto.Signer) signerInfo {
 	}
 }
 
-func (m *managerTest) requireBundleRootCAs(ctx context.Context, t *testing.T, rootCAs ...*x509.Certificate) {
+func (m *managerTest) requireIntermediateRootCA(ctx context.Context, t *testing.T, rootCAs ...*x509.Certificate) {
 	expected := &common.Bundle{}
 	for _, rootCA := range rootCAs {
 		expected.RootCas = append(expected.RootCas, &common.Certificate{
@@ -1009,6 +1377,21 @@ func (m *managerTest) requireBundleRootCAs(ctx context.Context, t *testing.T, ro
 	})
 }
 
+func (m *managerTest) requireBundleRootCAs(ctx context.Context, t *testing.T, rootCAs ...*x509certificate.X509Authority) {
+	expected := &common.Bundle{}
+	for _, rootCA := range rootCAs {
+		expected.RootCas = append(expected.RootCas, &common.Certificate{
+			DerBytes:   rootCA.Certificate.Raw,
+			TaintedKey: rootCA.Tainted,
+		})
+	}
+
+	bundle := m.fetchBundle(ctx)
+	spiretest.RequireProtoEqual(t, expected, &common.Bundle{
+		RootCas: bundle.RootCas,
+	})
+}
+
 func (m *managerTest) requireBundleJWTKeys(ctx context.Context, t *testing.T, jwtKeys ...*ca.JWTKey) {
 	expected := &common.Bundle{}
 	for _, jwtKey := range jwtKeys {
@@ -1088,7 +1471,9 @@ func (m *managerTest) addTimeAndPrune(d time.Duration) {
 }
 
 func (m *managerTest) wipeJournal(t *testing.T) {
-	require.NoError(t, os.Remove(filepath.Join(m.m.c.Dir, "journal.pem")))
+	// Have a clean datastore.
+	m.ds = fakedatastore.New(t)
+	m.cat.SetDataStore(m.ds)
 }
 
 func (m *managerTest) waitForBundleUpdatedNotification(ctx context.Context, ch <-chan *common.Bundle) {
@@ -1130,6 +1515,8 @@ type fakeCA struct {
 	mu     sync.Mutex
 	x509CA *ca.X509CA
 	jwtKey *ca.JWTKey
+
+	taintedAuthoritiesCh chan []*x509.Certificate
 }
 
 func (s *fakeCA) X509CA() *ca.X509CA {
@@ -1155,3 +1542,7 @@ func (s *fakeCA) SetJWTKey(jwtKey *ca.JWTKey) {
 	defer s.mu.Unlock()
 	s.jwtKey = jwtKey
 }
+
+func (s *fakeCA) NotifyTaintedX509Authorities(taintedAuthorities []*x509.Certificate) {
+	s.taintedAuthoritiesCh <- taintedAuthorities
+}
