@@ -4,7 +4,6 @@ import (
 	"context"
 	"crypto/x509"
 	"crypto/x509/pkix"
-	"path/filepath"
 	"testing"
 	"time"
 
@@ -30,7 +29,7 @@ func TestX509CASlotShouldPrepareNext(t *testing.T) {
 	clock := clock.NewMock()
 	now := clock.Now()
 
-	slot := &X509CASlot{
+	slot := &x509CASlot{
 		id:       "A",
 		issuedAt: clock.Now(),
 		x509CA:   nil,
@@ -60,7 +59,7 @@ func TestX509CASlotShouldActivateNext(t *testing.T) {
 	clock := clock.NewMock()
 	now := clock.Now()
 
-	slot := &X509CASlot{
+	slot := &x509CASlot{
 		id:       "A",
 		issuedAt: now,
 		x509CA:   nil,
@@ -90,7 +89,7 @@ func TestJWTKeySlotShouldPrepareNext(t *testing.T) {
 	clock := clock.NewMock()
 	now := clock.Now()
 
-	slot := &JwtKeySlot{
+	slot := &jwtKeySlot{
 		id:       "A",
 		issuedAt: now,
 		jwtKey:   nil,
@@ -116,7 +115,7 @@ func TestJWTKeySlotShouldPrepareNext(t *testing.T) {
 func TestJWTKeySlotShouldActivateNext(t *testing.T) {
 	now := time.Now()
 
-	slot := &JwtKeySlot{
+	slot := &jwtKeySlot{
 		id:       "A",
 		issuedAt: now,
 		jwtKey:   nil,
@@ -181,8 +180,7 @@ func TestJournalLoad(t *testing.T) {
 	jwtKeyBPKIX, err := x509.MarshalPKIXPublicKey(jwtKeyB.Public())
 	require.NoError(t, err)
 
-	tempDir := t.TempDir()
-	journalFile := filepath.Join(tempDir, "journal.pem")
+	activeX509AuthorityID := getOneX509AuthorityID(ctx, t, km)
 
 	// Dates
 	firstIssuedAtUnix := now.Add(-3 * time.Minute).Unix()
@@ -199,7 +197,7 @@ func TestJournalLoad(t *testing.T) {
 
 	for _, tt := range []struct {
 		name        string
-		entries     *JournalEntries
+		entries     *journal.Entries
 		expectSlots map[SlotPosition]Slot
 		expectError string
 		expectLogs  []spiretest.LogEntry
@@ -208,10 +206,10 @@ func TestJournalLoad(t *testing.T) {
 			name:    "Journal has no entries",
 			entries: &journal.Entries{},
 			expectSlots: map[SlotPosition]Slot{
-				CurrentX509CASlot: &X509CASlot{id: "A"},
-				NextX509CASlot:    &X509CASlot{id: "B"},
-				CurrentJWTKeySlot: &JwtKeySlot{id: "A"},
-				NextJWTKeySlot:    &JwtKeySlot{id: "B"},
+				CurrentX509CASlot: &x509CASlot{id: "A"},
+				NextX509CASlot:    &x509CASlot{id: "B"},
+				CurrentJWTKeySlot: &jwtKeySlot{id: "A"},
+				NextJWTKeySlot:    &jwtKeySlot{id: "B"},
 			},
 			expectLogs: []spiretest.LogEntry{
 				{
@@ -226,7 +224,7 @@ func TestJournalLoad(t *testing.T) {
 		},
 		{
 			name: "stored file has a single entry",
-			entries: &JournalEntries{
+			entries: &journal.Entries{
 				X509CAs: []*journal.X509CAEntry{
 					{
 						SlotId:      "B",
@@ -245,7 +243,7 @@ func TestJournalLoad(t *testing.T) {
 				},
 			},
 			expectSlots: map[SlotPosition]Slot{
-				CurrentX509CASlot: &X509CASlot{
+				CurrentX509CASlot: &x509CASlot{
 					id:       "B",
 					issuedAt: secondIssuedAt,
 					status:   journal.Status_UNKNOWN,
@@ -257,8 +255,8 @@ func TestJournalLoad(t *testing.T) {
 					publicKey:   x509KeyB.Public(),
 					notAfter:    x509RootB.NotAfter,
 				},
-				NextX509CASlot: &X509CASlot{id: "A"},
-				CurrentJWTKeySlot: &JwtKeySlot{
+				NextX509CASlot: &x509CASlot{id: "A"},
+				CurrentJWTKeySlot: &jwtKeySlot{
 					id:       "B",
 					issuedAt: secondIssuedAt,
 					status:   journal.Status_UNKNOWN,
@@ -270,7 +268,7 @@ func TestJournalLoad(t *testing.T) {
 					authorityID: "",
 					notAfter:    notAfter,
 				},
-				NextJWTKeySlot: &JwtKeySlot{id: "A"},
+				NextJWTKeySlot: &jwtKeySlot{id: "A"},
 			},
 			expectLogs: []spiretest.LogEntry{
 				{
@@ -285,7 +283,7 @@ func TestJournalLoad(t *testing.T) {
 		},
 		{
 			name: "Stored entries has unknown status",
-			entries: &JournalEntries{
+			entries: &journal.Entries{
 				X509CAs: []*journal.X509CAEntry{
 					{
 						SlotId:      "A",
@@ -334,7 +332,7 @@ func TestJournalLoad(t *testing.T) {
 				},
 			},
 			expectSlots: map[SlotPosition]Slot{
-				CurrentX509CASlot: &X509CASlot{
+				CurrentX509CASlot: &x509CASlot{
 					id:       "B",
 					issuedAt: secondIssuedAt,
 					status:   journal.Status_UNKNOWN,
@@ -346,7 +344,7 @@ func TestJournalLoad(t *testing.T) {
 					authorityID: "2",
 					notAfter:    x509RootB.NotAfter,
 				},
-				NextX509CASlot: &X509CASlot{
+				NextX509CASlot: &x509CASlot{
 					id:       "A",
 					issuedAt: thirdIssuedAt,
 					status:   journal.Status_UNKNOWN,
@@ -358,7 +356,7 @@ func TestJournalLoad(t *testing.T) {
 					authorityID: "1",
 					notAfter:    x509RootA.NotAfter,
 				},
-				CurrentJWTKeySlot: &JwtKeySlot{
+				CurrentJWTKeySlot: &jwtKeySlot{
 					id:       "B",
 					issuedAt: secondIssuedAt,
 					status:   journal.Status_UNKNOWN,
@@ -370,7 +368,7 @@ func TestJournalLoad(t *testing.T) {
 					authorityID: "b",
 					notAfter:    notAfter,
 				},
-				NextJWTKeySlot: &JwtKeySlot{
+				NextJWTKeySlot: &jwtKeySlot{
 					id:       "A",
 					issuedAt: thirdIssuedAt,
 					status:   journal.Status_UNKNOWN,
@@ -396,7 +394,7 @@ func TestJournalLoad(t *testing.T) {
 		},
 		{
 			name: "Stored entry has a single Prepared entry",
-			entries: &JournalEntries{
+			entries: &journal.Entries{
 				X509CAs: []*journal.X509CAEntry{
 					{
 						SlotId:      "A",
@@ -419,7 +417,7 @@ func TestJournalLoad(t *testing.T) {
 				},
 			},
 			expectSlots: map[SlotPosition]Slot{
-				CurrentX509CASlot: &X509CASlot{
+				CurrentX509CASlot: &x509CASlot{
 					id:       "A",
 					issuedAt: thirdIssuedAt,
 					status:   journal.Status_PREPARED,
@@ -431,10 +429,10 @@ func TestJournalLoad(t *testing.T) {
 					authorityID: "1",
 					notAfter:    x509RootA.NotAfter,
 				},
-				NextX509CASlot: &X509CASlot{
+				NextX509CASlot: &x509CASlot{
 					id: "B",
 				},
-				CurrentJWTKeySlot: &JwtKeySlot{
+				CurrentJWTKeySlot: &jwtKeySlot{
 					id:       "A",
 					issuedAt: thirdIssuedAt,
 					status:   journal.Status_PREPARED,
@@ -446,7 +444,7 @@ func TestJournalLoad(t *testing.T) {
 					authorityID: "a",
 					notAfter:    notAfter,
 				},
-				NextJWTKeySlot: &JwtKeySlot{
+				NextJWTKeySlot: &jwtKeySlot{
 					id: "B",
 				},
 			},
@@ -463,7 +461,7 @@ func TestJournalLoad(t *testing.T) {
 		},
 		{
 			name: "Stored entries has old and active",
-			entries: &JournalEntries{
+			entries: &journal.Entries{
 				X509CAs: []*journal.X509CAEntry{
 					{
 						SlotId:      "A",
@@ -518,7 +516,7 @@ func TestJournalLoad(t *testing.T) {
 				},
 			},
 			expectSlots: map[SlotPosition]Slot{
-				CurrentX509CASlot: &X509CASlot{
+				CurrentX509CASlot: &x509CASlot{
 					id:       "A",
 					issuedAt: thirdIssuedAt,
 					status:   journal.Status_ACTIVE,
@@ -530,7 +528,7 @@ func TestJournalLoad(t *testing.T) {
 					publicKey:   x509KeyA.Public(),
 					notAfter:    x509RootA.NotAfter,
 				},
-				NextX509CASlot: &X509CASlot{
+				NextX509CASlot: &x509CASlot{
 					id:       "B",
 					issuedAt: secondIssuedAt,
 					status:   journal.Status_OLD,
@@ -542,7 +540,7 @@ func TestJournalLoad(t *testing.T) {
 					publicKey:   x509KeyB.Public(),
 					notAfter:    x509RootB.NotAfter,
 				},
-				CurrentJWTKeySlot: &JwtKeySlot{
+				CurrentJWTKeySlot: &jwtKeySlot{
 					id:       "A",
 					issuedAt: thirdIssuedAt,
 					status:   journal.Status_ACTIVE,
@@ -554,7 +552,7 @@ func TestJournalLoad(t *testing.T) {
 					authorityID: "a",
 					notAfter:    notAfter,
 				},
-				NextJWTKeySlot: &JwtKeySlot{
+				NextJWTKeySlot: &jwtKeySlot{
 					id:       "B",
 					issuedAt: secondIssuedAt,
 					status:   journal.Status_OLD,
@@ -580,9 +578,9 @@ func TestJournalLoad(t *testing.T) {
 		},
 		{
 			name: "There are another entries before Active entry",
-			entries: &JournalEntries{
+			entries: &journal.Entries{
 				X509CAs: []*journal.X509CAEntry{
-					// This can happens when force rotation is executed
+					// This can happen when force rotation is executed
 					{
 						SlotId:      "A",
 						IssuedAt:    firstIssuedAtUnix,
@@ -606,7 +604,7 @@ func TestJournalLoad(t *testing.T) {
 					},
 				},
 				JwtKeys: []*journal.JWTKeyEntry{
-					// This can happens when force rotation is executed
+					// This can happen when force rotation is executed
 					{
 						SlotId:      "A",
 						IssuedAt:    firstIssuedAtUnix,
@@ -637,7 +635,7 @@ func TestJournalLoad(t *testing.T) {
 				},
 			},
 			expectSlots: map[SlotPosition]Slot{
-				CurrentX509CASlot: &X509CASlot{
+				CurrentX509CASlot: &x509CASlot{
 					id:       "A",
 					issuedAt: firstIssuedAt,
 					status:   journal.Status_ACTIVE,
@@ -649,7 +647,7 @@ func TestJournalLoad(t *testing.T) {
 					authorityID: "3",
 					notAfter:    x509RootA.NotAfter,
 				},
-				NextX509CASlot: &X509CASlot{
+				NextX509CASlot: &x509CASlot{
 					id:       "B",
 					issuedAt: thirdIssuedAt,
 					status:   journal.Status_PREPARED,
@@ -661,7 +659,7 @@ func TestJournalLoad(t *testing.T) {
 					authorityID: "1",
 					notAfter:    x509RootB.NotAfter,
 				},
-				CurrentJWTKeySlot: &JwtKeySlot{
+				CurrentJWTKeySlot: &jwtKeySlot{
 					id:       "A",
 					issuedAt: firstIssuedAt,
 					status:   journal.Status_ACTIVE,
@@ -673,7 +671,7 @@ func TestJournalLoad(t *testing.T) {
 					authorityID: "c",
 					notAfter:    notAfter,
 				},
-				NextJWTKeySlot: &JwtKeySlot{
+				NextJWTKeySlot: &jwtKeySlot{
 					id:       "B",
 					issuedAt: thirdIssuedAt,
 					status:   journal.Status_PREPARED,
@@ -702,11 +700,12 @@ func TestJournalLoad(t *testing.T) {
 			entries: &journal.Entries{
 				X509CAs: []*journal.X509CAEntry{
 					{
-						SlotId:      "A",
-						IssuedAt:    firstIssuedAtUnix,
-						Certificate: []byte("foo"),
-						Status:      journal.Status_ACTIVE,
-						AuthorityId: "1",
+						SlotId:              "A",
+						IssuedAt:            firstIssuedAtUnix,
+						Certificate:         []byte("foo"),
+						Status:              journal.Status_ACTIVE,
+						AuthorityId:         "1",
+						UpstreamAuthorityId: "2",
 					},
 				},
 			},
@@ -724,11 +723,12 @@ func TestJournalLoad(t *testing.T) {
 					Level:   logrus.ErrorLevel,
 					Message: "X509CA slot failed to load",
 					Data: logrus.Fields{
-						logrus.ErrorKey:            "unable to parse CA certificate: x509: malformed certificate",
-						telemetry.IssuedAt:         firstIssuedAt.String(),
-						telemetry.Slot:             "A",
-						telemetry.Status:           "ACTIVE",
-						telemetry.LocalAuthorityID: "1",
+						logrus.ErrorKey:               "unable to parse CA certificate: x509: malformed certificate",
+						telemetry.IssuedAt:            firstIssuedAt.String(),
+						telemetry.Slot:                "A",
+						telemetry.Status:              "ACTIVE",
+						telemetry.LocalAuthorityID:    "1",
+						telemetry.UpstreamAuthorityID: "2",
 					},
 				},
 			},
@@ -774,18 +774,24 @@ func TestJournalLoad(t *testing.T) {
 	} {
 		t.Run(tt.name, func(t *testing.T) {
 			loghook.Reset()
-			err = saveJournalEntries(journalFile, tt.entries)
+			journal := new(Journal)
+			journal.config = &journalConfig{
+				cat: cat,
+				log: log,
+			}
+			journal.setEntries(tt.entries)
+			journal.activeX509AuthorityID = activeX509AuthorityID
+			err = journal.save(ctx)
 			require.NoError(t, err)
 
 			loader := &SlotLoader{
 				TrustDomain: td,
 				Log:         log,
-				Dir:         tempDir,
 				Catalog:     cat,
 			}
 
-			loadedJournal, slots, err := loader.Load(context.Background())
-			spiretest.AssertLogs(t, loghook.AllEntries(), tt.expectLogs)
+			loadedJournal, slots, err := loader.load(ctx)
+			spiretest.AssertLastLogs(t, loghook.AllEntries(), tt.expectLogs)
 			if tt.expectError != "" {
 				spiretest.AssertErrorPrefix(t, err, tt.expectError)
 				assert.Nil(t, loadedJournal)
