@@ -8,16 +8,16 @@ import (
 	"encoding/asn1"
 	"errors"
 	"fmt"
+	"math"
 	"math/big"
 	"net/url"
 	"testing"
 	"time"
 
-	"github.com/go-jose/go-jose/v3/jwt"
+	"github.com/go-jose/go-jose/v4/jwt"
 	"github.com/spiffe/go-spiffe/v2/spiffeid"
 	credentialcomposerv1 "github.com/spiffe/spire-plugin-sdk/proto/spire/plugin/server/credentialcomposer/v1"
 	"github.com/spiffe/spire/pkg/common/catalog"
-	"github.com/spiffe/spire/pkg/common/x509svid"
 	"github.com/spiffe/spire/pkg/common/x509util"
 	"github.com/spiffe/spire/pkg/server/credtemplate"
 	"github.com/spiffe/spire/pkg/server/plugin/credentialcomposer"
@@ -68,7 +68,7 @@ func TestNewBuilderSetsDefaults(t *testing.T) {
 
 	// Assert that the Clock and NewSerialNumber are not nil and then set them
 	// to nil before comparing the whole config. Checking the whole config in a
-	// single equality check is more future proof but the defaults for these
+	// single equality check is more future-proof but the defaults for these
 	// fields are hard to compare.
 	assert.NotNil(t, config.Clock)
 	config.Clock = nil
@@ -76,15 +76,14 @@ func TestNewBuilderSetsDefaults(t *testing.T) {
 	config.NewSerialNumber = nil
 
 	assert.Equal(t, credtemplate.Config{
-		TrustDomain:            td,
-		X509CASubject:          credtemplate.DefaultX509CASubject(),
-		X509CATTL:              credtemplate.DefaultX509CATTL,
-		X509SVIDSubject:        credtemplate.DefaultX509SVIDSubject(),
-		X509SVIDTTL:            credtemplate.DefaultX509SVIDTTL,
-		JWTSVIDTTL:             credtemplate.DefaultJWTSVIDTTL,
-		JWTIssuer:              "",
-		AgentSVIDTTL:           credtemplate.DefaultX509SVIDTTL,
-		ExcludeSNFromCASubject: false,
+		TrustDomain:     td,
+		X509CASubject:   credtemplate.DefaultX509CASubject(),
+		X509CATTL:       credtemplate.DefaultX509CATTL,
+		X509SVIDSubject: credtemplate.DefaultX509SVIDSubject(),
+		X509SVIDTTL:     credtemplate.DefaultX509SVIDTTL,
+		JWTSVIDTTL:      credtemplate.DefaultJWTSVIDTTL,
+		JWTIssuer:       "",
+		AgentSVIDTTL:    credtemplate.DefaultX509SVIDTTL,
 	}, config)
 }
 
@@ -106,7 +105,7 @@ func TestNewBuilderAllowsConfigOverrides(t *testing.T) {
 
 	// Assert that the Clock and NewSerialNumber are not nil and then set them
 	// to nil before comparing the whole config. Checking the whole config in a
-	// single equality check is more future proof but the defaults for these
+	// single equality check is more future-proof but the defaults for these
 	// fields are hard to compare.
 	assert.NotNil(t, configOut.Clock)
 	configOut.Clock = nil
@@ -150,15 +149,6 @@ func TestBuildSelfSignedX509CATemplate(t *testing.T) {
 				expected.NotAfter = now.Add(time.Minute * 23)
 			},
 		},
-		{
-			desc: "exclude serial number from subject",
-			overrideConfig: func(config *credtemplate.Config) {
-				config.ExcludeSNFromCASubject = true
-			},
-			overrideExpected: func(expected *x509.Certificate) {
-				expected.Subject = pkix.Name{Country: []string{"US"}, Organization: []string{"SPIFFE"}}
-			},
-		},
 		{
 			desc: "override X509CASubject",
 			overrideConfig: func(config *credtemplate.Config) {
@@ -274,15 +264,6 @@ func TestBuildUpstreamSignedX509CACSR(t *testing.T) {
 			},
 			expectErr: "x509: unsupported public key type: <nil>",
 		},
-		{
-			desc: "exclude serial number from subject",
-			overrideConfig: func(config *credtemplate.Config) {
-				config.ExcludeSNFromCASubject = true
-			},
-			overrideExpected: func(expected *x509.CertificateRequest) {
-				expected.Subject = pkix.Name{Country: []string{"US"}, Organization: []string{"SPIFFE"}}
-			},
-		},
 		{
 			desc: "override X509CASubject",
 			overrideConfig: func(config *credtemplate.Config) {
@@ -405,10 +386,20 @@ func TestBuildDownstreamX509CATemplate(t *testing.T) {
 			},
 		},
 		{
-			desc: "override X509SVIDTTL",
+			desc: "default legacy TTL",
+			overrideConfig: func(config *credtemplate.Config) {
+				config.UseLegacyDownstreamX509CATTL = true
+			},
+			overrideExpected: func(expected *x509.Certificate) {
+				expected.NotAfter = now.Add(credtemplate.DefaultX509SVIDTTL)
+			},
+		},
+		{
+			desc: "override X509SVIDTTL with legacy TTL",
 			overrideConfig: func(config *credtemplate.Config) {
 				// Downstream CAs have historically been signed using the default X509-SVID TTL
 				config.X509SVIDTTL = credtemplate.DefaultX509SVIDTTL * 2
+				config.UseLegacyDownstreamX509CATTL = true
 			},
 			overrideExpected: func(expected *x509.Certificate) {
 				expected.NotAfter = now.Add(credtemplate.DefaultX509SVIDTTL * 2)
@@ -495,8 +486,7 @@ func TestBuildDownstreamX509CATemplate(t *testing.T) {
 					SubjectKeyId:          publicKeyID,
 					AuthorityKeyId:        parentKeyID,
 					NotBefore:             notBefore,
-					// Downstream CAs have historically been signed using the default X509-SVID TTL
-					NotAfter: x509SVIDNotAfter,
+					NotAfter:              x509CANotAfter,
 				}
 				if tc.overrideExpected != nil {
 					tc.overrideExpected(expected)
@@ -565,7 +555,6 @@ func TestBuildServerX509SVIDTemplate(t *testing.T) {
 			overrideExpected: func(expected *x509.Certificate) {
 				expected.Subject = pkix.Name{
 					CommonName: "OVERRIDE",
-					ExtraNames: []pkix.AttributeTypeAndValue{x509svid.UniqueIDAttribute(serverID)},
 				}
 			},
 		},
@@ -626,7 +615,6 @@ func TestBuildServerX509SVIDTemplate(t *testing.T) {
 					Subject: pkix.Name{
 						Country:      []string{"US"},
 						Organization: []string{"SPIRE"},
-						ExtraNames:   []pkix.AttributeTypeAndValue{x509svid.UniqueIDAttribute(serverID)},
 					},
 					SubjectKeyId:          publicKeyID,
 					AuthorityKeyId:        parentKeyID,
@@ -730,7 +718,6 @@ func TestBuildAgentX509SVIDTemplate(t *testing.T) {
 			overrideExpected: func(expected *x509.Certificate) {
 				expected.Subject = pkix.Name{
 					CommonName: "OVERRIDE",
-					ExtraNames: []pkix.AttributeTypeAndValue{x509svid.UniqueIDAttribute(agentID)},
 				}
 			},
 		},
@@ -792,7 +779,6 @@ func TestBuildAgentX509SVIDTemplate(t *testing.T) {
 					Subject: pkix.Name{
 						Country:      []string{"US"},
 						Organization: []string{"SPIRE"},
-						ExtraNames:   []pkix.AttributeTypeAndValue{x509svid.UniqueIDAttribute(agentID)},
 					},
 					SubjectKeyId:          publicKeyID,
 					AuthorityKeyId:        parentKeyID,
@@ -893,7 +879,6 @@ func TestBuildWorkloadX509SVIDTemplate(t *testing.T) {
 			overrideExpected: func(expected *x509.Certificate) {
 				expected.Subject = pkix.Name{
 					CommonName: "OVERRIDE",
-					ExtraNames: []pkix.AttributeTypeAndValue{x509svid.UniqueIDAttribute(workloadID)},
 				}
 			},
 		},
@@ -920,7 +905,6 @@ func TestBuildWorkloadX509SVIDTemplate(t *testing.T) {
 				// Subject is explicit.
 				expected.Subject = pkix.Name{
 					CommonName: "DNSNAME1",
-					ExtraNames: []pkix.AttributeTypeAndValue{x509svid.UniqueIDAttribute(workloadID)},
 				}
 			},
 		},
@@ -940,7 +924,6 @@ func TestBuildWorkloadX509SVIDTemplate(t *testing.T) {
 				// allowed to override.
 				expected.Subject = pkix.Name{
 					CommonName: "OVERRIDE-1",
-					ExtraNames: []pkix.AttributeTypeAndValue{x509svid.UniqueIDAttribute(workloadID)},
 				}
 			},
 		},
@@ -1021,7 +1004,6 @@ func TestBuildWorkloadX509SVIDTemplate(t *testing.T) {
 					Subject: pkix.Name{
 						Country:      []string{"US"},
 						Organization: []string{"SPIRE"},
-						ExtraNames:   []pkix.AttributeTypeAndValue{x509svid.UniqueIDAttribute(workloadID)},
 					},
 					SubjectKeyId:          publicKeyID,
 					AuthorityKeyId:        parentKeyID,
@@ -1170,6 +1152,43 @@ func TestBuildWorkloadJWTSVIDClaims(t *testing.T) {
 				config.CredentialComposers = []credentialcomposer.CredentialComposer{loadNoopV1Plugin(t)}
 			},
 		},
+		{
+			desc: "real grpc composer",
+			overrideConfig: func(config *credtemplate.Config) {
+				config.CredentialComposers = []credentialcomposer.CredentialComposer{loadGrpcPlugin(t)}
+			},
+			overrideExpected: func(expected map[string]any) {
+				expected["aud"] = []interface{}{"AUDIENCE"}
+				expected["iat"] = now.Unix()
+				expected["exp"] = jwtSVIDNotAfter.Unix()
+			},
+		},
+		{
+			desc: "real grpc composer overriding first composer",
+			overrideConfig: func(config *credtemplate.Config) {
+				config.CredentialComposers = []credentialcomposer.CredentialComposer{fakeCC{id: 1, onlyFoo: true, addInt64: true}, loadGrpcPlugin(t)}
+			},
+			overrideExpected: func(expected map[string]any) {
+				expected["aud"] = []interface{}{"AUDIENCE"}
+				expected["iat"] = now.Unix()
+				expected["exp"] = jwtSVIDNotAfter.Unix()
+				expected["foo"] = "VALUE-1"
+				expected["i64"] = float64(math.MaxInt64)
+			},
+		},
+		{
+			desc: "real grpc composer with second composer",
+			overrideConfig: func(config *credtemplate.Config) {
+				config.CredentialComposers = []credentialcomposer.CredentialComposer{loadGrpcPlugin(t), fakeCC{id: 1, onlyFoo: true, addInt64: true}}
+			},
+			overrideExpected: func(expected map[string]any) {
+				expected["aud"] = []interface{}{"AUDIENCE"}
+				expected["iat"] = now.Unix()
+				expected["exp"] = jwtSVIDNotAfter.Unix()
+				expected["foo"] = "VALUE-1"
+				expected["i64"] = math.MaxInt64
+			},
+		},
 	} {
 		t.Run(tc.desc, func(t *testing.T) {
 			testBuilder(t, tc.overrideConfig, func(t *testing.T, credBuilder *credtemplate.Builder) {
@@ -1248,6 +1267,7 @@ type fakeCC struct {
 	id             byte
 	onlyCommonName bool
 	onlyFoo        bool
+	addInt64       bool
 }
 
 func (cc fakeCC) ComposeServerX509CA(_ context.Context, attributes credentialcomposer.X509CAAttributes) (credentialcomposer.X509CAAttributes, error) {
@@ -1276,6 +1296,9 @@ func (cc fakeCC) ComposeWorkloadJWTSVID(_ context.Context, _ spiffeid.ID, attrib
 	if !cc.onlyFoo {
 		attributes.Claims["bar"] = cc.applySuffix("VALUE")
 	}
+	if cc.addInt64 {
+		attributes.Claims["i64"] = math.MaxInt64
+	}
 	return attributes, nil
 }
 
@@ -1306,3 +1329,20 @@ func loadNoopV1Plugin(t *testing.T) credentialcomposer.CredentialComposer {
 	plugintest.Load(t, catalog.MakeBuiltIn("noop", server), cc)
 	return cc
 }
+
+type grpcPlugin struct {
+	credentialcomposerv1.UnimplementedCredentialComposerServer
+}
+
+func (p grpcPlugin) ComposeWorkloadJWTSVID(_ context.Context, a *credentialcomposerv1.ComposeWorkloadJWTSVIDRequest) (*credentialcomposerv1.ComposeWorkloadJWTSVIDResponse, error) {
+	return &credentialcomposerv1.ComposeWorkloadJWTSVIDResponse{
+		Attributes: a.Attributes,
+	}, nil
+}
+
+func loadGrpcPlugin(t *testing.T) credentialcomposer.CredentialComposer {
+	server := credentialcomposerv1.CredentialComposerPluginServer(grpcPlugin{})
+	cc := new(credentialcomposer.V1)
+	plugintest.Load(t, catalog.MakeBuiltIn("grpcPlugin", server), cc)
+	return cc
+}
