@@ -212,6 +212,42 @@ import (
 // | v1.8.5  |        |                                                                           |
 // |---------|        |                                                                           |
 // | v1.8.6  |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.8.7  |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.8.8  |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.8.9  |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.8.10 |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.8.11 |        |                                                                           |
+// |*********|********|***************************************************************************|
+// | v1.9.0  |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.9.1  |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.9.2  |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.9.3  |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.9.4  |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.9.5  |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.9.6  |        |                                                                           |
+// |*********|********|***************************************************************************|
+// | v1.10.0 |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.10.1 |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.10.2 |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.10.3 |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.10.4 |        |                                                                           |
+// |*********|********|***************************************************************************|
+// | v1.11.0 |        |                                                                           |
 // ================================================================================================
 
 const (
@@ -221,7 +257,7 @@ const (
 	// lastMinorReleaseSchemaVersion is the schema version supported by the
 	// last minor release. When the migrations are opportunistically pruned
 	// from the code after a minor release, this number should be updated.
-	lastMinorReleaseSchemaVersion = 21
+	lastMinorReleaseSchemaVersion = 23
 )
 
 // the current code version
@@ -329,7 +365,7 @@ func migrateDB(db *gorm.DB, dbType string, disableMigration bool, log logrus.Fie
 }
 
 func isDisabledMigrationAllowed(thisCodeVersion, dbCodeVersion semver.Version) error {
-	// If auto-migrate is disabled and we are running a compatible version (+/- 1
+	// If auto-migrate is disabled, and we are running a compatible version (+/- 1
 	// minor from the stored code version) then we are done here
 	if !isCompatibleCodeVersion(thisCodeVersion, dbCodeVersion) {
 		return errors.New("auto-migration must be enabled for current DB")
@@ -409,7 +445,7 @@ func tableOptionsForDialect(tx *gorm.DB, dbType string) *gorm.DB {
 	// This allows for setting table options for a particular DB type.
 	// For MySQL, (for compatibility reasons) we want to make sure that
 	// we can support indexes on strings (varchar(255) in the DB).
-	if dbType == MySQL {
+	if isMySQLDbType(dbType) {
 		return tx.Set("gorm:table_options", "ENGINE=InnoDB  ROW_FORMAT=DYNAMIC DEFAULT CHARSET=utf8")
 	}
 	return tx
@@ -433,13 +469,23 @@ func migrateVersion(tx *gorm.DB, currVersion int, log logrus.FieldLogger) (versi
 	// Place all migrations handled by the current minor release here. This
 	// list can be opportunistically pruned after every minor release but won't
 	// break things if it isn't.
-	switch currVersion {
-	case 21:
-		// TODO: remove this migration in 1.9.0
-		err = migrateToV22(tx)
-	case 22:
-		// TODO: remove this migration in 1.9.0
-		err = migrateToV23(tx)
+	//
+	// When adding a supported migration to version XX, add a case and the
+	// corresponding function. The case in the following switch statement will
+	// look like this:
+	//
+	// case XX:
+	//   err = migrateToVXX(tx)
+	//
+	// And the migrateToVXX function will be like this:
+	// func migrateToVXX(tx *gorm.DB) error {
+	//   if err := tx.AutoMigrate(&Foo{}, &Bar{}).Error; err != nil {
+	//     return sqlError.Wrap(err)
+	//   }
+	//   return nil
+	// }
+	//
+	switch currVersion { //nolint: gocritic // No upgrade required yet, keeping switch for future additions
 	default:
 		err = sqlError.New("no migration support for unknown schema version %d", currVersion)
 	}
@@ -450,20 +496,6 @@ func migrateVersion(tx *gorm.DB, currVersion int, log logrus.FieldLogger) (versi
 	return nextVersion, nil
 }
 
-func migrateToV22(tx *gorm.DB) error {
-	if err := tx.AutoMigrate(&RegisteredEntryEvent{}, &AttestedNodeEvent{}).Error; err != nil {
-		return sqlError.Wrap(err)
-	}
-	return nil
-}
-
-func migrateToV23(tx *gorm.DB) error {
-	if err := tx.AutoMigrate(&CAJournal{}).Error; err != nil {
-		return sqlError.Wrap(err)
-	}
-	return nil
-}
-
 func addFederatedRegistrationEntriesRegisteredEntryIDIndex(tx *gorm.DB) error {
 	// GORM creates the federated_registration_entries implicitly with a primary
 	// key tuple (bundle_id, registered_entry_id). Unfortunately, MySQL5 does
