@@ -19,8 +19,10 @@ import (
 	"github.com/aws/aws-sdk-go-v2/service/kms/types"
 	"github.com/sirupsen/logrus"
 	"github.com/sirupsen/logrus/hooks/test"
+	"github.com/spiffe/go-spiffe/v2/spiffeid"
 	keymanagerv1 "github.com/spiffe/spire-plugin-sdk/proto/spire/plugin/server/keymanager/v1"
 	configv1 "github.com/spiffe/spire-plugin-sdk/proto/spire/service/common/config/v1"
+	"github.com/spiffe/spire/pkg/common/catalog"
 	"github.com/spiffe/spire/pkg/server/plugin/keymanager"
 	keymanagertest "github.com/spiffe/spire/pkg/server/plugin/keymanager/test"
 	"github.com/spiffe/spire/test/plugintest"
@@ -91,14 +93,17 @@ func TestKeyManagerContract(t *testing.T) {
 			func(aws.Config) (stsClient, error) { return fakeSTSClient, nil },
 		)
 		km := new(keymanager.V1)
-		keyMetadataFile := filepath.Join(dir, "metadata.json")
+		keyIdentifierFile := filepath.Join(dir, "metadata")
 		if isWindows {
-			keyMetadataFile = filepath.ToSlash(keyMetadataFile)
+			keyIdentifierFile = filepath.ToSlash(keyIdentifierFile)
 		}
-		plugintest.Load(t, builtin(p), km, plugintest.Configuref(`
+		plugintest.Load(t, builtin(p), km, plugintest.CoreConfig(catalog.CoreConfig{
+			TrustDomain: spiffeid.RequireTrustDomainFromString("example.org"),
+		}),
+			plugintest.Configuref(`
 			region = "fake-region"
-			key_metadata_file = %q
-		`, keyMetadataFile))
+			key_identifier_file = %q
+		`, keyIdentifierFile))
 		return km
 	}
 
@@ -164,42 +169,42 @@ func TestConfigure(t *testing.T) {
 				{
 					AliasName: aws.String(aliasName),
 					KeyID:     aws.String(keyID),
-					KeySpec:   types.CustomerMasterKeySpecRsa4096,
+					KeySpec:   types.KeySpecRsa4096,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
 				{
 					AliasName: aws.String(aliasName + "01"),
 					KeyID:     aws.String(keyID + "01"),
-					KeySpec:   types.CustomerMasterKeySpecRsa2048,
+					KeySpec:   types.KeySpecRsa2048,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
 				{
 					AliasName: aws.String(aliasName + "02"),
 					KeyID:     aws.String(keyID + "02"),
-					KeySpec:   types.CustomerMasterKeySpecRsa4096,
+					KeySpec:   types.KeySpecRsa4096,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
 				{
 					AliasName: aws.String(aliasName + "03"),
 					KeyID:     aws.String(keyID + "03"),
-					KeySpec:   types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:   types.KeySpecEccNistP256,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
 				{
 					AliasName: aws.String(aliasName + "04"),
 					KeyID:     aws.String(keyID + "04"),
-					KeySpec:   types.CustomerMasterKeySpecEccNistP384,
+					KeySpec:   types.KeySpecEccNistP384,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
 				{
 					AliasName: aws.String("alias/SPIRE_SERVER/wrong_prefix"),
 					KeyID:     aws.String("foo_id"),
-					KeySpec:   types.CustomerMasterKeySpecEccNistP384,
+					KeySpec:   types.KeySpecEccNistP384,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
@@ -209,39 +214,71 @@ func TestConfigure(t *testing.T) {
 			name:             "pass without keys",
 			configureRequest: configureRequestWithDefaults(t),
 		},
+		{
+			name:             "pass with key identifier file",
+			configureRequest: configureRequestWithVars("", "secret_access_key", "region", KeyIdentifierFile, getKeyIdentifierFile(t), ""),
+		},
+		{
+			name:             "pass with key identifier value",
+			configureRequest: configureRequestWithVars("", "secret_access_key", "region", KeyIdentifierValue, "server-id", ""),
+		},
 		{
 			name:             "missing access key id",
-			configureRequest: configureRequestWithVars("", "secret_access_key", "region", getKeyMetadataFile(t), ""),
+			configureRequest: configureRequestWithVars("", "secret_access_key", "region", KeyIdentifierFile, getKeyIdentifierFile(t), ""),
 		},
 		{
 			name:             "missing secret access key",
-			configureRequest: configureRequestWithVars("access_key", "", "region", getKeyMetadataFile(t), ""),
+			configureRequest: configureRequestWithVars("access_key", "", "region", KeyIdentifierFile, getKeyIdentifierFile(t), ""),
 		},
 		{
 			name:             "missing region",
-			configureRequest: configureRequestWithVars("access_key_id", "secret_access_key", "", getKeyMetadataFile(t), ""),
+			configureRequest: configureRequestWithVars("access_key_id", "secret_access_key", "", KeyIdentifierFile, getKeyIdentifierFile(t), ""),
 			err:              "configuration is missing a region",
 			code:             codes.InvalidArgument,
 		},
 		{
-			name:             "missing server id file path",
-			configureRequest: configureRequestWithVars("access_key_id", "secret_access_key", "region", "", ""),
-			err:              "configuration is missing server id file path",
+			name:             "missing key identifier file and key identifier value",
+			configureRequest: configureRequestWithVars("access_key_id", "secret_access_key", "region", KeyIdentifierFile, "", ""),
+			err:              "configuration requires a key identifier file or a key identifier value",
+			code:             codes.InvalidArgument,
+		},
+		{
+			name:             "both key identifier file and key identifier value",
+			configureRequest: configureRequestWithString(`{"access_key_id":"access_key_id","secret_access_key":"secret_access_key","region":"region","key_identifier_file":"key_identifier_file","key_identifier_value":"key_identifier_value","key_policy_file":""}`),
+			err:              "configuration can't have a key identifier file and a key identifier value at the same time",
+			code:             codes.InvalidArgument,
+		},
+		{
+			name:             "key identifier value invalid character",
+			configureRequest: configureRequestWithString(`{"access_key_id":"access_key_id","secret_access_key":"secret_access_key","region":"region","key_identifier_value":"@key_identifier_value@","key_policy_file":""}`),
+			err:              "Key identifier must contain only alphanumeric characters, forward slashes (/), underscores (_), and dashes (-)",
+			code:             codes.InvalidArgument,
+		},
+		{
+			name:             "key identifier value too long",
+			configureRequest: configureRequestWithString(`{"access_key_id":"access_key_id","secret_access_key":"secret_access_key","region":"region","key_identifier_value":"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA","key_policy_file":""}`),
+			err:              "Key identifier must not be longer than 256 characters",
+			code:             codes.InvalidArgument,
+		},
+		{
+			name:             "key identifier value starts with illegal alias",
+			configureRequest: configureRequestWithString(`{"access_key_id":"access_key_id","secret_access_key":"secret_access_key","region":"region","key_identifier_value":"alias/aws/key_identifier_value","key_policy_file":""}`),
+			err:              "Key identifier must not start with alias/aws/",
 			code:             codes.InvalidArgument,
 		},
 		{
 			name:             "custom policy file does not exists",
-			configureRequest: configureRequestWithVars("access_key", "secret_access_key", "region", getEmptyKeyMetadataFile(t), "non-existent-file.json"),
+			configureRequest: configureRequestWithVars("access_key", "secret_access_key", "region", KeyIdentifierFile, getEmptyKeyIdentifierFile(t), "non-existent-file.json"),
 			err:              fmt.Sprintf("failed to read file configured in 'key_policy_file': open non-existent-file.json: %s", spiretest.FileNotFound()),
 			code:             codes.Internal,
 		},
 		{
 			name:             "use custom policy file",
-			configureRequest: configureRequestWithVars("access_key", "secret_access_key", "region", getEmptyKeyMetadataFile(t), getCustomPolicyFile(t)),
+			configureRequest: configureRequestWithVars("access_key", "secret_access_key", "region", KeyIdentifierFile, getEmptyKeyIdentifierFile(t), getCustomPolicyFile(t)),
 		},
 		{
 			name:             "new server id file path",
-			configureRequest: configureRequestWithVars("access_key_id", "secret_access_key", "region", getEmptyKeyMetadataFile(t), ""),
+			configureRequest: configureRequestWithVars("access_key_id", "secret_access_key", "region", KeyIdentifierFile, getEmptyKeyIdentifierFile(t), ""),
 		},
 		{
 			name:             "decode error",
@@ -251,21 +288,21 @@ func TestConfigure(t *testing.T) {
 		},
 		{
 			name:             "list aliases error",
+			configureRequest: configureRequestWithDefaults(t),
 			err:              "failed to fetch aliases: fake list aliases error",
 			code:             codes.Internal,
-			configureRequest: configureRequestWithDefaults(t),
 			listAliasesErr:   "fake list aliases error",
 		},
 		{
 			name:             "describe key error",
+			configureRequest: configureRequestWithDefaults(t),
 			err:              "failed to describe key: describe key error",
 			code:             codes.Internal,
-			configureRequest: configureRequestWithDefaults(t),
 			fakeEntries: []fakeKeyEntry{
 				{
 					AliasName: aws.String(aliasName),
 					KeyID:     aws.String(keyID),
-					KeySpec:   types.CustomerMasterKeySpecRsa2048,
+					KeySpec:   types.KeySpecRsa2048,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
@@ -274,9 +311,9 @@ func TestConfigure(t *testing.T) {
 		},
 		{
 			name:             "unsupported key error",
+			configureRequest: configureRequestWithDefaults(t),
 			err:              "unsupported key spec: unsupported key spec",
 			code:             codes.Internal,
-			configureRequest: configureRequestWithDefaults(t),
 			fakeEntries: []fakeKeyEntry{
 				{
 					AliasName: aws.String(aliasName),
@@ -289,14 +326,14 @@ func TestConfigure(t *testing.T) {
 		},
 		{
 			name:             "get public key error",
+			configureRequest: configureRequestWithDefaults(t),
 			err:              "failed to fetch aliases: failed to get public key: get public key error",
 			code:             codes.Internal,
-			configureRequest: configureRequestWithDefaults(t),
 			fakeEntries: []fakeKeyEntry{
 				{
 					AliasName: aws.String(aliasName),
 					KeyID:     aws.String(keyID),
-					KeySpec:   types.CustomerMasterKeySpecRsa4096,
+					KeySpec:   types.KeySpecRsa4096,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
@@ -306,14 +343,14 @@ func TestConfigure(t *testing.T) {
 
 		{
 			name:             "disabled key",
+			configureRequest: configureRequestWithDefaults(t),
 			err:              "failed to fetch aliases: found disabled SPIRE key: \"arn:aws:kms:region:1234:key/abcd-fghi\", alias: \"arn:aws:kms:region:1234:alias/SPIRE_SERVER/test_example_org/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/spireKeyID\"",
 			code:             codes.FailedPrecondition,
-			configureRequest: configureRequestWithDefaults(t),
 			fakeEntries: []fakeKeyEntry{
 				{
 					AliasName: aws.String(aliasName),
 					KeyID:     aws.String(keyID),
-					KeySpec:   types.CustomerMasterKeySpecRsa4096,
+					KeySpec:   types.KeySpecRsa4096,
 					Enabled:   false,
 					PublicKey: []byte("foo"),
 				},
@@ -382,7 +419,7 @@ func TestGenerateKey(t *testing.T) {
 				KeyId:   spireKeyID,
 				KeyType: keymanagerv1.KeyType_EC_P256,
 			},
-			configureReq:      configureRequestWithVars("access_key_id", "secret_access_key", "region", getEmptyKeyMetadataFile(t), ""),
+			configureReq:      configureRequestWithVars("access_key_id", "secret_access_key", "region", KeyIdentifierFile, getEmptyKeyIdentifierFile(t), ""),
 			instanceAccountID: "example-account-id",
 			instanceRoleARN:   "arn:aws:sts::example-account-id:assumed-role/example-assumed-role-name/example-instance-id",
 			expectedKeyPolicy: &roleBasedPolicy,
@@ -393,7 +430,7 @@ func TestGenerateKey(t *testing.T) {
 				KeyId:   spireKeyID,
 				KeyType: keymanagerv1.KeyType_EC_P256,
 			},
-			configureReq:      configureRequestWithVars("access_key_id", "secret_access_key", "region", getEmptyKeyMetadataFile(t), getCustomPolicyFile(t)),
+			configureReq:      configureRequestWithVars("access_key_id", "secret_access_key", "region", KeyIdentifierFile, getEmptyKeyIdentifierFile(t), getCustomPolicyFile(t)),
 			instanceAccountID: "example-account-id",
 			instanceRoleARN:   "arn:aws:sts::example-account-id:assumed-role/example-assumed-role-name/example-instance-id",
 			expectedKeyPolicy: &customPolicy,
@@ -408,7 +445,7 @@ func TestGenerateKey(t *testing.T) {
 				{
 					AliasName:            aws.String(aliasName),
 					KeyID:                aws.String(keyID),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:              types.KeySpecEccNistP256,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					AliasLastUpdatedDate: &unixEpoch,
@@ -435,7 +472,7 @@ func TestGenerateKey(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/bundle-acme-foo_2ebar_2brsa"),
 					KeyID:                aws.String(keyID),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:              types.KeySpecEccNistP256,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					AliasLastUpdatedDate: &unixEpoch,
@@ -524,7 +561,7 @@ func TestGenerateKey(t *testing.T) {
 				{
 					AliasName: aws.String(aliasName),
 					KeyID:     aws.String(keyID),
-					KeySpec:   types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:   types.KeySpecEccNistP256,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
@@ -551,7 +588,7 @@ func TestGenerateKey(t *testing.T) {
 				{
 					AliasName: aws.String(aliasName),
 					KeyID:     aws.String(keyID),
-					KeySpec:   types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:   types.KeySpecEccNistP256,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
@@ -579,7 +616,7 @@ func TestGenerateKey(t *testing.T) {
 				{
 					AliasName: aws.String(aliasName),
 					KeyID:     aws.String(keyID),
-					KeySpec:   types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:   types.KeySpecEccNistP256,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
@@ -607,7 +644,7 @@ func TestGenerateKey(t *testing.T) {
 				{
 					AliasName: aws.String(aliasName),
 					KeyID:     aws.String(keyID),
-					KeySpec:   types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:   types.KeySpecEccNistP256,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
@@ -635,7 +672,7 @@ func TestGenerateKey(t *testing.T) {
 				{
 					AliasName: aws.String(aliasName),
 					KeyID:     aws.String(keyID),
-					KeySpec:   types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:   types.KeySpecEccNistP256,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
@@ -665,7 +702,7 @@ func TestGenerateKey(t *testing.T) {
 				KeyId:   spireKeyID,
 				KeyType: keymanagerv1.KeyType_EC_P256,
 			},
-			configureReq:         configureRequestWithVars("access_key_id", "secret_access_key", "region", getEmptyKeyMetadataFile(t), ""),
+			configureReq:         configureRequestWithVars("access_key_id", "secret_access_key", "region", KeyIdentifierFile, getEmptyKeyIdentifierFile(t), ""),
 			getCallerIdentityErr: "something went wrong",
 			err:                  "cannot get caller identity: something went wrong",
 			code:                 codes.Internal,
@@ -676,7 +713,7 @@ func TestGenerateKey(t *testing.T) {
 				KeyId:   spireKeyID,
 				KeyType: keymanagerv1.KeyType_EC_P256,
 			},
-			configureReq:    configureRequestWithVars("access_key_id", "secret_access_key", "region", getEmptyKeyMetadataFile(t), ""),
+			configureReq:    configureRequestWithVars("access_key_id", "secret_access_key", "region", KeyIdentifierFile, getEmptyKeyIdentifierFile(t), ""),
 			instanceRoleARN: "arn:aws:sts::example-account-id",
 			logs: []spiretest.LogEntry{
 				{
@@ -691,7 +728,7 @@ func TestGenerateKey(t *testing.T) {
 				KeyId:   spireKeyID,
 				KeyType: keymanagerv1.KeyType_EC_P256,
 			},
-			configureReq:    configureRequestWithVars("access_key_id", "secret_access_key", "region", getKeyMetadataFile(t), ""),
+			configureReq:    configureRequestWithVars("access_key_id", "secret_access_key", "region", KeyIdentifierFile, getKeyIdentifierFile(t), ""),
 			instanceRoleARN: "arn:aws:sts::example-account-id:user/development",
 			logs: []spiretest.LogEntry{
 				{
@@ -1063,7 +1100,7 @@ func TestGetPublicKey(t *testing.T) {
 				{
 					AliasName: aws.String(aliasName),
 					KeyID:     aws.String(keyID),
-					KeySpec:   types.CustomerMasterKeySpecRsa4096,
+					KeySpec:   types.KeySpecRsa4096,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
@@ -1076,7 +1113,7 @@ func TestGetPublicKey(t *testing.T) {
 				{
 					AliasName: aws.String("alias/SPIRE_SERVER/test_example_org/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/bundle-acme-foo_2ebar_2brsa"),
 					KeyID:     aws.String(keyID),
-					KeySpec:   types.CustomerMasterKeySpecRsa4096,
+					KeySpec:   types.KeySpecRsa4096,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
@@ -1129,7 +1166,7 @@ func TestGetPublicKeys(t *testing.T) {
 				{
 					AliasName: aws.String(aliasName),
 					KeyID:     aws.String(keyID),
-					KeySpec:   types.CustomerMasterKeySpecRsa4096,
+					KeySpec:   types.KeySpecRsa4096,
 					Enabled:   true,
 					PublicKey: []byte("foo"),
 				},
@@ -1181,7 +1218,7 @@ func TestRefreshAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/id_01"),
 					KeyID:                aws.String("key_id_01"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1196,7 +1233,7 @@ func TestRefreshAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/id_01"),
 					KeyID:                aws.String("key_id_01"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1205,7 +1242,7 @@ func TestRefreshAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/id_02"),
 					KeyID:                aws.String("key_id_02"),
-					KeySpec:              types.CustomerMasterKeySpecRsa2048,
+					KeySpec:              types.KeySpecRsa2048,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1214,7 +1251,7 @@ func TestRefreshAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/another_server_id/id_03"),
 					KeyID:                aws.String("key_id_03"),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP384,
+					KeySpec:              types.KeySpecEccNistP384,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1223,7 +1260,7 @@ func TestRefreshAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/another_td/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/id_04"),
 					KeyID:                aws.String("key_id_04"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1232,7 +1269,7 @@ func TestRefreshAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/another_td/another_server_id/id_05"),
 					KeyID:                aws.String("key_id_05"),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP384,
+					KeySpec:              types.KeySpecEccNistP384,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1241,7 +1278,7 @@ func TestRefreshAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/unrelated"),
 					KeyID:                aws.String("key_id_06"),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP384,
+					KeySpec:              types.KeySpecEccNistP384,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1250,7 +1287,7 @@ func TestRefreshAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/unrelated/unrelated/id_07"),
 					KeyID:                aws.String("key_id_07"),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP384,
+					KeySpec:              types.KeySpecEccNistP384,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1259,7 +1296,7 @@ func TestRefreshAliases(t *testing.T) {
 				{
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_08"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1380,7 +1417,7 @@ func TestDisposeAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/id_01"),
 					KeyID:                aws.String("key_id_01"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1389,7 +1426,7 @@ func TestDisposeAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/id_02"),
 					KeyID:                aws.String("key_id_02"),
-					KeySpec:              types.CustomerMasterKeySpecRsa2048,
+					KeySpec:              types.KeySpecRsa2048,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1398,7 +1435,7 @@ func TestDisposeAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/another_server_id/id_03"),
 					KeyID:                aws.String("key_id_03"),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP384,
+					KeySpec:              types.KeySpecEccNistP384,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1407,7 +1444,7 @@ func TestDisposeAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/another_td/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/id_04"),
 					KeyID:                aws.String("key_id_04"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1416,7 +1453,7 @@ func TestDisposeAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/another_td/another_server/id_05"),
 					KeyID:                aws.String("key_id_05"),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:              types.KeySpecEccNistP256,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1425,7 +1462,7 @@ func TestDisposeAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/unrelated"),
 					KeyID:                aws.String("key_id_06"),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:              types.KeySpecEccNistP256,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1434,7 +1471,7 @@ func TestDisposeAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/unrelated/unrelated/id_07"),
 					KeyID:                aws.String("key_id_07"),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:              types.KeySpecEccNistP256,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1443,7 +1480,7 @@ func TestDisposeAliases(t *testing.T) {
 				{
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_08"),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:              types.KeySpecEccNistP256,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1452,7 +1489,7 @@ func TestDisposeAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/another_server_id/id_09"),
 					KeyID:                aws.String("key_id_09"),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP384,
+					KeySpec:              types.KeySpecEccNistP384,
 					Enabled:              false,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1500,7 +1537,7 @@ func TestDisposeAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/id_01"),
 					KeyID:                aws.String("key_id_01"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1517,7 +1554,7 @@ func TestDisposeAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/another_server/id_01"),
 					KeyID:                aws.String("key_id_01"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1534,7 +1571,7 @@ func TestDisposeAliases(t *testing.T) {
 				{
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/another_server/id_01"),
 					KeyID:                aws.String("key_id_01"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1622,7 +1659,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_01"),
 					Description:          aws.String("SPIRE_SERVER_KEY/test_example_org"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1632,7 +1669,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/id_02"),
 					KeyID:                aws.String("key_id_02"),
 					Description:          aws.String("SPIRE_SERVER_KEY/test_example_org"),
-					KeySpec:              types.CustomerMasterKeySpecRsa2048,
+					KeySpec:              types.KeySpecRsa2048,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1642,7 +1679,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/another_server_id/id_03"),
 					KeyID:                aws.String("key_id_03"),
 					Description:          aws.String("SPIRE_SERVER_KEY/test_example_org"),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP384,
+					KeySpec:              types.KeySpecEccNistP384,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1652,7 +1689,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            aws.String("alias/SPIRE_SERVER/another_td/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/id_04"),
 					KeyID:                aws.String("key_id_04"),
 					Description:          aws.String("SPIRE_SERVER_KEY/another_td"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1662,7 +1699,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            aws.String("alias/SPIRE_SERVER/another_td/another_server_id/id_05"),
 					KeyID:                aws.String("key_id_05"),
 					Description:          aws.String("SPIRE_SERVER_KEY/another_td"),
-					KeySpec:              types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:              types.KeySpecEccNistP256,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1672,7 +1709,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            aws.String("alias/SPIRE_SERVER/unrelated"),
 					KeyID:                aws.String("key_id_06"),
 					Description:          nil,
-					KeySpec:              types.CustomerMasterKeySpecEccNistP256,
+					KeySpec:              types.KeySpecEccNistP256,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1682,7 +1719,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            aws.String("alias/SPIRE_SERVER/unrelated/unrelated/id_07"),
 					KeyID:                aws.String("key_id_07"),
 					Description:          nil,
-					KeySpec:              types.CustomerMasterKeySpecEccNistP384,
+					KeySpec:              types.KeySpecEccNistP384,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1692,7 +1729,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_08"),
 					Description:          nil,
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1702,7 +1739,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            aws.String("alias/SPIRE_SERVER/test_example_org/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/id_01"),
 					KeyID:                aws.String("key_id_09"),
 					Description:          aws.String("SPIRE_SERVER_KEY/test_example_org"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1712,7 +1749,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_10"),
 					Description:          aws.String("SPIRE_SERVER_KEY/another_td"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1722,7 +1759,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_11"),
 					Description:          aws.String("SPIRE_SERVER_KEY/"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1732,7 +1769,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_12"),
 					Description:          aws.String("SPIRE_SERVER_KEY"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1742,7 +1779,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_13"),
 					Description:          aws.String("test_example_org"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1752,7 +1789,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_14"),
 					Description:          aws.String("unrelated"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1762,7 +1799,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_15"),
 					Description:          aws.String("disabled key"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              false,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1772,7 +1809,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_16"),
 					Description:          aws.String("SPIRE_SERVER_KEY/test_example_org/extra"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1838,7 +1875,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_01"),
 					Description:          aws.String("SPIRE_SERVER_KEY/test_example_org"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1856,7 +1893,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_01"),
 					Description:          aws.String("SPIRE_SERVER_KEY/test_example_org"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1874,7 +1911,7 @@ func TestDisposeKeys(t *testing.T) {
 					AliasName:            nil,
 					KeyID:                aws.String("key_id_01"),
 					Description:          aws.String("SPIRE_SERVER_KEY/test_example_org"),
-					KeySpec:              types.CustomerMasterKeySpecRsa4096,
+					KeySpec:              types.KeySpecRsa4096,
 					Enabled:              true,
 					PublicKey:            []byte("foo"),
 					CreationDate:         &unixEpoch,
@@ -1932,49 +1969,59 @@ func TestDisposeKeys(t *testing.T) {
 
 func configureRequestWithString(config string) *configv1.ConfigureRequest {
 	return &configv1.ConfigureRequest{
-		HclConfiguration: config,
+		CoreConfiguration: &configv1.CoreConfiguration{TrustDomain: "test.example.org"},
+		HclConfiguration:  config,
 	}
 }
 
-func configureRequestWithVars(accessKeyID, secretAccessKey, region, keyMetadataFile, keyPolicyFile string) *configv1.ConfigureRequest {
+type KeyIdentifierConfigName string
+
+const (
+	KeyIdentifierFile  KeyIdentifierConfigName = "key_identifier_file"
+	KeyIdentifierValue KeyIdentifierConfigName = "key_identifier_value"
+)
+
+func configureRequestWithVars(accessKeyID, secretAccessKey, region, keyIdentifierConfigName KeyIdentifierConfigName, keyIdentifierConfigValue, keyPolicyFile string) *configv1.ConfigureRequest {
 	return &configv1.ConfigureRequest{
+		CoreConfiguration: &configv1.CoreConfiguration{TrustDomain: "test.example.org"},
 		HclConfiguration: fmt.Sprintf(`{
 			"access_key_id": "%s",
 			"secret_access_key": "%s",
 			"region":"%s",
-			"key_metadata_file":"%s",
+			"%s":"%s",
 			"key_policy_file":"%s"
 			}`,
 			accessKeyID,
 			secretAccessKey,
 			region,
-			keyMetadataFile,
+			keyIdentifierConfigName,
+			keyIdentifierConfigValue,
 			keyPolicyFile),
-		CoreConfiguration: &configv1.CoreConfiguration{TrustDomain: "test.example.org"},
 	}
 }
 
 func configureRequestWithDefaults(t *testing.T) *configv1.ConfigureRequest {
 	return &configv1.ConfigureRequest{
-		HclConfiguration:  serializedConfiguration(validAccessKeyID, validSecretAccessKey, validRegion, getKeyMetadataFile(t)),
 		CoreConfiguration: &configv1.CoreConfiguration{TrustDomain: "test.example.org"},
+		HclConfiguration:  serializedConfiguration(validAccessKeyID, validSecretAccessKey, validRegion, KeyIdentifierFile, getKeyIdentifierFile(t)),
 	}
 }
 
-func serializedConfiguration(accessKeyID, secretAccessKey, region string, keyMetadataFile string) string {
+func serializedConfiguration(accessKeyID, secretAccessKey, region string, keyIdentifierConfigName KeyIdentifierConfigName, keyIdentifierConfigValue string) string {
 	return fmt.Sprintf(`{
 		"access_key_id": "%s",
 		"secret_access_key": "%s",
 		"region":"%s",
-		"key_metadata_file":"%s"
+		"%s":"%s"
 		}`,
 		accessKeyID,
 		secretAccessKey,
 		region,
-		keyMetadataFile)
+		keyIdentifierConfigName,
+		keyIdentifierConfigValue)
 }
 
-func getKeyMetadataFile(t *testing.T) string {
+func getKeyIdentifierFile(t *testing.T) string {
 	tempDir := t.TempDir()
 	tempFilePath := path.Join(tempDir, validServerIDFile)
 	err := os.WriteFile(tempFilePath, []byte(validServerID), 0o600)
@@ -1987,13 +2034,13 @@ func getKeyMetadataFile(t *testing.T) string {
 	return tempFilePath
 }
 
-func getEmptyKeyMetadataFile(t *testing.T) string {
+func getEmptyKeyIdentifierFile(t *testing.T) string {
 	tempDir := t.TempDir()
-	keyMetadataFile := path.Join(tempDir, validServerIDFile)
+	keyIdentifierFile := path.Join(tempDir, validServerIDFile)
 	if isWindows {
-		keyMetadataFile = filepath.ToSlash(keyMetadataFile)
+		keyIdentifierFile = filepath.ToSlash(keyIdentifierFile)
 	}
-	return keyMetadataFile
+	return keyIdentifierFile
 }
 
 func getCustomPolicyFile(t *testing.T) string {
