@@ -19,7 +19,7 @@ import (
 	"github.com/Azure/azure-sdk-for-go/sdk/azidentity"
 	"github.com/Azure/azure-sdk-for-go/sdk/keyvault/azkeys"
 	"github.com/andres-erbsen/clock"
-	"github.com/gofrs/uuid"
+	"github.com/gofrs/uuid/v5"
 	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/hcl"
 	"github.com/lestrrat-go/jwx/v2/jwk"
@@ -27,6 +27,7 @@ import (
 	configv1 "github.com/spiffe/spire-plugin-sdk/proto/spire/service/common/config/v1"
 	"github.com/spiffe/spire/pkg/common/catalog"
 	"github.com/spiffe/spire/pkg/common/diskutil"
+	"github.com/spiffe/spire/pkg/common/pluginconf"
 	"golang.org/x/crypto/cryptobyte"
 	"golang.org/x/crypto/cryptobyte/asn1"
 	"google.golang.org/grpc/codes"
@@ -77,16 +78,42 @@ type pluginHooks struct {
 
 // Config provides configuration context for the plugin.
 type Config struct {
-	KeyMetadataFile string `hcl:"key_metadata_file" json:"key_metadata_file"`
-	KeyVaultURI     string `hcl:"key_vault_uri" json:"key_vault_uri"`
-	TenantID        string `hcl:"tenant_id" json:"tenant_id"`
-	SubscriptionID  string `hcl:"subscription_id" json:"subscription_id"`
-	AppID           string `hcl:"app_id" json:"app_id"`
-	AppSecret       string `hcl:"app_secret" json:"app_secret"`
+	KeyIdentifierFile  string `hcl:"key_identifier_file" json:"key_identifier_file"`
+	KeyIdentifierValue string `hcl:"key_identifier_value" json:"key_identifier_value"`
+	KeyVaultURI        string `hcl:"key_vault_uri" json:"key_vault_uri"`
+	TenantID           string `hcl:"tenant_id" json:"tenant_id"`
+	SubscriptionID     string `hcl:"subscription_id" json:"subscription_id"`
+	AppID              string `hcl:"app_id" json:"app_id"`
+	AppSecret          string `hcl:"app_secret" json:"app_secret"`
+}
 
-	// Deprecated: use_msi is deprecated and will be removed in a future release.
-	// Will be used implicitly if other mechanisms to authenticate fail.
-	UseMSI bool `hcl:"use_msi" json:"use_msi"`
+func buildConfig(coreConfig catalog.CoreConfig, hclText string, status *pluginconf.Status) *Config {
+	newConfig := new(Config)
+
+	if err := hcl.Decode(newConfig, hclText); err != nil {
+		status.ReportErrorf("unable to decode configuration: %v", err)
+		return nil
+	}
+
+	if newConfig.KeyVaultURI == "" {
+		status.ReportError("configuration is missing the Key Vault URI")
+	}
+
+	if newConfig.KeyIdentifierValue != "" {
+		if len(newConfig.KeyIdentifierValue) > 256 {
+			status.ReportError("Key identifier must not be longer than 256 characters")
+		}
+	}
+
+	if newConfig.KeyIdentifierFile == "" && newConfig.KeyIdentifierValue == "" {
+		status.ReportError("configuration requires a key identifier file or a key identifier value")
+	}
+
+	if newConfig.KeyIdentifierFile != "" && newConfig.KeyIdentifierValue != "" {
+		status.ReportError("configuration can't have a key identifier file and a key identifier value at the same time")
+	}
+
+	return newConfig
 }
 
 // Plugin is the main representation of this keymanager plugin
@@ -134,57 +161,52 @@ func (p *Plugin) SetLogger(log hclog.Logger) {
 }
 
 func (p *Plugin) Configure(ctx context.Context, req *configv1.ConfigureRequest) (*configv1.ConfigureResponse, error) {
-	config, err := parseAndValidateConfig(req.HclConfiguration)
+	newConfig, _, err := pluginconf.Build(req, buildConfig)
 	if err != nil {
 		return nil, err
 	}
 
-	serverID, err := getOrCreateServerID(config.KeyMetadataFile)
-	if err != nil {
-		return nil, err
+	serverID := newConfig.KeyIdentifierValue
+	if serverID == "" {
+		serverID, err = getOrCreateServerID(newConfig.KeyIdentifierFile)
+		if err != nil {
+			return nil, err
+		}
 	}
-	p.log.Debug("Loaded server ID", "server_id", serverID)
+	p.log.Debug("Loaded server id", "server_id", serverID)
 
 	var client cloudKeyManagementService
 
 	switch {
-	case config.SubscriptionID != "", config.AppID != "", config.AppSecret != "", config.TenantID != "":
-		if config.UseMSI {
-			return nil, status.Errorf(codes.InvalidArgument, "invalid configuration, cannot use both MSI and app authentication")
-		}
-		if config.TenantID == "" {
+	case newConfig.SubscriptionID != "", newConfig.AppID != "", newConfig.AppSecret != "", newConfig.TenantID != "":
+		if newConfig.TenantID == "" {
 			return nil, status.Errorf(codes.InvalidArgument, "invalid configuration, missing tenant id")
 		}
-		if config.SubscriptionID == "" {
+		if newConfig.SubscriptionID == "" {
 			return nil, status.Errorf(codes.InvalidArgument, "invalid configuration, missing subscription id")
 		}
-		if config.AppID == "" {
+		if newConfig.AppID == "" {
 			return nil, status.Errorf(codes.InvalidArgument, "invalid configuration, missing application id")
 		}
-		if config.AppSecret == "" {
+		if newConfig.AppSecret == "" {
 			return nil, status.Errorf(codes.InvalidArgument, "invalid configuration, missing app secret")
 		}
 
-		creds, err := azidentity.NewClientSecretCredential(config.TenantID, config.AppID, config.AppSecret, nil)
+		creds, err := azidentity.NewClientSecretCredential(newConfig.TenantID, newConfig.AppID, newConfig.AppSecret, nil)
 		if err != nil {
 			return nil, status.Errorf(codes.Internal, "unable to get client credential: %v", err)
 		}
 
-		client, err = p.hooks.newKeyVaultClient(creds, config.KeyVaultURI)
+		client, err = p.hooks.newKeyVaultClient(creds, newConfig.KeyVaultURI)
 		if err != nil {
 			return nil, status.Errorf(codes.Internal, "failed to create Key Vault client with client credentials: %v", err)
 		}
-
-	case config.UseMSI:
-		p.log.Warn("use_msi is deprecated and will be removed in a future release")
-		fallthrough // use default credential which attempts to fetch credentials using MSI
-
 	default:
 		cred, err := p.hooks.fetchCredential()
 		if err != nil {
 			return nil, status.Errorf(codes.Internal, "unable to fetch client credential: %v", err)
 		}
-		client, err = p.hooks.newKeyVaultClient(cred, config.KeyVaultURI)
+		client, err = p.hooks.newKeyVaultClient(cred, newConfig.KeyVaultURI)
 		if err != nil {
 			return nil, status.Errorf(codes.Internal, "failed to create Key Vault client with MSI credential: %v", err)
 		}
@@ -197,7 +219,7 @@ func (p *Plugin) Configure(ctx context.Context, req *configv1.ConfigureRequest)
 		trustDomain:    req.CoreConfiguration.TrustDomain,
 	}
 
-	p.log.Debug("Fetching keys from Azure Key Vault", "key_vault_uri", config.KeyVaultURI)
+	p.log.Debug("Fetching keys from Azure Key Vault", "key_vault_uri", newConfig.KeyVaultURI)
 	keyEntries, err := fetcher.fetchKeyEntries(ctx)
 	if err != nil {
 		return nil, err
@@ -228,6 +250,15 @@ func (p *Plugin) Configure(ctx context.Context, req *configv1.ConfigureRequest)
 	return &configv1.ConfigureResponse{}, nil
 }
 
+func (p *Plugin) Validate(ctx context.Context, req *configv1.ValidateRequest) (*configv1.ValidateResponse, error) {
+	_, notes, err := pluginconf.Build(req, buildConfig)
+
+	return &configv1.ValidateResponse{
+		Valid: err == nil,
+		Notes: notes,
+	}, nil
+}
+
 // refreshKeysTask will update the keys in the cache every 6 hours.
 // Keys will be updated with the same Operations they already have (Sign and Verify).
 // The consequence of this is that the value of the field "Updated" in each key belonging to the server will be set to the current timestamp.
@@ -280,7 +311,7 @@ func (p *Plugin) refreshKeys(ctx context.Context) error {
 	}
 
 	if errs != nil {
-		return fmt.Errorf(strings.Join(errs, ": "))
+		return errors.New(strings.Join(errs, ": "))
 	}
 	return nil
 }
@@ -673,25 +704,6 @@ func (p *Plugin) generateKeyName(spireKeyID string) (keyName string, err error)
 	return fmt.Sprintf("%s-%s-%s", keyNamePrefix, uniqueID, spireKeyID), nil
 }
 
-// parseAndValidateConfig returns an error if any configuration provided does not meet acceptable criteria
-func parseAndValidateConfig(c string) (*Config, error) {
-	config := new(Config)
-
-	if err := hcl.Decode(config, c); err != nil {
-		return nil, status.Errorf(codes.InvalidArgument, "unable to decode configuration: %v", err)
-	}
-
-	if config.KeyVaultURI == "" {
-		return nil, status.Error(codes.InvalidArgument, "configuration is missing the Key Vault URI")
-	}
-
-	if config.KeyMetadataFile == "" {
-		return nil, status.Error(codes.InvalidArgument, "configuration is missing server ID file path")
-	}
-
-	return config, nil
-}
-
 func getOrCreateServerID(idPath string) (string, error) {
 	data, err := os.ReadFile(idPath)
 	switch {
