@@ -20,8 +20,10 @@ import (
 	"github.com/gofrs/uuid/v5"
 	"github.com/sirupsen/logrus"
 	"github.com/sirupsen/logrus/hooks/test"
+	"github.com/spiffe/go-spiffe/v2/spiffeid"
 	keymanagerv1 "github.com/spiffe/spire-plugin-sdk/proto/spire/plugin/server/keymanager/v1"
 	configv1 "github.com/spiffe/spire-plugin-sdk/proto/spire/service/common/config/v1"
+	"github.com/spiffe/spire/pkg/common/catalog"
 	"github.com/spiffe/spire/pkg/server/plugin/keymanager"
 	keymanagertest "github.com/spiffe/spire/pkg/server/plugin/keymanager/test"
 	"github.com/spiffe/spire/test/plugintest"
@@ -66,13 +68,17 @@ func TestKeyManagerContract(t *testing.T) {
 			func(azcore.TokenCredential, string) (cloudKeyManagementService, error) { return kmsClient, nil },
 		)
 		km := new(keymanager.V1)
-		keyMetadataFile := createKeyMetadataFile(t)
+		keyIdentifierFile := createKeyIdentifierFile(t)
 
-		plugintest.Load(t, builtin(p), km, plugintest.Configuref(`
-			key_metadata_file = %q
+		plugintest.Load(t, builtin(p), km,
+			plugintest.CoreConfig(catalog.CoreConfig{
+				TrustDomain: spiffeid.RequireTrustDomainFromString("example.org"),
+			}),
+			plugintest.Configuref(`
+			key_identifier_file = %q
 			key_vault_uri = "https://spire-server.vault.azure.net/"
 			use_msi=true
-		`, keyMetadataFile))
+		`, keyIdentifierFile))
 		return km
 	}
 
@@ -113,54 +119,68 @@ func TestConfigure(t *testing.T) {
 			configureRequest: configureRequestWithDefaults(t),
 		},
 		{
-			name:             "missing key metadata file",
-			configureRequest: configureRequestWithVars("", validKeyVaultURI, "", "", "", validAppSecret, "true"),
-			err:              "configuration is missing server ID file path",
+			name:             "pass with identity file",
+			configureRequest: configureRequestWithVars(KeyIdentifierValue, createKeyIdentifierFile(t), validKeyVaultURI, validTenantID, validSubscriptionID, validAppID, validAppSecret),
+		},
+		{
+			name:             "pass with identity value",
+			configureRequest: configureRequestWithVars(KeyIdentifierValue, "server-id", validKeyVaultURI, validTenantID, validSubscriptionID, validAppID, validAppSecret),
+		},
+		{
+			name:             "missing key identifier file and key identifier value",
+			configureRequest: configureRequestWithVars(KeyIdentifierFile, "", validKeyVaultURI, validTenantID, validSubscriptionID, validAppID, validAppSecret),
+			err:              "configuration requires a key identifier file or a key identifier value",
+			code:             codes.InvalidArgument,
+		},
+		{
+			name:             "both key identifier file and key identifier value",
+			configureRequest: configureRequestWithString(fmt.Sprintf(`{"access_key_id":"access_key_id","secret_access_key":"secret_access_key","region":"region","key_identifier_file":"key_identifier_file","key_identifier_value":"key_identifier_value","key_policy_file":"","key_vault_uri":"%s"}`, validKeyVaultURI)),
+			err:              "configuration can't have a key identifier file and a key identifier value at the same time",
+			code:             codes.InvalidArgument,
+		},
+		{
+			name:             "key identifier value too long",
+			configureRequest: configureRequestWithVars(KeyIdentifierValue, "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA", validKeyVaultURI, validTenantID, validSubscriptionID, validAppID, validAppSecret),
+			err:              "Key identifier must not be longer than 256 characters",
 			code:             codes.InvalidArgument,
 		},
 		{
 			name:             "missing client authentication config",
-			configureRequest: configureRequestWithVars(createKeyMetadataFile(t), validKeyVaultURI, "", "", "", "", "false"),
-		},
-		{
-			name:             "use MSI while app secret is set",
-			configureRequest: configureRequestWithVars(createKeyMetadataFile(t), validKeyVaultURI, "", "", "", validAppSecret, "true"),
-			err:              "invalid configuration, cannot use both MSI and app authentication",
-			code:             codes.InvalidArgument,
+			configureRequest: configureRequestWithVars(KeyIdentifierFile, createKeyIdentifierFile(t), validKeyVaultURI, "", "", "", ""),
 		},
 		{
 			name:             "missing Key Vault URI",
-			configureRequest: configureRequestWithVars(createKeyMetadataFile(t), "", validTenantID, validSubscriptionID, validAppID, validAppSecret, "false"),
+			configureRequest: configureRequestWithVars(KeyIdentifierFile, createKeyIdentifierFile(t), "", validTenantID, validSubscriptionID, validAppID, validAppSecret),
 			err:              "configuration is missing the Key Vault URI",
 			code:             codes.InvalidArgument,
 		},
 		{
 			name:             "missing tenant ID",
-			configureRequest: configureRequestWithVars(createKeyMetadataFile(t), validKeyVaultURI, "", validSubscriptionID, validAppID, validAppSecret, "false"),
+			configureRequest: configureRequestWithVars(KeyIdentifierFile, createKeyIdentifierFile(t), validKeyVaultURI, "", validSubscriptionID, validAppID, validAppSecret),
 			err:              "invalid configuration, missing tenant id",
 			code:             codes.InvalidArgument,
 		},
 		{
 			name:             "missing subscription ID ",
-			configureRequest: configureRequestWithVars(createKeyMetadataFile(t), validKeyVaultURI, validTenantID, "", validAppID, validAppSecret, "false"),
+			configureRequest: configureRequestWithVars(KeyIdentifierFile, createKeyIdentifierFile(t), validKeyVaultURI, validTenantID, "", validAppID, validAppSecret),
 			err:              "invalid configuration, missing subscription id",
 			code:             codes.InvalidArgument,
 		},
 		{
-			name:             "missing server id file path",
-			configureRequest: configureRequestWithVars("", validKeyVaultURI, validTenantID, validSubscriptionID, validAppID, validAppSecret, "false"),
-			err:              "configuration is missing server ID file path",
+			name:             "missing key identifier file and key identifier value",
+			configureRequest: configureRequestWithVars(KeyIdentifierFile, "", validKeyVaultURI, validTenantID, validSubscriptionID, validAppID, validAppSecret),
+			err:              "configuration requires a key identifier file or a key identifier value",
 			code:             codes.InvalidArgument,
 		},
 		{
 			name:             "missing application ID",
-			configureRequest: configureRequestWithVars(createKeyMetadataFile(t), validKeyVaultURI, validTenantID, validSubscriptionID, "", validAppSecret, "false"),
+			configureRequest: configureRequestWithVars(KeyIdentifierFile, createKeyIdentifierFile(t), validKeyVaultURI, validTenantID, validSubscriptionID, "", validAppSecret),
 			err:              "invalid configuration, missing application id",
 			code:             codes.InvalidArgument,
 		},
 		{
 			name:             "missing application secret",
-			configureRequest: configureRequestWithVars(createKeyMetadataFile(t), validKeyVaultURI, validTenantID, validSubscriptionID, validAppID, "", "false"),
+			configureRequest: configureRequestWithVars(KeyIdentifierFile, createKeyIdentifierFile(t), validKeyVaultURI, validTenantID, validSubscriptionID, validAppID, ""),
 			err:              "invalid configuration, missing app secret",
 			code:             codes.InvalidArgument,
 		},
@@ -950,7 +970,7 @@ func setupTest(t *testing.T) *pluginTest {
 
 func configureRequestWithDefaults(t *testing.T) *configv1.ConfigureRequest {
 	return &configv1.ConfigureRequest{
-		HclConfiguration:  serializedConfiguration(createKeyMetadataFile(t), validKeyVaultURI, validTenantID, validSubscriptionID, validAppID, validAppSecret, ""),
+		HclConfiguration:  serializedConfiguration(KeyIdentifierFile, createKeyIdentifierFile(t), validKeyVaultURI, validTenantID, validSubscriptionID, validAppID, validAppSecret),
 		CoreConfiguration: &configv1.CoreConfiguration{TrustDomain: trustDomain},
 	}
 }
@@ -961,39 +981,46 @@ func getUUID(t *testing.T) string {
 	return uuid.String()
 }
 
-func serializedConfiguration(keyMetadataFile, keyVaultURI, tenantID, subscriptionID, appID, appSecret, useMsi string) string {
+type KeyIdentifierConfigName string
+
+const (
+	KeyIdentifierFile  KeyIdentifierConfigName = "key_identifier_file"
+	KeyIdentifierValue KeyIdentifierConfigName = "key_identifier_value"
+)
+
+func serializedConfiguration(keyIdentifierConfigName KeyIdentifierConfigName, keyIdentifierConfigValue, keyVaultURI, tenantID, subscriptionID, appID, appSecret string) string {
 	return fmt.Sprintf(`{
-		"key_metadata_file":"%s",
+		"%s":"%s",
 		"key_vault_uri":"%s",
 		"tenant_id":"%s",
 		"subscription_id":"%s",
 		"app_id":"%s",
 		"app_secret":"%s",
-		"use_msi":%s
 		}`,
-		keyMetadataFile,
+		keyIdentifierConfigName,
+		keyIdentifierConfigValue,
 		keyVaultURI,
 		tenantID,
 		subscriptionID,
 		appID,
-		appSecret,
-		useMsi)
+		appSecret)
 }
 
-func configureRequestWithVars(keyMetadataFile, keyVaultURI, tenantID, subscriptionID, appID, appSecret, useMsi string) *configv1.ConfigureRequest {
+func configureRequestWithVars(keyIdentifierConfigName KeyIdentifierConfigName, keyIdentifierConfigValue, keyVaultURI, tenantID, subscriptionID, appID, appSecret string) *configv1.ConfigureRequest {
 	return &configv1.ConfigureRequest{
-		HclConfiguration:  serializedConfiguration(keyMetadataFile, keyVaultURI, tenantID, subscriptionID, appID, appSecret, useMsi),
 		CoreConfiguration: &configv1.CoreConfiguration{TrustDomain: trustDomain},
+		HclConfiguration:  serializedConfiguration(keyIdentifierConfigName, keyIdentifierConfigValue, keyVaultURI, tenantID, subscriptionID, appID, appSecret),
 	}
 }
 
 func configureRequestWithString(config string) *configv1.ConfigureRequest {
 	return &configv1.ConfigureRequest{
-		HclConfiguration: config,
+		CoreConfiguration: &configv1.CoreConfiguration{TrustDomain: trustDomain},
+		HclConfiguration:  config,
 	}
 }
 
-func createKeyMetadataFile(t *testing.T) string {
+func createKeyIdentifierFile(t *testing.T) string {
 	tempDir := t.TempDir()
 	tempFilePath := filepath.ToSlash(filepath.Join(tempDir, validServerIDFile))
 	err := os.WriteFile(tempFilePath, []byte(validServerID), 0o600)
