@@ -14,6 +14,7 @@ import (
 	"math"
 	"os"
 	"regexp"
+	"slices"
 	"sort"
 	"strings"
 	"sync"
@@ -35,6 +36,7 @@ import (
 	"github.com/spiffe/spire/pkg/common/agentpathtemplate"
 	"github.com/spiffe/spire/pkg/common/catalog"
 	caws "github.com/spiffe/spire/pkg/common/plugin/aws"
+	"github.com/spiffe/spire/pkg/common/pluginconf"
 	nodeattestorbase "github.com/spiffe/spire/pkg/server/plugin/nodeattestor/base"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
@@ -53,7 +55,7 @@ var (
 	}
 
 	defaultPartition = "aws"
-	// No constant was found in the sdk, using the list of paritions defined on
+	// No constant was found in the sdk, using the list of partitions defined on
 	// the page https://docs.aws.amazon.com/IAM/latest/UserGuide/reference-arns.html
 	partitions = []string{
 		defaultPartition,
@@ -100,6 +102,8 @@ type IIDAttestorPlugin struct {
 	mtx     sync.RWMutex
 	clients *clientsCache
 
+	orgValidation *orgValidator
+
 	// test hooks
 	hooks struct {
 		getAWSCACertificate func(string, PublicKeyType) (*x509.Certificate, error)
@@ -112,20 +116,69 @@ type IIDAttestorPlugin struct {
 // IIDAttestorConfig holds hcl configuration for IID attestor plugin
 type IIDAttestorConfig struct {
 	SessionConfig                   `hcl:",squash"`
-	SkipBlockDevice                 bool     `hcl:"skip_block_device"`
-	DisableInstanceProfileSelectors bool     `hcl:"disable_instance_profile_selectors"`
-	LocalValidAcctIDs               []string `hcl:"account_ids_for_local_validation"`
-	AgentPathTemplate               string   `hcl:"agent_path_template"`
-	AssumeRole                      string   `hcl:"assume_role"`
-	Partition                       string   `hcl:"partition"`
+	SkipBlockDevice                 bool                 `hcl:"skip_block_device"`
+	DisableInstanceProfileSelectors bool                 `hcl:"disable_instance_profile_selectors"`
+	LocalValidAcctIDs               []string             `hcl:"account_ids_for_local_validation"`
+	AgentPathTemplate               string               `hcl:"agent_path_template"`
+	AssumeRole                      string               `hcl:"assume_role"`
+	Partition                       string               `hcl:"partition"`
+	ValidateOrgAccountID            *orgValidationConfig `hcl:"verify_organization"`
 	pathTemplate                    *agentpathtemplate.Template
 	trustDomain                     spiffeid.TrustDomain
 	getAWSCACertificate             func(string, PublicKeyType) (*x509.Certificate, error)
 }
 
+func (p *IIDAttestorPlugin) buildConfig(coreConfig catalog.CoreConfig, hclText string, status *pluginconf.Status) *IIDAttestorConfig {
+	newConfig := new(IIDAttestorConfig)
+	if err := hcl.Decode(newConfig, hclText); err != nil {
+		status.ReportErrorf("unable to decode configuration: %v", err)
+		return nil
+	}
+
+	// Function to get the AWS CA certificate. We do this lazily on configure so deployments
+	// not using this plugin don't pay for parsing it on startup. This
+	// operation should not fail, but we check the return value just in case.
+	newConfig.getAWSCACertificate = p.hooks.getAWSCACertificate
+
+	if err := newConfig.Validate(p.hooks.getenv(accessKeyIDVarName), p.hooks.getenv(secretAccessKeyVarName)); err != nil {
+		status.ReportError(err.Error())
+	}
+
+	newConfig.trustDomain = coreConfig.TrustDomain
+
+	newConfig.pathTemplate = defaultAgentPathTemplate
+	if len(newConfig.AgentPathTemplate) > 0 {
+		tmpl, err := agentpathtemplate.Parse(newConfig.AgentPathTemplate)
+		if err != nil {
+			status.ReportErrorf("failed to parse agent svid template: %q", newConfig.AgentPathTemplate)
+		} else {
+			newConfig.pathTemplate = tmpl
+		}
+	}
+
+	if newConfig.Partition == "" {
+		newConfig.Partition = defaultPartition
+	}
+
+	if !isValidAWSPartition(newConfig.Partition) {
+		status.ReportErrorf("invalid partition %q, must be one of: %v", newConfig.Partition, partitions)
+	}
+
+	// Check if Feature flag for account belongs to organization is enabled.
+	if newConfig.ValidateOrgAccountID != nil {
+		err := validateOrganizationConfig(newConfig)
+		if err != nil {
+			status.ReportError(err.Error())
+		}
+	}
+
+	return newConfig
+}
+
 // New creates a new IIDAttestorPlugin.
 func New() *IIDAttestorPlugin {
 	p := &IIDAttestorPlugin{}
+	p.orgValidation = newOrganizationValidationBase(&orgValidationConfig{})
 	p.clients = newClientsCache(defaultNewClientCallback)
 	p.hooks.getAWSCACertificate = getAWSCACertificate
 	p.hooks.getenv = os.Getenv
@@ -154,6 +207,26 @@ func (p *IIDAttestorPlugin) Attest(stream nodeattestorv1.NodeAttestor_AttestServ
 		return err
 	}
 
+	// Feature account belongs to organization
+	// Get the account id of the node from attestation and then check if respective account belongs to organization
+	if c.ValidateOrgAccountID != nil {
+		ctxValidateOrg, cancel := context.WithTimeout(stream.Context(), awsTimeout)
+		defer cancel()
+		orgClient, err := p.clients.getClient(ctxValidateOrg, c.ValidateOrgAccountID.AccountRegion, c.ValidateOrgAccountID.AccountID)
+		if err != nil {
+			return status.Errorf(codes.Internal, "failed to get org client: %v", err)
+		}
+
+		valid, err := p.orgValidation.IsMemberAccount(ctxValidateOrg, orgClient, attestationData.AccountID)
+		if err != nil {
+			return status.Errorf(codes.Internal, "failed aws ec2 attestation, issue while verifying if nodes account id: %v belong to org: %v", attestationData.AccountID, err)
+		}
+
+		if !valid {
+			return status.Errorf(codes.Internal, "failed aws ec2 attestation, nodes account id: %v is not part of configured organization or doesn't have ACTIVE status", attestationData.AccountID)
+		}
+	}
+
 	inTrustAcctList := false
 	for _, id := range c.LocalValidAcctIDs {
 		if attestationData.AccountID == id {
@@ -183,7 +256,7 @@ func (p *IIDAttestorPlugin) Attest(stream nodeattestorv1.NodeAttestor_AttestServ
 	// e.g. do it after the call to `p.AssessTOFU`, however, we may need
 	// the instance to construct tags used in the agent ID.
 	//
-	// This overhead will only effect agents attempting to re-attest which
+	// This overhead will only affect agents attempting to re-attest which
 	// should be a very small portion of the overall server workload. This
 	// is a potential DoS vector.
 	shouldCheckBlockDevice := !inTrustAcctList && !c.SkipBlockDevice
@@ -232,68 +305,56 @@ func (p *IIDAttestorPlugin) Attest(stream nodeattestorv1.NodeAttestor_AttestServ
 
 // Configure configures the IIDAttestorPlugin.
 func (p *IIDAttestorPlugin) Configure(_ context.Context, req *configv1.ConfigureRequest) (*configv1.ConfigureResponse, error) {
-	config := new(IIDAttestorConfig)
-	if err := hcl.Decode(config, req.HclConfiguration); err != nil {
-		return nil, status.Errorf(codes.InvalidArgument, "unable to decode configuration: %v", err)
-	}
-
-	// Function to get the AWS CA certificate. We do this lazily on configure so deployments
-	// not using this plugin don't pay for parsing it on startup. This
-	// operation should not fail, but we check the return value just in case.
-	config.getAWSCACertificate = p.hooks.getAWSCACertificate
-
-	if err := config.Validate(p.hooks.getenv(accessKeyIDVarName), p.hooks.getenv(secretAccessKeyVarName)); err != nil {
-		return nil, err
-	}
-
-	if req.CoreConfiguration == nil {
-		return nil, status.Error(codes.InvalidArgument, "core configuration is required")
-	}
-
-	var err error
-	config.trustDomain, err = spiffeid.TrustDomainFromString(req.CoreConfiguration.TrustDomain)
+	newConfig, _, err := pluginconf.Build(req, p.buildConfig)
 	if err != nil {
-		return nil, status.Errorf(codes.InvalidArgument, "core configuration has invalid trust domain: %v", err)
-	}
-
-	config.pathTemplate = defaultAgentPathTemplate
-	if len(config.AgentPathTemplate) > 0 {
-		tmpl, err := agentpathtemplate.Parse(config.AgentPathTemplate)
-		if err != nil {
-			return nil, status.Errorf(codes.InvalidArgument, "failed to parse agent svid template: %q", config.AgentPathTemplate)
-		}
-		config.pathTemplate = tmpl
-	}
-
-	if config.Partition == "" {
-		config.Partition = defaultPartition
-	}
-	if !isValidAWSPartition(config.Partition) {
-		return nil, status.Errorf(codes.InvalidArgument, "invalid partition %q, must be one of: %v", config.Partition, partitions)
+		return nil, err
 	}
 
 	p.mtx.Lock()
 	defer p.mtx.Unlock()
+	p.config = newConfig
 
-	p.config = config
-	p.clients.configure(config.SessionConfig)
+	if newConfig.ValidateOrgAccountID == nil {
+		// unconfigure existing clients
+		p.clients.configure(p.config.SessionConfig, orgValidationConfig{})
+	} else {
+		p.clients.configure(p.config.SessionConfig, *p.config.ValidateOrgAccountID)
+		// Setup required config, for validation and for bootstrapping org client
+		if err := p.orgValidation.configure(p.config.ValidateOrgAccountID); err != nil {
+			return nil, err
+		}
+	}
 
 	return &configv1.ConfigureResponse{}, nil
 }
 
+func (p *IIDAttestorPlugin) Validate(_ context.Context, req *configv1.ValidateRequest) (*configv1.ValidateResponse, error) {
+	_, notes, err := pluginconf.Build(req, p.buildConfig)
+
+	return &configv1.ValidateResponse{
+		Valid: err == nil,
+		Notes: notes,
+	}, nil
+}
+
 // SetLogger sets this plugin's logger
 func (p *IIDAttestorPlugin) SetLogger(log hclog.Logger) {
 	p.log = log
+	p.orgValidation.setLogger(log)
 }
 
 func (p *IIDAttestorPlugin) checkBlockDevice(instance ec2types.Instance) error {
-	ifaceZeroDeviceIndex := *instance.NetworkInterfaces[0].Attachment.DeviceIndex
-
-	if ifaceZeroDeviceIndex != 0 {
-		return fmt.Errorf("the EC2 instance network interface attachment device index must be zero (has %d)", ifaceZeroDeviceIndex)
+	ifaceZeroIndex := slices.IndexFunc(
+		instance.NetworkInterfaces,
+		func(net ec2types.InstanceNetworkInterface) bool {
+			return *net.Attachment.DeviceIndex == 0
+		},
+	)
+	if ifaceZeroIndex == -1 {
+		return errors.New("the EC2 instance network interface with device index 0 is inaccessible")
 	}
 
-	ifaceZeroAttachTime := instance.NetworkInterfaces[0].Attachment.AttachTime
+	ifaceZeroAttachTime := instance.NetworkInterfaces[ifaceZeroIndex].Attachment.AttachTime
 
 	// skip anti-tampering mechanism when RootDeviceType is instance-store
 	// specifically, if device type is persistent, and the device was attached past
@@ -390,12 +451,12 @@ func unmarshalAndValidateIdentityDocument(data []byte, getAWSCACertificate func(
 	switch publicKeyType {
 	case RSA1024:
 		if err := verifyRSASignature(caCert.PublicKey.(*rsa.PublicKey), attestationData.Document, signature); err != nil {
-			return imds.InstanceIdentityDocument{}, status.Errorf(codes.InvalidArgument, err.Error())
+			return imds.InstanceIdentityDocument{}, status.Error(codes.InvalidArgument, err.Error())
 		}
 	case RSA2048:
 		pkcs7Sig, err := decodeAndParsePKCS7Signature(signature, caCert)
 		if err != nil {
-			return imds.InstanceIdentityDocument{}, status.Errorf(codes.InvalidArgument, err.Error())
+			return imds.InstanceIdentityDocument{}, status.Error(codes.InvalidArgument, err.Error())
 		}
 
 		if err := pkcs7Sig.Verify(); err != nil {
@@ -558,3 +619,38 @@ func isValidAWSPartition(partition string) bool {
 	}
 	return false
 }
+
+func validateOrganizationConfig(config *IIDAttestorConfig) error {
+	checkAccID := config.ValidateOrgAccountID.AccountID
+	checkAccRole := config.ValidateOrgAccountID.AccountRole
+	checkAccRegion := config.ValidateOrgAccountID.AccountRegion
+
+	if checkAccID == "" || checkAccRole == "" {
+		return status.Errorf(codes.InvalidArgument, "please ensure that %q & %q are present inside block or remove the block: %q for feature node attestation using account id verification", orgAccountID, orgAccountRole, "verify_organization")
+	}
+
+	if checkAccRegion == "" {
+		config.ValidateOrgAccountID.AccountRegion = orgDefaultAccRegion
+	}
+
+	// check TTL if specified
+	ttl := orgAccountDefaultListDuration
+	checkTTL := config.ValidateOrgAccountID.AccountListTTL
+	if checkTTL != "" {
+		t, err := time.ParseDuration(checkTTL)
+		if err != nil {
+			return status.Errorf(codes.InvalidArgument, "please ensure that %q if configured, it should be in duration and is suffixed with required 'm' for time duration in minute ex. '5m'. Otherwise, remove the: %q, in the block: %q. Default TTL will be: %q", orgAccountListTTL, orgAccountListTTL, "verify_organization", orgAccountDefaultListTTL)
+		}
+
+		if t.Minutes() < orgAccountMinTTL.Minutes() {
+			return status.Errorf(codes.InvalidArgument, "please ensure that %q if configured, it should be greater than or equal to %q. Otherwise remove the: %q, in the block: %q. Default TTL will be: %q", orgAccountListTTL, orgAccountMinListTTL, orgAccountListTTL, "verify_organization", orgAccountDefaultListTTL)
+		}
+
+		ttl = t
+	}
+
+	// Assign default ttl if ttl doesnt exist.
+	config.ValidateOrgAccountID.AccountListTTL = ttl.String()
+
+	return nil
+}
