@@ -22,6 +22,8 @@ import (
 	ec2types "github.com/aws/aws-sdk-go-v2/service/ec2/types"
 	"github.com/aws/aws-sdk-go-v2/service/iam"
 	iamtypes "github.com/aws/aws-sdk-go-v2/service/iam/types"
+	"github.com/aws/aws-sdk-go-v2/service/organizations"
+	"github.com/aws/aws-sdk-go-v2/service/organizations/types"
 	"github.com/fullsailor/pkcs7"
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
@@ -54,6 +56,7 @@ var (
 	testAvailabilityZone = "test-az"
 	testImageID          = "test-image-id"
 	testProfile          = "test-profile"
+	testAccountID        = "123456789"
 	zeroDeviceIndex      = int32(0)
 	nonzeroDeviceIndex   = int32(1)
 	instanceStoreType    = ec2types.DeviceTypeInstanceStore
@@ -78,6 +81,8 @@ func TestAttest(t *testing.T) {
 		describeInstancesError         error
 		mutateGetInstanceProfileOutput func(output *iam.GetInstanceProfileOutput)
 		getInstanceProfileError        error
+		mutateListAccountOutput        func(output *organizations.ListAccountsOutput)
+		listOrgAccountError            error
 		overrideAttestationData        func(caws.IIDAttestationData) caws.IIDAttestationData
 		overridePayload                func() []byte
 		expectCode                     codes.Code
@@ -216,7 +221,28 @@ func TestAttest(t *testing.T) {
 				output.Reservations[0].Instances[0].NetworkInterfaces[0].Attachment.DeviceIndex = &nonzeroDeviceIndex
 			},
 			expectCode:      codes.Internal,
-			expectMsgPrefix: "nodeattestor(aws_iid): failed aws ec2 attestation: the EC2 instance network interface attachment device index must be zero (has 1)",
+			expectMsgPrefix: "nodeattestor(aws_iid): failed aws ec2 attestation: the EC2 instance network interface with device index 0 is inaccessible",
+		},
+		{
+			name: "block device anti-tampering check succeeds when network devices are not ordered by device index",
+			mutateDescribeInstancesOutput: func(output *ec2.DescribeInstancesOutput) {
+				output.Reservations[0].Instances[0].NetworkInterfaces[0].Attachment.DeviceIndex = &nonzeroDeviceIndex
+				output.Reservations[0].Instances[0].NetworkInterfaces = append(
+					output.Reservations[0].Instances[0].NetworkInterfaces,
+					ec2types.InstanceNetworkInterface{
+						Attachment: &ec2types.InstanceNetworkInterfaceAttachment{
+							DeviceIndex: &zeroDeviceIndex,
+						},
+					},
+				)
+			},
+			expectID: "spiffe://example.org/spire/agent/aws_iid/test-account/test-region/test-instance",
+			expectSelectors: []*common.Selector{
+				{Type: caws.PluginName, Value: "az:test-az"},
+				{Type: caws.PluginName, Value: "image:id:test-image-id"},
+				{Type: caws.PluginName, Value: "instance:id:test-instance"},
+				{Type: caws.PluginName, Value: "region:test-region"},
+			},
 		},
 		{
 			name: "block device anti-tampering check fails to locate root device",
@@ -390,6 +416,66 @@ func TestAttest(t *testing.T) {
 				{Type: caws.PluginName, Value: "tag:Hostname:host1"},
 			},
 		},
+		{
+			name:            "fail with account id not belonging to organization", // Default attestation data already has different account id
+			config:          `verify_organization = { management_account_id = "12345" assume_org_role = "test-orgrole" management_account_region = "test-region"}`,
+			expectCode:      codes.Internal,
+			expectMsgPrefix: fmt.Sprintf("nodeattestor(aws_iid): failed aws ec2 attestation, nodes account id: %v is not part of configured organization or doesn't have ACTIVE status", testAccount),
+		},
+		{
+			name:                "fail call for organization list account",
+			config:              `verify_organization = { management_account_id = "12345" assume_org_role = "test-orgrole" management_account_region = "test-region"}`,
+			expectCode:          codes.Internal,
+			listOrgAccountError: errors.New("oh no"),
+			expectMsgPrefix:     fmt.Sprintf("nodeattestor(aws_iid): failed aws ec2 attestation, issue while verifying if nodes account id: %v belong to org: %v", testAccount, "issue while getting list of accounts"),
+		},
+		{
+			name:       "fail for account id with not ACTIVE status in organization list",
+			config:     `verify_organization = { management_account_id = "12345" assume_org_role = "test-orgrole" management_account_region = "test-orgregion" }`,
+			expectCode: codes.Internal,
+			mutateListAccountOutput: func(output *organizations.ListAccountsOutput) {
+				output.Accounts = []types.Account{{
+					Id:     &testAccountID,
+					Status: types.AccountStatusSuspended,
+				}}
+			},
+			overrideAttestationData: func(id caws.IIDAttestationData) caws.IIDAttestationData {
+				doc := imds.InstanceIdentityDocument{
+					AccountID:        testAccountID,
+					InstanceID:       testInstance,
+					Region:           testRegion,
+					AvailabilityZone: testAvailabilityZone,
+					ImageID:          testImageID,
+				}
+				docBytes, _ := json.Marshal(doc)
+				id.Document = string(docBytes)
+				return id
+			},
+			expectMsgPrefix: fmt.Sprintf("nodeattestor(aws_iid): failed aws ec2 attestation, nodes account id: %v is not part of configured organization or doesn't have ACTIVE status", testAccountID),
+		},
+		{
+			name:   "success when organization validation feature is turned on",
+			config: `verify_organization = { management_account_id = "12345" assume_org_role = "test-orgrole" management_account_region = "test-orgregion" }`,
+			overrideAttestationData: func(id caws.IIDAttestationData) caws.IIDAttestationData {
+				doc := imds.InstanceIdentityDocument{
+					AccountID:        testAccountID,
+					InstanceID:       testInstance,
+					Region:           testRegion,
+					AvailabilityZone: testAvailabilityZone,
+					ImageID:          testImageID,
+				}
+				docBytes, _ := json.Marshal(doc)
+				id.Document = string(docBytes)
+				return id
+			},
+			expectID: "spiffe://example.org/spire/agent/aws_iid/123456789/test-region/test-instance",
+			expectSelectors: []*common.Selector{
+				{Type: caws.PluginName, Value: "az:test-az"},
+				{Type: caws.PluginName, Value: "image:id:test-image-id"},
+				{Type: caws.PluginName, Value: "instance:id:test-instance"},
+				{Type: caws.PluginName, Value: "region:test-region"},
+			},
+		},
 	} {
 		t.Run(tt.name, func(t *testing.T) {
 			client := newFakeClient()
@@ -401,6 +487,10 @@ func TestAttest(t *testing.T) {
 			if tt.mutateGetInstanceProfileOutput != nil {
 				tt.mutateGetInstanceProfileOutput(client.GetInstanceProfileOutput)
 			}
+			client.ListAccountError = tt.listOrgAccountError
+			if tt.mutateListAccountOutput != nil {
+				tt.mutateListAccountOutput(client.ListAccountOutput)
+			}
 
 			agentStore := fakeagentstore.New()
 			if tt.alreadyAttested {
@@ -435,7 +525,7 @@ func TestAttest(t *testing.T) {
 				return testAWSCACert, nil
 			}
 
-			attestor.clients = newClientsCache(func(ctx context.Context, config *SessionConfig, region string, assumeRoleARN string) (Client, error) {
+			attestor.clients = newClientsCache(func(ctx context.Context, config *SessionConfig, region string, assumeRoleARN string, orgRoleArn string) (Client, error) {
 				return client, nil
 			})
 
@@ -492,7 +582,7 @@ func TestConfigure(t *testing.T) {
 
 	t.Run("missing trust domain", func(t *testing.T) {
 		err := doConfig(t, catalog.CoreConfig{}, ``)
-		spiretest.RequireGRPCStatusContains(t, err, codes.InvalidArgument, "core configuration has invalid trust domain: trust domain is missing")
+		spiretest.RequireGRPCStatusContains(t, err, codes.InvalidArgument, "server core configuration must contain trust_domain")
 	})
 
 	t.Run("fails with access id but no secret", func(t *testing.T) {
@@ -545,6 +635,40 @@ func TestConfigure(t *testing.T) {
 		err := doConfig(t, coreConfig, ``)
 		require.NoError(t, err)
 	})
+
+	orgVerificationFeatureErr := fmt.Errorf("make %v, %v & %v are present inside block : %v for feature node attestation using account id verification", "verify_organization", orgAccountID, orgAccountRole, orgAccRegion)
+	orgVerificationFeatureTTLErr := fmt.Errorf("make %v if configured, should be in hours and is suffix with required `h` for time duration in hour ex. 1h. or remove the : %v, in the block : %v. Default TTL will be : %v,  for feature node attestation using account id verification", orgAccountListTTL, orgAccountListTTL, "verify_organization", orgAccountDefaultListTTL)
+	orgVerificationFeatureMinTTLErr := fmt.Errorf("make %v if configured, should be more than >= %v. or remove the : %v, in the block : %v. Default TTL will be : %v,  for feature node attestation using account id verification", orgAccountListTTL, orgAccountMinListTTL, orgAccountListTTL, "verify_organization", orgAccountDefaultListTTL)
+
+	t.Run("fail, account belongs to org, if params are not specified and feature enabled", func(t *testing.T) {
+		err := doConfig(t, coreConfig, `verify_organization = {}`)
+		require.Error(t, err, orgVerificationFeatureErr)
+	})
+
+	t.Run("fail, account belongs to org, if only account id is specified, roles & region are not specified", func(t *testing.T) {
+		err := doConfig(t, coreConfig, `verify_organization = { management_account_id = "dummy_account" }`)
+		require.Error(t, err, orgVerificationFeatureErr)
+	})
+
+	t.Run("fail, account belongs to org, if ttl is not specified in proper format", func(t *testing.T) {
+		err := doConfig(t, coreConfig, `verify_organization = { management_account_id = "dummy_account" assume_org_role = "dummy_role" org_account_map_ttl = "2" }`)
+		require.Error(t, err, orgVerificationFeatureTTLErr)
+	})
+
+	t.Run("fail, account belongs to org, if ttl is specified and is less than min ttl required", func(t *testing.T) {
+		err := doConfig(t, coreConfig, `verify_organization = { management_account_id = "dummy_account" assume_org_role = "dummy_role" org_account_map_ttl = "30s" }`)
+		require.Error(t, err, orgVerificationFeatureMinTTLErr)
+	})
+
+	t.Run("success, verify_organization featured enabled with required params", func(t *testing.T) {
+		err := doConfig(t, coreConfig, `verify_organization = { management_account_id = "dummy_account" assume_org_role = "dummy_role" }`)
+		require.NoError(t, err)
+	})
+
+	t.Run("success, verify_organization featured enabled with all params", func(t *testing.T) {
+		err := doConfig(t, coreConfig, `verify_organization = { management_account_id = "dummy_account" assume_org_role = "dummy_role" org_account_map_ttl = "1m30s" }`)
+		require.NoError(t, err)
+	})
 }
 
 func TestInstanceProfileArnParsing(t *testing.T) {
@@ -572,6 +696,8 @@ type fakeClient struct {
 	DescribeInstancesError   error
 	GetInstanceProfileOutput *iam.GetInstanceProfileOutput
 	GetInstanceProfileError  error
+	ListAccountOutput        *organizations.ListAccountsOutput
+	ListAccountError         error
 }
 
 func newFakeClient() *fakeClient {
@@ -595,6 +721,7 @@ func newFakeClient() *fakeClient {
 			},
 		},
 		GetInstanceProfileOutput: &iam.GetInstanceProfileOutput{},
+		ListAccountOutput:        &organizations.ListAccountsOutput{},
 	}
 }
 
@@ -619,6 +746,22 @@ func (c *fakeClient) GetInstanceProfile(_ context.Context, input *iam.GetInstanc
 	return c.GetInstanceProfileOutput, c.GetInstanceProfileError
 }
 
+func (c *fakeClient) ListAccounts(_ context.Context, input *organizations.ListAccountsInput, _ ...func(*organizations.Options)) (*organizations.ListAccountsOutput, error) {
+	// Only modify the output if it's not being mutated in test for : mutateListAccountOutput.
+	if c.ListAccountOutput.Accounts == nil {
+		c.ListAccountOutput = &organizations.ListAccountsOutput{
+			Accounts: []types.Account{{
+				Id:     &testAccountID,
+				Status: types.AccountStatusActive,
+			}},
+		}
+	}
+	if input.NextToken != nil {
+		return nil, fmt.Errorf("failing request for pagination")
+	}
+	return c.ListAccountOutput, c.ListAccountError
+}
+
 func buildAttestationDataRSA2048Signature(t *testing.T) caws.IIDAttestationData {
 	// doc body
 	doc := imds.InstanceIdentityDocument{
