@@ -17,8 +17,8 @@ import (
 	"testing"
 	"time"
 
-	jose "github.com/go-jose/go-jose/v3"
-	"github.com/go-jose/go-jose/v3/jwt"
+	jose "github.com/go-jose/go-jose/v4"
+	"github.com/go-jose/go-jose/v4/jwt"
 	"github.com/google/go-cmp/cmp"
 	"github.com/spiffe/go-spiffe/v2/spiffeid"
 	"github.com/spiffe/spire/pkg/common/catalog"
@@ -368,15 +368,15 @@ func (s *AttestorSuite) TestConfigure() {
 
 	// malformed configuration
 	err := doConfig(coreConfig, "blah")
-	s.RequireGRPCStatusContains(err, codes.InvalidArgument, "unable to decode configuration")
+	s.RequireGRPCStatusContains(err, codes.InvalidArgument, "plugin configuration is malformed")
 
 	// missing trust domain
 	err = doConfig(catalog.CoreConfig{}, "")
-	s.RequireGRPCStatus(err, codes.InvalidArgument, "core configuration missing trust domain")
+	s.RequireGRPCStatus(err, codes.InvalidArgument, "server core configuration must contain trust_domain")
 
 	// missing clusters
 	err = doConfig(coreConfig, "")
-	s.RequireGRPCStatus(err, codes.InvalidArgument, "configuration must have at least one cluster")
+	s.Require().NoError(err)
 
 	// cluster missing service account allow list
 	err = doConfig(coreConfig, `clusters = {
@@ -410,7 +410,7 @@ func (s *AttestorSuite) signToken(signer jose.Signer, tokenData *TokenData) stri
 	builder := jwt.Signed(signer)
 	builder = builder.Claims(claims)
 
-	token, err := builder.CompactSerialize()
+	token, err := builder.Serialize()
 	s.Require().NoError(err)
 	return token
 }
@@ -473,7 +473,7 @@ func createAndWriteSelfSignedCert(cn string, signer crypto.Signer, path string)
 	if err != nil {
 		return err
 	}
-	return os.WriteFile(path, pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: certDER}), 0600)
+	return os.WriteFile(path, pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: certDER}), 0o600)
 }
 
 func createTokenStatus(tokenData *TokenData, authenticated bool, audience []string) *authv1.TokenReviewStatus {
