@@ -521,7 +521,7 @@ func TestConfigureTLSWithTokenAuth(t *testing.T) {
 
 	testPool, err := testRootCAs()
 	require.NoError(t, err)
-	require.Equal(t, testPool.Subjects(), tcc.RootCAs.Subjects()) //nolint: staticcheck // these pools are not system pools so the use of Subjects() is ok for now
+	require.Equal(t, testPool.Subjects(), tcc.RootCAs.Subjects())
 }
 
 func TestConfigureTLSWithAppRoleAuth(t *testing.T) {
@@ -543,7 +543,7 @@ func TestConfigureTLSWithAppRoleAuth(t *testing.T) {
 
 	testPool, err := testRootCAs()
 	require.NoError(t, err)
-	require.Equal(t, testPool.Subjects(), tcc.RootCAs.Subjects()) //nolint: staticcheck // these pools are not system pools so the use of Subjects() is ok for now
+	require.Equal(t, testPool.Subjects(), tcc.RootCAs.Subjects())
 }
 
 func TestConfigureTLSInvalidCACert(t *testing.T) {
