@@ -21,6 +21,7 @@ import (
 	"github.com/spiffe/spire/pkg/common/telemetry"
 	"github.com/spiffe/spire/pkg/common/uptime"
 	"github.com/spiffe/spire/pkg/common/util"
+	"github.com/spiffe/spire/pkg/common/version"
 	"github.com/spiffe/spire/pkg/server/authpolicy"
 	bundle_client "github.com/spiffe/spire/pkg/server/bundle/client"
 	ds_pubmanager "github.com/spiffe/spire/pkg/server/bundle/datastore"
@@ -71,8 +72,10 @@ func (s *Server) Run(ctx context.Context) error {
 func (s *Server) run(ctx context.Context) (err error) {
 	// Log configuration values that are useful for debugging
 	s.config.Log.WithFields(logrus.Fields{
-		telemetry.AdminIDs: s.config.AdminIDs,
-		telemetry.DataDir:  s.config.DataDir,
+		telemetry.AdminIDs:       s.config.AdminIDs,
+		telemetry.DataDir:        s.config.DataDir,
+		telemetry.LaunchLogLevel: s.config.Log.GetLevel(),
+		telemetry.Version:        version.Version(),
 	}).Info("Configured")
 
 	// create the data directory if needed
@@ -207,6 +210,7 @@ func (s *Server) run(ctx context.Context) (err error) {
 		bundleManager.Run,
 		registrationManager.Run,
 		bundlePublishingManager.Run,
+		catalog.ReconfigureTask(s.config.Log.WithField(telemetry.SubsystemName, "reconfigurer"), cat),
 		util.SerialRun(s.waitForTestDial, healthChecker.ListenAndServe),
 	}
 
@@ -288,21 +292,21 @@ func (s *Server) loadCatalog(ctx context.Context, metrics telemetry.Metrics, ide
 		IdentityProvider: identityProvider,
 		AgentStore:       agentStore,
 		HealthChecker:    healthChecker,
-		EventsBasedCache: s.config.EventsBasedCache,
 	})
 }
 
 func (s *Server) newCredBuilder(cat catalog.Catalog) (*credtemplate.Builder, error) {
 	return credtemplate.NewBuilder(credtemplate.Config{
-		TrustDomain:            s.config.TrustDomain,
-		X509CASubject:          s.config.CASubject,
-		X509CATTL:              s.config.CATTL,
-		AgentSVIDTTL:           s.config.AgentTTL,
-		X509SVIDTTL:            s.config.X509SVIDTTL,
-		JWTSVIDTTL:             s.config.JWTSVIDTTL,
-		JWTIssuer:              s.config.JWTIssuer,
-		ExcludeSNFromCASubject: s.config.ExcludeSNFromCASubject,
-		CredentialComposers:    cat.GetCredentialComposers(),
+		TrustDomain:                  s.config.TrustDomain,
+		X509CASubject:                s.config.CASubject,
+		X509CATTL:                    s.config.CATTL,
+		AgentSVIDTTL:                 s.config.AgentTTL,
+		X509SVIDTTL:                  s.config.X509SVIDTTL,
+		JWTSVIDTTL:                   s.config.JWTSVIDTTL,
+		JWTIssuer:                    s.config.JWTIssuer,
+		CredentialComposers:          cat.GetCredentialComposers(),
+		UseLegacyDownstreamX509CATTL: s.config.UseLegacyDownstreamX509CATTL,
+		TLSPolicy:                    s.config.TLSPolicy,
 	})
 }
 
@@ -378,32 +382,36 @@ func (s *Server) newSVIDRotator(ctx context.Context, serverCA ca.ServerCA, metri
 	return svidRotator, nil
 }
 
-func (s *Server) newEndpointsServer(ctx context.Context, catalog catalog.Catalog, svidObserver svid.Observer, serverCA ca.ServerCA, metrics telemetry.Metrics, jwtKeyPublisher manager.JwtKeyPublisher, authPolicyEngine *authpolicy.Engine, bundleManager *bundle_client.Manager) (endpoints.Server, error) {
+func (s *Server) newEndpointsServer(ctx context.Context, catalog catalog.Catalog, svidObserver svid.Observer, serverCA ca.ServerCA, metrics telemetry.Metrics, authorityManager manager.AuthorityManager, authPolicyEngine *authpolicy.Engine, bundleManager *bundle_client.Manager) (endpoints.Server, error) {
 	config := endpoints.Config{
-		TCPAddr:              s.config.BindAddress,
-		LocalAddr:            s.config.BindLocalAddress,
-		SVIDObserver:         svidObserver,
-		TrustDomain:          s.config.TrustDomain,
-		Catalog:              catalog,
-		ServerCA:             serverCA,
-		Log:                  s.config.Log.WithField(telemetry.SubsystemName, telemetry.Endpoints),
-		Metrics:              metrics,
-		JWTKeyPublisher:      jwtKeyPublisher,
-		RateLimit:            s.config.RateLimit,
-		Uptime:               uptime.Uptime,
-		Clock:                clock.New(),
-		CacheReloadInterval:  s.config.CacheReloadInterval,
-		EventsBasedCache:     s.config.EventsBasedCache,
-		PruneEventsOlderThan: s.config.PruneEventsOlderThan,
-		AuditLogEnabled:      s.config.AuditLogEnabled,
-		AuthPolicyEngine:     authPolicyEngine,
-		BundleManager:        bundleManager,
-		AdminIDs:             s.config.AdminIDs,
+		TCPAddr:                      s.config.BindAddress,
+		LocalAddr:                    s.config.BindLocalAddress,
+		SVIDObserver:                 svidObserver,
+		TrustDomain:                  s.config.TrustDomain,
+		Catalog:                      catalog,
+		ServerCA:                     serverCA,
+		Log:                          s.config.Log.WithField(telemetry.SubsystemName, telemetry.Endpoints),
+		RootLog:                      s.config.Log,
+		Metrics:                      metrics,
+		AuthorityManager:             authorityManager,
+		RateLimit:                    s.config.RateLimit,
+		Uptime:                       uptime.Uptime,
+		Clock:                        clock.New(),
+		CacheReloadInterval:          s.config.CacheReloadInterval,
+		EventsBasedCache:             s.config.EventsBasedCache,
+		PruneEventsOlderThan:         s.config.PruneEventsOlderThan,
+		SQLTransactionTimeout:        s.config.SQLTransactionTimeout,
+		AuditLogEnabled:              s.config.AuditLogEnabled,
+		AuthPolicyEngine:             authPolicyEngine,
+		BundleManager:                bundleManager,
+		AdminIDs:                     s.config.AdminIDs,
+		UseLegacyDownstreamX509CATTL: s.config.UseLegacyDownstreamX509CATTL,
 	}
 	if s.config.Federation.BundleEndpoint != nil {
 		config.BundleEndpoint.Address = s.config.Federation.BundleEndpoint.Address
 		config.BundleEndpoint.RefreshHint = s.config.Federation.BundleEndpoint.RefreshHint
 		config.BundleEndpoint.ACME = s.config.Federation.BundleEndpoint.ACME
+		config.BundleEndpoint.DiskCertManager = s.config.Federation.BundleEndpoint.DiskCertManager
 	}
 	return endpoints.New(ctx, config)
 }
